package cbsw;

import javax.xml.bind.annotation.*;
import java.awt.GridBagConstraints;
import java.awt.BorderLayout;
import javax.swing.*;
import java.awt.event.*;
import cbsw.*;

/**
<p>A GUI display that tabulates data read through a RegisterIo interface</p>
@author Steve Wotton, Cavendish Lab (HEP), 2018-
*/
@XmlAccessorType(XmlAccessType.NONE)
public class RegisterIoTable
{
    RegisterIo io;
    @XmlAttribute
    String title;
    @XmlAttribute
    int baseAddress;
    @XmlAttribute
    int csrAddress;
    @XmlAttribute
    int csrSize;
    @XmlAttribute
    int tableDataDisplayWidth;
    String [] tableColumnNames = {"Addr","Data"};
    Object [][] tableData;
    JScrollPane tableSP;
    JTable table;
    byte [] csrCache;
    JButton decodeButton;

/**
<p>No-argument constructor used by unmarshaller</p>
 */
  public RegisterIoTable(){}

/**
<p>Returns the table title</p>
@return The title
 */
  public String getTitle(){ return title; }

/**
<p>Called after unmarshalling to complete initialisation</p>
@param io The hardware I/O interface
 */
  public void init(RegisterIo io)
  {
    this.io = io;
    csrCache = new byte [csrSize];
  }

/**
<p>Replaced by {@link getTable()}</p>
@return The table
 */
@Deprecated
  public JTable getStatusTable(){ return table; }
    
/**
<p>Returns the table</p>
@return The table
 */
  public JTable getTable(){ return table; }

/**
<p>Creates a panel containing the table</p>
@return The panel
 */
    public JPanel makePanel()
    {
        tableData = new Object[(csrSize+tableDataDisplayWidth-1)/tableDataDisplayWidth][];

// Initialise the register table

        for ( int reg=0;reg<csrSize;reg+=tableDataDisplayWidth )
        {
	  int row = reg/tableDataDisplayWidth;
	  tableData[row] = new Object [2];
	  tableData[row][0] = Integer.valueOf(reg);
	  tableData[row][1] = "";
	}

// Table of register values

        table = new JTable(tableData,tableColumnNames);
        table.setFont(new java.awt.Font("Monospaced",java.awt.Font.PLAIN,11));
        table.setPreferredScrollableViewportSize(new java.awt.Dimension(40+tableDataDisplayWidth*16, 200));
        table.getColumnModel().getColumn(0).setPreferredWidth(40);
        table.getColumnModel().getColumn(1).setPreferredWidth(tableDataDisplayWidth*16);

        JScrollPane jSP = new JScrollPane(table);
        JPanel jP = new JPanel();
	jP.setLayout(new BorderLayout());
        jP.add(jSP,BorderLayout.NORTH);
        jP.setBorder( BorderFactory.createTitledBorder(title) );

	decodeButton = new JButton("Decode");
        jP.add(decodeButton,BorderLayout.SOUTH);
	
        javax.swing.Timer timer = new javax.swing.Timer( 1000, l->{cache(); updateTable();} );
        timer.setInitialDelay(1000);
        timer.setCoalesce( false );
        timer.start();

	return jP;
     }

/**
<p>Add a decode ActionListener</p>
<p>The ActionListener is called when the decode button is pressed.</p> 
@param al The ActionListener
*/
public void addDecodeAction(ActionListener al){ decodeButton.addActionListener(al);}
    
/**
<p>Writes cached values to hardware</p>
@param regs Data to write
*/
public void writeArray( byte [] regs )
{
  try
  { 
    io.put( baseAddress, csrAddress, regs );
  } catch (RegisterIoException de) {}
  return;
}

/**
<p>Reads the register array from hardware into local cache.</p>
 */
public void cache()
{
  try
  {
    io.get( baseAddress, csrAddress, csrCache );
  } catch (Exception de) {}
  return;
}

/**
<p>Returns the cache.</p>
@return The data cache
 */
public byte [] getCache(){return csrCache;}

/**
<p>Replaced by {@link updateTable()}</p>
*/
@Deprecated
public void updateStatusTable(){ updateTable(); }

/**
<p>Update table from previously cached data</p>
*/
public void updateTable()
{
  for ( int reg=0;reg<csrSize;reg+=tableDataDisplayWidth )
  {
    int row = reg/tableDataDisplayWidth;
    String sreg = "";
    for (int n=0;n<tableDataDisplayWidth;n++)
    {
      sreg = String.format("%1$02x",csrCache[reg+n]) + sreg;
    }
    table.setValueAt(sreg,row,1);
  }
}
    
/**
<p>Creates a panel containing all the unmarshalled tables</p>
@param tableList The list of tables
@return The panel
 */
static public JPanel makePanel(java.util.ArrayList<RegisterIoTable> tableList)
{
  JPanel dtjP = new JPanel();
    
  dtjP.setLayout(new java.awt.GridBagLayout());
  GridBagConstraints gbC = new GridBagConstraints();
  gbC.fill = GridBagConstraints.VERTICAL;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new java.awt.Insets(1,1,1,1);
  gbC.weightx = 1.0f;
  gbC.weighty = 1.0f;
  gbC.fill = GridBagConstraints.BOTH;

  for (RegisterIoTable dT : tableList )
  {
    JPanel jP = dT.makePanel();
    gbC.gridwidth = 1;
    dtjP.add( jP, gbC );
  }
  return dtjP;
}

}
