package cbsw;

import javax.xml.bind.annotation.*;

@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="sequence")
public class Sequence
{
@XmlElement(name="command")
java.util.ArrayList<Command> commandList;

java.util.Iterator<Command> it;
boolean suspend=false;

static javax.swing.JTextArea textArea;
javax.swing.JButton recordingButton;
    
boolean recording = false;

public Sequence(){}

public javax.swing.JPanel makePanel()
{
    javax.swing.JPanel jP = new javax.swing.JPanel();
    jP.setLayout(new java.awt.GridBagLayout());
    java.awt.GridBagConstraints gbC = new java.awt.GridBagConstraints();
    gbC.fill = java.awt.GridBagConstraints.BOTH;
    gbC.anchor = java.awt.GridBagConstraints.CENTER;
    gbC.insets = new java.awt.Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    
    for (Command cmd : commandList )
    {
      javax.swing.JButton button = new javax.swing.JButton(cmd.label);
      button.addActionListener(l->{try {cmd.go();}catch (Exception e) {textArea.append("Sequence: "+e.getMessage()+"\n");}});
      gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
      jP.add(button,gbC);

      cmd.setButton(button);
    }

    javax.swing.JPanel playJP = new javax.swing.JPanel();
    
    javax.swing.JButton button = new javax.swing.JButton("Play");
    button.addActionListener(l->{prepare();resume();});
    playJP.add(button);

    button = new javax.swing.JButton("Pause");
    button.addActionListener(l->pause());
    playJP.add(button);

    button = new javax.swing.JButton("Resume");
    button.addActionListener(l->resume());
    playJP.add(button);

    recordingButton = new javax.swing.JButton("Record");
    recordingButton.addActionListener(l->recording());
    playJP.add(recordingButton);

    button = new javax.swing.JButton("First step");
    button.addActionListener(l->{prepare();step();});
    playJP.add(button);
    
    button = new javax.swing.JButton("Step");
    button.addActionListener(l->step());
    playJP.add(button);

    // button = new javax.swing.JButton("Flush RX pipe");
    // button.addActionListener(l->readFlush());
    // playJP.add(button);

    gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    jP.add(playJP,gbC);

    return jP;
}

public void setTextArea(javax.swing.JTextArea textArea){this.textArea = textArea;}
	
private void prepare(){ it = commandList.iterator(); }

private void pause(){ suspend=true; }

private void resume()
{
  suspend=false;
  TaskWorker tt = new TaskWorker();
  tt.execute();
}

private void recording()
{
  recording=!recording;
  if (recording) recordingButton.setBackground(java.awt.Color.RED);
  else recordingButton.setBackground(new java.awt.Color(220,220,220));
}
    
private void step()
{
  if (it==null) it = commandList.iterator();
    
  try { if (it.hasNext()) it.next().go(); }
  catch (Exception ex) {textArea.append("Sequence.step(): "+ex+"\n");}
}
    
public void go()
{
  try { for (Command cmd : commandList ) cmd.go(); }
  catch (Exception ex) {textArea.append("Sequence.go(): "+ex+"\n");}
}
	
public void launch()
{
  TaskWorker tt = new TaskWorker();
  tt.execute();
}
    
class TaskWorker extends javax.swing.SwingWorker<Boolean, Integer>
{

  @Override
  protected Boolean doInBackground()
  {
    boolean pass = true;
    try
    {
      while(!suspend)
      {
	textArea.setText("");
        while(it.hasNext())
        {
          it.next().go();
          if (suspend) break;
        }
        it = commandList.iterator();
      }
    }
    catch (Exception ex) {textArea.append("Sequence.TaskWorker.doInBackground(): "+ex+"\n");}
    return Boolean.valueOf(pass);
  }
      
  @Override
  protected void done(){ try {  } catch (Exception e){} }

  @Override
  protected void process( java.util.List<Integer> chunks ){}
}
    
}
