/*
Unmarshall register descriptors into members of this Csr class.
*/
package cbsw;

import java.nio.file.Paths;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.net.URL;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.*;
import cbsw.*;

public class Cbsw
{
  public Cbsw(){}

  public static Object unmarshal(String xml,Class... classes)
  {
    Object obj = null;
      
    try
    {
      URL url = new URL(xml);
      String [] pathTokens = url.getPath().split("/");
      String fileName = pathTokens[pathTokens.length-1];

// Look for file in current directory and use that otherwise copy from URL.
      File file = new File(fileName);
      if (!file.canRead())
      {
        InputStream is = url.openStream();
        Files.copy(is,Paths.get("www-"+fileName),StandardCopyOption.REPLACE_EXISTING);
        file = new File("www-"+fileName);
      }

      JAXBContext jaxbContext = JAXBContext.newInstance(classes);
      Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
      jaxbUnmarshaller.setEventHandler(new javax.xml.bind.helpers.DefaultValidationEventHandler());
      System.out.println("Unmarshalling configuration from "+file.getAbsolutePath());
      obj = jaxbUnmarshaller.unmarshal(file);
    }
    catch (JAXBException e) { e.printStackTrace(); System.exit(0);  }
    catch ( IOException ioe ){ System.err.println("Cbsw.unmarshal() Warning: " + ioe.getMessage() ); }

    return obj;
  }
    
  public static void exec(Class<?> cl, String cmd)
  {
    System.out.println("Cbsw.exec() ["+cmd+"]");
    String [] cmdTokens = cmd.split("\\|"); // Have to escape the vert bar in regex

    try
    {
      if (cmdTokens.length == 1)
      {
        java.lang.reflect.Method method = cl.getMethod(cmdTokens[0]);
        method.invoke(cl);
      }
      else if (cmdTokens.length == 2)
      {
        java.lang.reflect.Method method = cl.getMethod(cmdTokens[0],String.class);
        method.invoke(cl,cmdTokens[1]);
      }
      else if (cmdTokens.length == 3)
      {
        java.lang.reflect.Method method = cl.getMethod(cmdTokens[0],String.class,String.class);
        method.invoke(cl,cmdTokens[1],cmdTokens[2]);
      }
    }
    catch (Exception e) {System.out.println("Cbsw.exec(): "+e.toString()+" for "+cl.toString()+"["+cmd+"]");}
  }

}
