package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.lang.Integer;
import java.lang.Boolean;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Color;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.SwingWorker;
import java.io.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.file.*;
import java.nio.file.attribute.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.gui.*;

/**
<p>This class encapsulates the interactions with the collection of CLAROs.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
class ClaroControl implements ActionListener
{
  Preferences prefs;
  ScaDepp sca;
  ArrayList<Claro> claros;
  public static final int nFebs = 2;
  public static final int nClaros = 8;
  int ec;
  @XmlElement
  int writeEnable;
  @XmlElement
  int feb1Enable;
  @XmlElement
  int feb2Enable;
  @XmlElement
  int npor;
  @XmlElement
  int aldoEnable;
  @XmlElement
  int seud;

  JCheckBox testEnableCB;
  public boolean testEnable = false;
  JComboBox<String> regNameCB;
  JTextField regValueTF;
    
  public ClaroControl()
  {
    prefs = Preferences.userNodeForPackage(ClaroControl.class).node("SCA");
    claros = new ArrayList<Claro>();
  }

  public void actionPerformed( ActionEvent e )
  {
    Object ob = e.getSource();
    String command = e.getActionCommand();

    if ( command.equals("configure") )
    {
      sca.allocateTxRxBuffers(10*claros.size()); // 10 SCA commands per CLARO
      for ( Claro claro : claros ) claro.prepareSpi(true);
      sca.putBytes(0,true);
    }
    else if ( command.equals("verify") )
    {
      JButton button = (JButton)ob;
      button.setBackground(Color.yellow);
      if (verify(sca,claros))
        button.setBackground(Color.green);
      else
        button.setBackground(Color.orange);
    }
  }
    
  public void init(ScaDepp sca, int ec)
  {
    this.sca = sca;
    this.ec = ec;
    
    File cf = new File(prefs.get("claro.filename","claro.spi"));
    for (int ifeb=0;ifeb<nFebs;ifeb++)
    {
      for (int iclaro=0;iclaro<nClaros;iclaro++)
      {
	  int febEnable = (ifeb == 0) ? feb1Enable : feb2Enable;
	  Claro claro = new Claro(); claro.init(sca,ec,ifeb,iclaro,febEnable,writeEnable);
	  claro.set(cf);
	  claros.add(claro);
      }
    }
    
    sca.gpioDirection.set(writeEnable);
    sca.gpioOut.clear(writeEnable);
    sca.gpioDirection.set(feb1Enable);
    sca.gpioOut.clear(feb1Enable);
    sca.gpioDirection.set(feb2Enable);
    sca.gpioOut.clear(feb2Enable);
    sca.gpioDirection.clear(npor);
    sca.gpioOut.clear(npor);
    sca.gpioDirection.set(aldoEnable);
    sca.gpioOut.clear(aldoEnable);
    sca.gpioDirection.clear(seud);
    sca.gpioOut.clear(seud);
  }

  public JPanel makePanel()
  {
    JPanel jp = new JPanel();
    jp.setLayout(new GridBagLayout());
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JButton button = new JButton("CLARO Configure");
    button.addActionListener(this);
    button.setActionCommand("configure");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("CLARO Verify");
    button.addActionListener(this);
    button.setActionCommand("verify");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    testEnableCB = new JCheckBox("Test enable",testEnable);
    testEnableCB.addActionListener(l->testEnable=testEnableCB.isSelected());
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(testEnableCB,gbC);

    regNameCB = new JComboBox<String>();
    regNameCB.setEditable(true);
    regNameCB.addItem("ch.*.threshold");
    regNameCB.addItem("ch.*.attenuation");
    regNameCB.addItem("ch.*.threshold-offset");
    regNameCB.addItem("ch.*.enable");
    regNameCB.addItem("ch.*.hysteresis-enable");
    regNameCB.addItem("ch.*.testpulse-disable");
    regNameCB.addItem("testpulse-enable");
    regNameCB.addItem("aux-testpulse");
    regNameCB.addItem("internal-correction");
    regNameCB.addItem("seu-count-reset");
    regNameCB.addItem("seu-generate-enable");
    regNameCB.addItem("mux-address");
    gbC.gridwidth = 1;
    jp.add(regNameCB,gbC);
    
    regValueTF = new JTextField("0");
    gbC.gridwidth = 1;
    jp.add(regValueTF,gbC);

    button = new JButton("Set");
    button.addActionListener(l->set());
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(button,gbC);
    
    return jp;
  }

public void set(){  for ( Claro claro : claros ) claro.set((String)regNameCB.getSelectedItem(),regValueTF.getText()); }
    
public static String spiTest(String [] args)
{
  ArrayList<Claro> claroList = MuDaq.muDaq.pdmdbMaster.ecList.get(0).claro.claros;
  ScaDepp sca = PdmdbMaster.pdmdbMaster.sca;
  
  PdmdbModuleTester.exec("TEST_SPI|11111111");
  PdmdbModuleTester.rxSeq++;
  PdmdbModuleTester.readResponse(3000);
  try {Thread.sleep(1000);} catch (InterruptedException e){}
  sca.allocateTxRxBuffers(10*claroList.size()); // 10 SCA commands per CLARO
  for ( Claro claro : claroList ) claro.prepareSpi(false);
  sca.putBytes(0,true);
  String [] spiResponse = PdmdbModuleTester.readResponse(15000);
  if (spiResponse.length > 2)
  {
    try
    {
      TesterSequence.testSummaryXml.write("<spi>\n"); TesterSequence.testSummaryXml.flush();
      for ( String ss : spiResponse )
      {
	if ( ss.startsWith("SS") )
	{
	  int index = Integer.parseInt(ss.substring(2,3));
	  String [] ssTokens = ss.split("[:,=]");
	  TesterSequence.testSummaryXml.write("  <ss index=\""+index+"\" toggles=\""+ssTokens[2]+"\" clocks=\""+ssTokens[4]+"\" />\n");
	}
      }
      TesterSequence.testSummaryXml.write("</spi>\n");
      TesterSequence.testSummaryXml.flush();
    }
    catch (IOException ioe){return "FAIL|ClaroControl.spiTest|IO exception";}
  }
  
  return "OK|ClaroControl.spiTest";
}    

  public static String configure(String [] args)
  {
    int ecId = Integer.decode(args[0]).intValue();
    
    ArrayList<Claro> claroList = MuDaq.muDaq.pdmdbMaster.ecList.get(ecId).claro.claros;
    ScaDepp sca = PdmdbMaster.pdmdbMaster.sca;
    
    if (claroList.size()>16) {return "FAIL|ClaroControl.configure|Wrong list size";}
    sca.allocateTxRxBuffers(10*claroList.size()); // 10 SCA commands per CLARO
    for ( Claro claro : claroList ) claro.prepareSpi(true);
    sca.putBytes(0,true);
    boolean cfgOK = ClaroControl.verify(sca, claroList);
    return "OK|ClaroControl.configure|Verify:"+cfgOK;
  }
    
// Write or read CLARO configuration registers for up to 16 CLAROs.
// The FPGA buffer (36kb) is large enough for all the commands needed for both FEBs.

/*
  public static void doSpi(ScaDepp sca, ArrayList<Claro> claroList, boolean wrflag)
  {
    if (claroList.size()>16) {System.err.println("ClaroControl.doSpi(): Greater than 16 CLAROs. Bailing out..."); return;}
    sca.allocateTxRxBuffers(10*claroList.size()); // 10 SCA commands per CLARO
    for ( Claro claro : claroList ) claro.prepareSpi(wrflag);
    sca.putBytes(0,true);
    return;
  }
*/
    
// Deselect all CLARO
// Put chip selects into safe state

  public void deselectAllCLARO()
  {
    sca.allocateTxRxBuffers(2);
    sca.infoTx("GPIO_W_DATAOUT",sca.gpioOut);
    sca.infoTx("SPI_W_SS",0x00000000); // Deselect all CLARO
    sca.putBytes(0,true);
  }

// Read CLARO configuration registers and compare with file.

  public static boolean verify(ScaDepp sca, ArrayList<Claro> claroList)
  {
    try {Thread.sleep(10);}catch(InterruptedException ex){System.err.println("ClaroControl.verify(): sleep interrupted");};
// Do the SPI read
    sca.allocateTxRxBuffers(10*claroList.size()); // 10 SCA commands per CLARO
    for ( Claro claro : claroList ) claro.prepareSpi(false);
    sca.putBytes(0,true);

// Now check the received data agrees with the file data.

    try {Thread.sleep(10);}catch(InterruptedException ex){System.err.println("ClaroControl.verify(): sleep interrupted");};

// Now decode the SCA responses to extract the GBTX register data

    byte [] hdlcrx = new byte [10*16*claroList.size()];
    sca.getBytes( 0, hdlcrx);
    
    int irx = 0;
    boolean pass = true;
    for (Claro claro: claroList)
    {
      int iclaro = 8*claro.feb + claro.claro;
      byte [] rxbuf = new byte [16];
      byte [] claroBuf = claro.toByteArray();
      int frame = 10*iclaro + 6; // Skip to the frames containing MISO
      for ( int ifr=0;ifr<4;ifr++,frame++ )
      {
	for (int j=0;j<4;j++,irx++)
	{
	  int idx = sca.getRxIndex(frame,j);
          if (idx<0) continue;
          rxbuf[irx%16] = hdlcrx[frame*16+idx];
	}
      }
      if (!claro.verify(rxbuf))
      {
	System.err.println(String.format("ClaroControl.verify() [EC,FEB,CLARO]=[%d,%d,%d] Verification mismatch",claro.ec,claro.feb,claro.claro));
	pass = false;
      }
    }
    return pass;
  }

}
