/*

A display class that makes a coloured array

Steve Wotton, 2017-
Cavendish Lab (HEP)

*/
package cbsw.gui;

import java.awt.Dimension;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.Container;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import java.awt.image.*;
import java.awt.print.*;
import java.net.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import javax.swing.border.*;
import java.awt.Point;
import java.util.Vector;
import java.util.prefs.*;
import cbsw.gui.*;

public class ColorArray extends JPanel implements MouseMotionListener, ActionListener
{
Container container;
BufferedImage bibg;
Graphics2D bibgg2d;
BufferedImage bifg;
Graphics2D bifgg2d;
Color bgColor = Color.blue;
int pixelHeight, pixelWidth, nxpixels, nypixels;
Color [][] cellColor;

public ColorArray( Container cont, int nx, int ny, int width, int height )
{
   container = cont;
   nxpixels = nx;
   nypixels = ny;
   pixelWidth = width;
   pixelHeight = height;

   cellColor = new Color [nxpixels][nypixels];
   setColor(bgColor);

   setOpaque( true );

   //   setPreferredSize( new Dimension(nx*width+16,ny*height+24) );
   setPreferredSize( new Dimension(nx*width,ny*height) );
   //   setBorder( BorderFactory.createTitledBorder("Image area") );

   bibg = new BufferedImage(nx*pixelWidth,ny*pixelHeight,BufferedImage.TYPE_INT_RGB);
   bibgg2d = bibg.createGraphics();

   bifg = new BufferedImage(nx*pixelWidth,ny*pixelHeight,BufferedImage.TYPE_INT_RGB);
   bifgg2d = bifg.createGraphics();

   bifgg2d.setColor(bgColor);
   bifgg2d.fillRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight);

   addMouseMotionListener( this );

}

// MouseMotionListener implementation

public void mouseDragged( MouseEvent e ){ return; }

public void mouseMoved( MouseEvent e )
{
   int x = e.getX();
   int y = e.getY();
   int nx = x/pixelWidth;
   int ny = y/pixelHeight;

   if ( nx < 0 || nx >= nxpixels || ny < 0 || ny >= nypixels ) return;

   //   showStatus( "(x,y)=(" + nx + "," + ny + ") state=" + cellState[nx][ny] );

   return;
}


    public void mouseEntered( MouseEvent e )
    {
	//	showStatus( "Entered x:" + e.getX() + " y:" + e.getY() );
       return;
    }

    public void mouseExited( MouseEvent e )
    {
	//	showStatus( "Exited x:" + e.getX() + " y:" + e.getY() );
       return;
    }

// ActionListener implementation

public void actionPerformed( ActionEvent e )
{
   String cmd = e.getActionCommand();

  return;
}

public void update()
{

   repaint();

   return;
}

public void setColor( Color col, int x, int y ){cellColor[x][y] = col; repaint();}

public void setColor( Color col )
{
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	 cellColor[x][y] = col;
      }
   }

   repaint();
}

public void setColor( Color col[][] )
{
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	 if (col[x][y] != null) cellColor[x][y] = col[x][y];
      }
   }

   repaint();
}

public void paintNonBuffered( Graphics2D g, Paper pa, int ixa, int iya, int nxAnodes, int nyAnodes, boolean printMono )
{

// Save the current transform

   AffineTransform tr = g.getTransform();

// Choose scaling to fit page

   double xscale = pa.getImageableWidth()/(nxAnodes*nxpixels*pixelWidth);
   double yscale = pa.getImageableHeight()/(nyAnodes*nypixels*pixelHeight);
   double scale;

   if ( xscale < yscale )
   {
       scale = xscale;
   }
   else
   {
       scale = yscale;
   }

// Translate and scale to fit page

   g.translate( pa.getImageableX(), pa.getImageableY() );
   g.scale( scale, scale );
   g.translate( (ixa*nxpixels*pixelWidth), (iya*nypixels*pixelHeight) );

// Draw background

   g.clearRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight );

// Paint hit pixels

   if ( printMono )
   {
     g.setColor(Color.white);
     g.fillRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight);
     g.setColor( Color.black );
   }
   else
   {
     g.setColor(bgColor);
     g.fillRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight);
   }
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
        g.setColor(cellColor[x][y]);
	g.fillRect(x*pixelWidth, y*pixelHeight, pixelWidth, pixelHeight);
      }
   }

// Draw a frame around the image

   g.setColor( Color.black );
   g.drawRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight );

// Restore the original transform

   g.setTransform( tr );

   return;
}

public void paintComponent(Graphics g)
{
   super.paintComponent(g); // paints the parent

// Draw background

   bifgg2d.clearRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight );
   bifgg2d.drawImage( bifg, 0, 0, this );

// Paint hit pixels

   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {       
        bifgg2d.setColor(cellColor[x][y]);
	bifgg2d.fillRect(x*pixelWidth, y*pixelHeight, pixelWidth, pixelHeight);
      }
   }

// Draw foreground

   g.drawImage( bifg, 0, 0, this );

   return;
}

}
//














