package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.util.ArrayList;
import java.nio.channels.SocketChannel;
import java.lang.reflect.*;
import cbsw.*;

/**
<p>This class encapsulates a Command execute through reflection.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
public class Command
{
@XmlAttribute
public String label;
@XmlAttribute
public String targetMethod;
@XmlAttribute
public String targetClass;
@XmlAttribute
public boolean stopOnFail;
@XmlAttribute
public boolean pause;
@XmlAttribute
public String data;
@XmlElement(name="response")
ArrayList<TesterResponse> responseList;
    
javax.swing.JTextField statusTF;
javax.swing.JButton button;


public Command(){}

// Use Java reflection to call the static exec method of the target class
// Allows multiple groups of target method args that are applied repeatedly to the same method
public void go() throws Exception
{
  try
  {

    // Split the groups of target method args at the ? character
    
    String [] dataTokenised = (data==null) ? new String []{""} : data.split("\\?");
  
    if (button!=null) button.setBackground(java.awt.Color.YELLOW); // Command send

    for (String targetMethodData : dataTokenised)
    {
	//      System.out.println(targetMethodData);
      if (statusTF!=null && targetClass.equals("PdmdbModuleTester")) statusTF.setText((PdmdbModuleTester.getCmdSeq()+1)+">");

      if (!targetClass.equals("PdmdbModuleTester")) TesterSequence.logMessage(">["+java.time.LocalTime.now().toString()+"] "+targetClass+"."+targetMethod+"("+targetMethodData+")\n");

      String [] targetMethodArgs = targetMethodData.split("\\|");

      String rv = (String)Command.exec(Class.forName("cbsw.lhcb.pdmdb."+targetClass), targetMethod, targetMethodArgs);
      TesterSequence.logMessage("<["+java.time.LocalTime.now().toString()+"] "+rv+"\n");
      String cmdStatus = rv.split("\\|")[0];
      if (!cmdStatus.equals("OK") && (!cmdStatus.equals("FAIL") || stopOnFail)) throw new RuntimeException(rv);

      if (responseList == null) continue; // No registered responses
  
      if (button!=null) button.setBackground(java.awt.Color.ORANGE); // Waiting for reponses.
// Get all responses to this command
      for ( TesterResponse response : responseList )
      {
	if (response.incrementSequenceNumber) PdmdbModuleTester.rxSeq++;
        String [] msgTokens = PdmdbModuleTester.readResponse(response.timeout);
// Check the token array exists. Bail out if null.
        if (msgTokens == null) throw new NullPointerException("readResponse() returned null");
	//	for (String msgToken : msgTokens) TesterSequence.logMessage(" "+msgToken);
// Check the token array has at least 2 elements. Bail out if not.
        if (msgTokens.length < 2) throw new java.util.NoSuchElementException("readResponse() returned insufficient tokens ");

        if (statusTF!=null) statusTF.setText(statusTF.getText()+"|"+msgTokens[0]+">"+msgTokens[1]);
      
// Check for out of sequence response
        if (Integer.parseInt(msgTokens[0]) != PdmdbModuleTester.rxSeq) throw new RuntimeException("Out-of-sequence response to targetMethod "+targetMethod+" "+msgTokens[0]+"|"+msgTokens[1]);
// Check for error acknowledge
        if (msgTokens[1].equals("ACK_ERROR")) throw new RuntimeException("ACK_ERROR received for targetMethod "+targetMethod);
// Check that the response targetMethod is the expected one.
        if (!msgTokens[1].equals(response.opcode)) throw new UnsupportedOperationException("Unexpected response "+msgTokens[1]+"!="+response.opcode);
// Check for ERROR status
	if (msgTokens[msgTokens.length-1].startsWith("TEST_STATUS=ERROR")) throw new RuntimeException("TEST_STATUS=ERROR for targetMethod "+targetMethod);

	if (response.postProcessor != null)
	{
	  String post = (String)Command.exec(Class.forName("cbsw.lhcb.pdmdb.TesterSequence"), response.postProcessor, msgTokens);
          String postStatus = post.split("\\|")[0];
          if (!postStatus.equals("OK") && (!postStatus.equals("FAIL") || stopOnFail)) throw new RuntimeException(rv);
	}
      }
    }
    if (button!=null) button.setBackground(java.awt.Color.GREEN); // All done.
  }
  catch (Exception e) {throw new Exception("Command.go(): "+e.toString()+" for "+targetMethod+"["+data+"]");}
}

public static Object exec(Class<?> targetClass, String targetMethod, String [] targetArgs)
{
    //  System.err.println(String.format("Command.exec(): %s.%s()",targetClass.toString(),targetMethod));
  Object rv = null;
  try
  {
    java.lang.reflect.Method method = targetClass.getMethod(targetMethod,String[].class);
    rv = method.invoke(targetClass,new Object[] {targetArgs});
  }
  catch (Exception e) {System.out.println(String.format("Command.exec(): %s for %s.%s()",e.toString(),targetClass.toString(),targetMethod));}
  return rv;
}
    
static public String shell(String [] args)
{
    // args[0] command filename
    // args[1...] command argument tokens
  String msg = "FAIL|Command.shell";
  java.nio.ByteBuffer bb = java.nio.ByteBuffer.allocate(1024);
  bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);

  try
  {
      Process pr = java.lang.Runtime.getRuntime().exec(args,null,TesterSequence.testDirectory.toFile());
    //    ostream = pr.getOutputStream();
    //    ochannel = Channels.newChannel(pr.getOutputStream());
    java.nio.channels.ReadableByteChannel ichannel = java.nio.channels.Channels.newChannel(pr.getInputStream());
    ichannel.read(bb);
    bb.flip();
    byte [] rdMsg = new byte[bb.limit()];
    bb.get(rdMsg);
    msg = new String(rdMsg);
    //    pr.waitFor(5L,java.util.concurrent.TimeUnit.SECONDS);
  }
  catch (java.io.IOException ioe){ System.err.println("Command.shell(): "+ioe.getMessage()); msg = "FAIL|Command.shell|IOException"; }
  //  catch (InterruptedException inte){ System.err.println("Command.shell(): "+inte.getMessage()); msg = "FAIL|Command.shell|InterruptedException"; }

  return msg;
}

public void setStatusTextField(javax.swing.JTextField statusTF){this.statusTF = statusTF;}
public void setButton(javax.swing.JButton button){this.button = button;}
public void clearStatus()
{
  if (statusTF!=null) statusTF.setText("");
  if (button!=null) button.setBackground(new java.awt.Color(220,220,220));
}
}
