package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.xml.stream.*;
import java.io.*;
import java.nio.file.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.gui.*;

/**
<p>This class encapsulates the interactions with the muDaq K7 dynamic reconfiguration port.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
public class DrpPanel
{
static DrpPanel staticThis;
JniAdeptDepp depp;
@XmlAttribute(name="deppBase")
int deppBase;
@XmlAttribute(name="deppConfigBase")
int deppConfigBase;
JTextField addrTF, dataTF, drpSelectTF, drpPrescaleTF;
ColorArray ca;
Color [] colorMap = {Color.black,Color.blue,Color.cyan,Color.green,
Color.yellow,Color.orange,Color.red,Color.magenta,Color.pink,Color.white};

public DrpPanel(){ staticThis = this; }

public GuiJPanel makePanel()
{
  GuiJPanel jP = new GuiJPanel(Preferences.userNodeForPackage(DrpPanel.class).node("DRP"),"");

  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(1,1,1,1);
  gbC.weightx = 1.0f;
  gbC.weighty = 1.0f;

  gbC.gridwidth = 1;
  jP.add(new JLabel("Address"),gbC);
  addrTF = new JTextField("0000");
  gbC.gridwidth = 1;
  jP.add(addrTF,gbC);

  gbC.gridwidth = 1;
  jP.add(new JLabel("Data"),gbC);
  dataTF = new JTextField("0000");
  gbC.gridwidth = 1;
  jP.add(dataTF,gbC);

  JButton button = new JButton( "Read" );
  button.addActionListener(ae->drpRead());
  gbC.gridwidth = 1;
  jP.add( button,gbC );

  button = new JButton( "Write" );
  button.addActionListener(ae->drpWrite());
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add( button,gbC );

  gbC.gridwidth = 1;
  jP.add(new JLabel("MGT"),gbC);
  drpSelectTF = new JTextField("5");
  gbC.gridwidth = 1;
  jP.add(drpSelectTF,gbC);

  gbC.gridwidth = 1;
  jP.add(new JLabel("Prescale"),gbC);
  drpPrescaleTF = new JTextField("2",2);
  gbC.gridwidth = 1;
  jP.add(drpPrescaleTF,gbC);

  button = new JButton( "EyeScan" );
  button.addActionListener(ae->eyeScan());
  gbC.gridwidth = 1;
  jP.add( button,gbC );

  ca = new ColorArray(jP,32,32,4,4);
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add(ca,gbC);

  return jP;
}

public void init(JniAdeptDepp depp){ this.depp = depp; }

public void drpRead()
{
  try
  {
    int addr = Integer.parseInt(addrTF.getText(),16);
    addr <<= 1; // DEPP registers are 8 bit, DRP are 16
    byte [] buf = new byte [2];
    depp.getBytes( deppBase, addr, buf );
    dataTF.setText(String.format("%1$02x%2$02x",buf[1],buf[0]));
  } catch (DeppException de) {System.err.println("DrpPanel.drpRead(): Depp exception. Code="+de.getDescription());}

   return;
}

public void drpWrite()
{
  try
  {
    int addr = Integer.parseInt(addrTF.getText(),16);
    addr <<= 1; // DEPP registers ar 8 bit, DRP are 16
    int data = Integer.parseInt(dataTF.getText(),16);
    byte [] buf = {(byte)(data&0xff),(byte)((data>>8)&0xff)};
    depp.put1( deppBase, addr, buf );
  } catch (DeppException de) {System.err.println("DrpPanel.drpWrite(): Depp exception. Code="+de.getDescription());}

   return;
}

public void deppPutShort(int address, short val)
{
  byte [] buf = new byte [2];
  buf[0] = (byte)(val&0xff);
  buf[1] = (byte)((val>>8)&0xff);
  try { depp.put1( deppBase, address<<1, buf ); } catch (DeppException de) {System.err.println("DrpPanel.deppPutShort(): Depp exception. Code="+de.getDescription());}
}

public static String doEyeScan(String [] args)
{
  
  int drpsel = Integer.parseInt(args[0]);
  int prescale = Integer.parseInt(args[1]);
  if ( prescale > 31 ) prescale = 31;
  if ( prescale < 0 ) prescale = 0;
  int maxErrors = Integer.parseInt(args[2]);

  int noErrorsCount = 0;

  String rv = "OK|DrpPanel.doEyeScan({"+args[0]+","+args[1]+"})";

  try
  {
    BufferedWriter writer = Files.newBufferedWriter(TesterSequence.eyeScanDataDirectory.resolve(Path.of("eyeScan-"+drpsel+".txt")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);

    writer.write("eye"+drpsel);
    writer.newLine();

    staticThis.eyeScanPrepare(drpsel);

    for (int x=0;x<32;x++)
    {
      for (int y=0;y<32;y++)
      {
        ArrayList<Integer> al = staticThis.eyeScanDo(x,y,prescale);
        if (al.size() != 2) break;

	if (al.get(1) == 0) noErrorsCount++;

        writer.write(String.format("%d,%d,%d,%d\n",x,y,al.get(0),al.get(1))); // x,y,samples,errors
	writer.flush();

      }
    }
    writer.close();
    rv = "OK|DrpPanel.doEyeScan({"+args[0]+","+args[1]+"})|Quality="+noErrorsCount;
    System.err.println("DrpPanel.doEyeScan(): "+noErrorsCount+" samples without errors");

// Create the histograms (get data from the file we've just created)
    BufferedReader reader = Files.newBufferedReader(TesterSequence.eyeScanDataDirectory.resolve(Path.of("eyeScan-"+drpsel+".txt")));
    cbsw.gui.Histogram2D hist = new cbsw.gui.Histogram2D("");
    hist.setPaintScale(-5.5,1.5,1.4);
    try { hist.readStream(reader); } catch(cbsw.gui.Histogram2D.EOFException eof){}
    reader.close();

// Output the plots and dataset statistics
    hist.saveAsSVG(TesterSequence.testDirectory.resolve(Path.of("eye"+drpsel+".svg")).toFile()); // Write as an SVG file
    TesterSequence.testSummaryXml.write("<eye id=\"eye"+drpsel+"\" binsWithoutErrors=\""+noErrorsCount+"\" />\n");
    TesterSequence.testSummaryXml.flush();
  }
  catch (IOException ioe)
  {
    System.err.println("DrpPanel.doEyeScan(): "+ioe.getMessage());
    rv = "FAIL|DrpPanel.doEyeScan({"+args[0]+","+args[1]+"})|IO exception";
  }

  if (noErrorsCount<maxErrors) rv = "FAIL|DrpPanel.doEyeScan({"+args[0]+","+args[1]+"})|Quality="+noErrorsCount;
  return rv;
}

public void eyeScanPrepare(int mgt)
{
  try
  {
    byte [] drpbuf = {0,0,0,0,(byte)(mgt&0x07)};
    depp.put( deppConfigBase, 160, drpbuf );
  } catch (cbsw.RegisterIoException de){System.err.println("DrpPanel.eyeScanPrepare(): Depp exception. Code="+de.getDescription());}
}
    
public ArrayList<Integer> eyeScanDo(int x, int y, int prescale)
{
  ArrayList<Integer> al = new ArrayList<Integer>();
    
  byte [] buf = new byte [2];
  try
  {
    int h = 2 * (x - 16);
    buf[0] = (byte)(h&0xff);
    buf[1] = (byte)((h>>8)&0x0f);
    depp.put1( deppBase, 0x3c<<1, buf );
    
    int v = 8 * (y-16) + 1;
    int vabs = Math.abs(v);
    buf[0] = (byte)(vabs&0x7f); buf[0] = (byte)((v<0) ? buf[0] | 0x80 : buf[0]);
    buf[1] = (byte)(((prescale&0x1f)<<3)|0x01);
    depp.put1( deppBase, 0x3b<<1, buf );
    
// Start the measurement
    depp.getBytes( deppBase, 0x3d<<1, buf );
    buf[0] |= (byte)0x1;
    depp.put1( deppBase, 0x3d<<1, buf );

// Check that the measurement is done
    depp.getBytes( deppBase, 0x151<<1, buf );
    while ((buf[0]&(byte)0x01) != 0x01)
    {
      try { Thread.sleep(20); } catch (InterruptedException inte){}
      depp.getBytes( deppBase, 0x151<<1, buf );
    }

// Read the sample and error counters
    depp.getBytes( deppBase, 0x150<<1, buf ); // Sample count
    int samples = (buf[1]<<8)&0xffff | buf[0]&0xff;
    //    System.out.println(String.format("[%1$03d,%2$03d] Samples = %3$d",h,v,samples));
    samples = samples * (1<<(prescale+1));

    depp.getBytes( deppBase, 0x14f<<1, buf ); // Error count
    int errors = (buf[1]<<8)&0xffff | buf[0]&0xff;
    //    System.out.println(String.format("Errors = %1$d",errors));

// End the measurement
    depp.getBytes( deppBase, 0x3d<<1, buf );
    buf[0] &= (byte)0xfe;
    depp.put1( deppBase, 0x3d<<1, buf );

    al.add(samples);
    al.add(errors);
    
  } catch (DeppException de){System.err.println("DrpPanel.eyeScanDo(): Depp exception. Code="+de.getDescription());}

  return al;
}
    
public void eyeScan()
{
  byte [] buf = new byte [2];
  double alphaMin = 0.0;
  double [][] alpha = new double [32][32];
  Color [][] caColors = new Color [32][32];

  // Loop over H and V offsets

  int prescale = Integer.parseInt(drpPrescaleTF.getText());
  if ( prescale > 31 ) prescale = 31;
  if ( prescale < 0 ) prescale = 0;

  eyeScanPrepare(Integer.parseInt(drpSelectTF.getText()));

  for (int x=0;x<32;x++)
  {
    for (int y=0;y<32;y++)
    {
      ArrayList<Integer> al = eyeScanDo(x,y,prescale);
      if (al.size() != 2) break;

      if ( (al.get(1) == 0xffff) || (al.get(0) == 0) )
      {
        caColors[x][y] = colorMap[colorMap.length-1];
      }
      else if ( al.get(1) == 0 )
      {
        caColors[x][y] = colorMap[0];
      }
      else
      {
        alpha[x][y] = Math.log10((double)al.get(1)/(double)al.get(0));
        if (alpha[x][y]<alphaMin) alphaMin = alpha[x][y];
      }
    }
  }

// The color index is calculated to span the length of the colorMap
// array excluding the first and last colors which flag saturation or
// 0.

  for (int x=0;x<32;x++)
  {
    for (int y=0;y<32;y++)
    {
      if (caColors[x][y] == null)
      {
	long icol = Math.round((1.0 - alpha[x][y]/alphaMin) * (colorMap.length - 2)) + 1;
	if ( icol < 1 ) icol = 1;
	if ( icol > (colorMap.length - 2) ) icol = colorMap.length - 2;
        caColors[x][y] = colorMap[(int)icol];
      }
    }
  }
  ca.setColor(caColors);
  
}

}
