package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.gui.*;

/**
<p>This class encapsulates the interactions with the DTM.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
class DtmControl implements ActionListener
{
  ScaDepp sca;
  @XmlAttribute(name="id")
  int dtm;
  @XmlAttribute(name="i2cBus")
  int i2cBus;
  @XmlElement(name="tap")
  XilS7Control xilS7;
  @XmlElement(name="gbtx")
  List<GbtxControl> gbtxList;
  @XmlElement(name="fpgaI2cSlaveAddress")
  int fpgaI2cSlaveAddress;
  JTextField csrOffsetTF;
  JTextField readValueTF, writeValueTF;

  public DtmControl() {}

  public GuiJPanel makePanel()
  {
    GuiJPanel gjp = new GuiJPanel(Preferences.userNodeForPackage(DtmControl.class).node("JPDMDBDepp"),"");
    gjp.setName( "DTM."+dtm );

    gjp.setLayout(new GridBagLayout());
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JButton button;
    button = new JButton("Hold all reset");
    button.addActionListener(l->holdAllReset());
    button.setActionCommand("holdReset");
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    gjp.add(button,gbC);

    for ( GbtxControl gbtx : gbtxList.toArray(new GbtxControl[0]) )
    {
	gbC.gridwidth = (gbtx.getGbtxId() < 1) ? 2 : GridBagConstraints.REMAINDER;
	gjp.add(gbtx.makePanel(),gbC);
    }

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    gjp.add(xilS7.makePanel(),gbC);

    
    gbC.gridwidth = 1;
    gjp.add(new JLabel("FPGA CSR"),gbC);
    
    csrOffsetTF = new JTextField("0",10);
    gbC.gridwidth = 1;
    gjp.add(csrOffsetTF,gbC);

    button = new JButton("Read");
    button.addActionListener(this);
    button.setActionCommand("readFpga");
    gbC.gridwidth = 1;
    gjp.add(button,gbC);

    readValueTF = new JTextField("0",10);
    readValueTF.setEditable(false);
    gbC.gridwidth = 1;
    gjp.add(readValueTF,gbC);
    
    button = new JButton("Write");
    button.addActionListener(this);
    button.setActionCommand("writeFpga");
    gbC.gridwidth = 1;
    gjp.add(button,gbC);

    writeValueTF = new JTextField("0",10);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    gjp.add(writeValueTF,gbC);
    
    return gjp;
  }

  public void actionPerformed( ActionEvent e )
  {
    Object ob = e.getSource();
    String command = e.getActionCommand();

    if ( command.equals("readFpga") )
    {
      int offset = Integer.parseInt(csrOffsetTF.getText(),16);
      byte readVal = sca.readI2c(i2cBus,fpgaI2cSlaveAddress,offset);
      readValueTF.setText(String.format("%1$X",readVal));
    }
    else if ( command.equals("writeFpga") )
    {
      int offset = Integer.parseInt(csrOffsetTF.getText(),16);
      int writeVal = Integer.parseInt(writeValueTF.getText(),16);
      sca.writeI2c(i2cBus,fpgaI2cSlaveAddress,offset,writeVal);
    }
  }

  public void init(ScaDepp sca)
  {
    this.sca = sca;
    for ( GbtxControl gbtx : gbtxList ){ gbtx.init(sca,dtm,i2cBus); }
    xilS7.init(sca);
  }  

  public void holdAllReset(){ for ( GbtxControl gbtx : gbtxList ) gbtx.assertReset(); }

}
