package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Color;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.gui.*;

/**
<p>This class encapsulates the interactions with the elementary cell and its components.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
class EcControl implements ActionListener
{
  ScaDepp sca;
  @XmlAttribute(name="id")
  int ec;
  JTextField dacTF;
  JSpinner adcS;
  @XmlElement(name="pt1000")
  List<EcPt1000> ptList;
  List<JTextField> seuTFList;
  @XmlElement(name="seu")
  List<EcSeuMonitor> seuList;
  byte [] claroBuf;
  @XmlElement(name="testPulseDac")
  int testPulseDac;
  @XmlElement(name="claro")
  ClaroControl claro;

  public EcControl() { }

  public void actionPerformed( ActionEvent e )
  {
    Object ob = e.getSource();
    String command = e.getActionCommand();

    if ( command.matches("dac") )
    {
      int dacValue = Integer.parseInt(dacTF.getText(),16);
      sca.setDac(testPulseDac,(byte)dacValue);
    }
    else if ( command.matches("adc") )
    {
      ArrayList<String> adcValues = new ArrayList<String>();
      for (EcPt1000 pt : ptList)
      {
	int adcVal = pt.getAdc();
	adcValues.add(String.format("%1$s: %2$d",pt.getName(),adcVal));
      }
      ((SpinnerListModel)adcS.getModel()).setList(adcValues);
    }
    else if ( command.matches("seu") )
    {
      int ordinal = 0;
      for (EcSeuMonitor seu : seuList )
      {
	int seuVal = seu.getAdc();
	seuTFList.get(ordinal).setText(String.format("%1$s: %2$d",seu.getName(),seuVal));
	ordinal++;
      }
    }
  }
    
  public void setEc( int ec ){ this.ec = ec; }
  public int getEc(){ return this.ec; }
  public void init(ScaDepp sca)
  {
    this.sca = sca;
    if (ptList != null) for (EcPt1000 pt : ptList) pt.init(sca);
    if (seuList != null) for (EcSeuMonitor seu : seuList ) seu.init(sca);
    claro.init(sca,ec);
  }
  public void setTestPulseDac(int testPulseDac){ this.testPulseDac=testPulseDac; }

  public JPanel makePanel()
  {
    JPanel jp = new JPanel();
    jp.setName( "EC."+ec );
    jp.setLayout(new GridBagLayout());
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(claro.makePanel(),gbC);

    JButton button = new JButton("Set DAC");
    button.addActionListener(this);
    button.setActionCommand("dac");
    gbC.gridwidth = 1;
    jp.add(button,gbC);
    
    dacTF = new JTextField("00");
    gbC.gridwidth = 1;
    jp.add(dacTF,gbC);

    button = new JButton("Read ADCs");
    button.addActionListener(this);
    button.setActionCommand("adc");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    adcS = new JSpinner(new SpinnerListModel());
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(adcS,gbC);
    ((JSpinner.DefaultEditor)adcS.getEditor()).getTextField().setEditable(false);
    
    button = new JButton("Read SEU Sensors");
    button.addActionListener(this);
    button.setActionCommand("seu");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    seuTFList = new ArrayList<JTextField>();
    if (seuList != null)
    {
      for (EcSeuMonitor seu : seuList )
      {
        JTextField tf = new JTextField();
        tf.setEditable(false);
        seuTFList.add(tf);
        gbC.gridwidth = 1;
        jp.add( tf, gbC );
      }
    }
    return jp;
  }
    
}
