/*

A display class that implements persistence

Steve Wotton, 2000
Cavendish Lab (HEP)

*/
package cbsw.gui;

import java.awt.Dimension;
import java.awt.Color;
import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.Container;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import java.awt.image.*;
import java.awt.print.*;
import java.net.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import javax.swing.border.*;
import java.awt.Point;
import java.util.Vector;
import java.util.prefs.*;
import cbsw.gui.*;

public class CellArray extends JPanel implements MouseMotionListener, MouseListener, ActionListener
{
Container container;
JPopupMenu popupMenu;
JColorChooser colorChooser = new JColorChooser();
BufferedImage bibg;
Graphics2D bibgg2d;
BufferedImage bifg;
Graphics2D bifgg2d;
boolean onOffAlphaMode = false;
Color onFgColor = Color.green;
float onAlpha = 1.0f;
Color offFgColor = Color.red;
float offAlpha = 0.3f;
Color bgColor = Color.blue;
Color gridColor = Color.black;
BasicStroke gridStroke = new BasicStroke(0.0f); // 0.0 means thinnest possible
int pixelHeight, pixelWidth, nxpixels, nypixels;
Point lastClicked;
Point lastPressed, lastReleased;
AffineTransform qrot;
int quadrantRotation = 0;
    //String lastText;

// State of each cell (0=off,1=on)

int[][] cellState;
float[][] alpha;

AffineTransform atfgg2d;
AffineTransform atbgg2d;

public CellArray( Container cont, int nx, int ny, int width, int height )
{
   container = cont;
   nxpixels = nx;
   nypixels = ny;
   pixelWidth = width;
   pixelHeight = height;
   lastPressed = new Point(0,0);
   lastReleased = new Point(0,0);
   lastClicked = new Point(0,0);
   //   lastText = "";

   cellState = new int[nxpixels][nypixels];
   alpha = new float[nxpixels][nypixels];
   for ( int x=0; x<nxpixels; x++ )
   {
     for ( int y=0; y<nypixels; y++ )
     {
	 alpha[x][y] = 1.0f;
     }
   }

   setOpaque( true );

   //   setPreferredSize( new Dimension(nx*width+16,ny*height+24) );
   setPreferredSize( new Dimension(nx*width,ny*height) );
   //   setBorder( BorderFactory.createTitledBorder("Image area") );

   bibg = new BufferedImage(nx*pixelWidth,ny*pixelHeight,BufferedImage.TYPE_INT_RGB);
   bibgg2d = bibg.createGraphics();
   atbgg2d = new AffineTransform(bibgg2d.getTransform());

   bibgg2d.setStroke(gridStroke);

   bifg = new BufferedImage(nx*pixelWidth,ny*pixelHeight,BufferedImage.TYPE_INT_RGB);
   bifgg2d = bifg.createGraphics();
   atfgg2d = new AffineTransform(bifgg2d.getTransform());
   qrot = new AffineTransform(bifgg2d.getTransform());

   bifgg2d.setStroke(gridStroke);

   AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,1.0f );
   bibgg2d.setComposite(ac);

   bifgg2d.setColor(bgColor);
   bifgg2d.fillRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight);
   drawGrid( bibgg2d, gridColor );

   popupMenu = new JPopupMenu();
   makePopupMenu( popupMenu, this );

   addMouseListener( this );
   addMouseMotionListener( this );


}

public void makePopupMenu( JPopupMenu m, ActionListener l )
{

// Colour buttons

        JMenuItem menuItem = m.add( "On colour..." );
        menuItem.setActionCommand("oncolor");
        menuItem.addActionListener( l );

        menuItem = m.add( "Off colour..." );
        menuItem.setActionCommand("offcolor");
        menuItem.addActionListener( l );

        menuItem = m.add( "Background colour..." );
        menuItem.setActionCommand("bgcolor");
        menuItem.addActionListener( l );

        menuItem = m.add( "Grid colour..." );
        menuItem.setActionCommand("gridcolor");
        menuItem.addActionListener( l );

	//        m.addSeparator();

   return;
}

// MouseMotionListener implementation

public void mouseDragged( MouseEvent e ){ return; }

public void mouseMoved( MouseEvent e )
{
   int x = e.getX();
   int y = e.getY();
   int nx = x/pixelWidth;
   int ny = y/pixelHeight;

   if ( nx < 0 || nx >= nxpixels || ny < 0 || ny >= nypixels ) return;

   showStatus( "(x,y)=(" + nx + "," + ny + ") state=" + cellState[nx][ny] );

   return;
}

// MouseListener implementation

    public void mousePressed( MouseEvent e )
    {
	//       showStatus( "Pressed x:" + e.getX() + " y:" + e.getY() );
       if ( e.isPopupTrigger() )
       {
	   popupMenu.show( e.getComponent(), e.getX(), e.getY() );
       }

       try
       {
         Point2D xy = qrot.inverseTransform(new Point(e.getX(),e.getY()),null);
         lastPressed.x = (int)xy.getX()/pixelWidth;
         lastPressed.x = (lastPressed.x<0)?0:lastPressed.x;
         lastPressed.x = (lastPressed.x>=nxpixels)?nxpixels-1:lastPressed.x;

         lastPressed.y = (int)xy.getY()/pixelHeight;
         lastPressed.y = (lastPressed.y<0)?0:lastPressed.y;
         lastPressed.y = (lastPressed.y>=nypixels)?nypixels-1:lastPressed.y;
       } catch (Exception ex) {ex.printStackTrace();}

       return;
    }

    public void mouseReleased( MouseEvent e )
    {
	//       showStatus( "Released x:" + e.getX() + " y:" + e.getY() );
       if ( e.isPopupTrigger() )
       {
	   popupMenu.show( e.getComponent(), e.getX(), e.getY() );
       }

       //       Point xy = null;
       try
       {
         Point2D xy = qrot.inverseTransform(new Point(e.getX(),e.getY()),null);
         lastReleased.x = (int)xy.getX()/pixelWidth;
         lastReleased.x = (lastReleased.x<0)?0:lastReleased.x;
         lastReleased.x = (lastReleased.x>=nxpixels)?nxpixels-1:lastReleased.x;

         lastReleased.y = (int)xy.getY()/pixelHeight;
         lastReleased.y = (lastReleased.y<0)?0:lastReleased.y;
         lastReleased.y = (lastReleased.y>=nypixels)?nypixels-1:lastReleased.y;
       } catch (Exception ex) {ex.printStackTrace();}

       int xstart = (lastReleased.x>=lastPressed.x)?lastPressed.x:lastReleased.x;
       int xend = (lastReleased.x>=lastPressed.x)?lastReleased.x:lastPressed.x;
       int ystart = (lastReleased.y>=lastPressed.y)?lastPressed.y:lastReleased.y;
       int yend = (lastReleased.y>=lastPressed.y)?lastReleased.y:lastPressed.y;
       for ( int x=xstart; x<=xend; x+=1 )
       {
         for ( int y=ystart; y<=yend; y+=1 )
         {
           if ( cellState[x][y] == 0 )
           {
             setCellOn( x, y );
           }
           else
           {
             setCellOff( x, y );
	   }
         }
       }
       update();

       return;
    }

    public void mouseClicked( MouseEvent e )
    {
	//	showStatus( "Clicked x:" + e.getX() + " y:" + e.getY() );
	int x = e.getX();
        int y = e.getY();
        lastClicked.x = x/pixelWidth;
        lastClicked.y = y/pixelHeight;

       return;
    }

    public void mouseEntered( MouseEvent e )
    {
	//	showStatus( "Entered x:" + e.getX() + " y:" + e.getY() );
       return;
    }

    public void mouseExited( MouseEvent e )
    {
	//	showStatus( "Exited x:" + e.getX() + " y:" + e.getY() );
       return;
    }

public int getLastClickedX()
{
  return lastClicked.x;
}

public int getLastClickedY()
{
  return lastClicked.y;
}

// ActionListener implementation

public void actionPerformed( ActionEvent e )
{
   String cmd = e.getActionCommand();

  if ( cmd.equals( "oncolor" ) || cmd.equals( "offcolor" ) || cmd.equals( "bgcolor" ) || cmd.equals( "gridcolor" ) )
  {
     setColor( cmd );
  }
  return;
}

public void drawGrid( Graphics2D g, Color gc )
{
   g.setColor(gc);
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	 g.drawRect(x*pixelWidth, y*pixelHeight, pixelWidth-1, pixelHeight-1);
      }
   }

   return;
}

public void update()
{

   repaint();

   return;
}

public void clear()
{

   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	  cellState[x][y] = 0;
      }
   }

   repaint();

   return;
}

public void setColor( Color col[][] )
{
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {

      }
   }

   repaint();
}

public void setAlpha( float al[][] )
{
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	  alpha[x][y] = al[x][y];
      }
   }

   repaint();
}

public void setAlpha( float al )
{
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	  alpha[x][y] = al;
      }
   }

   repaint();
}

public void setOnColor( Color col ){ onFgColor = col; return; }
public void setOffColor( Color col ){ offFgColor = col; return; }
public Color getOnColor(){ return onFgColor; }
public Color getOffColor(){ return offFgColor; }
public void setBackgroundColor( Color col )
{
   bgColor = col;
   bibgg2d.setColor(bgColor);
   bibgg2d.fillRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight);
   return;
}
public Color getBackgroundColor(){ return bgColor; }
public void setGridColor( Color col )
{
   gridColor = col;
   drawGrid( bibgg2d, gridColor );
   return;
}
public Color getGridColor(){ return gridColor; }

public void setColor( String cmd )
{
Color newColor = null;

   if ( cmd.equals( "oncolor" ) )
     newColor = colorChooser.showDialog( this, "On colour chooser", onFgColor );
   else if ( cmd.equals( "offcolor" ) )
     newColor = colorChooser.showDialog( this, "Off colour chooser", offFgColor );
   else if ( cmd.equals( "bgcolor" ) )
     newColor = colorChooser.showDialog( this, "Background colour chooser", bgColor );
   else if ( cmd.equals( "gridcolor" ) )
     newColor = colorChooser.showDialog( this, "Grid colour chooser", gridColor );

   if ( newColor != null )
   {
	if ( cmd.equals( "oncolor" ) )
	   setOnColor( newColor );
	else if ( cmd.equals( "offcolor" ) )
	   setOffColor( newColor );
	else if ( cmd.equals( "bgcolor" ) )
	   setBackgroundColor( newColor );
	else if ( cmd.equals( "gridcolor" ) )
	   setGridColor( newColor );
   }

   return;
}

public void setCellOn( int x, int y )
{
   if ( (x>=nxpixels) || (y>=nypixels) || (x<0) || (y<0) )
   {
      showStatus( "Array index out of bounds [x][y]=[" + x + "][" + y + "]" );
      return;
   }

   cellState[x][y] = 1;
   if ( onOffAlphaMode ) alpha[x][y] = onAlpha;

   update();

   return;
}

public void setCellOff( int x, int y )
{
   if ( (x>=nxpixels) || (y>=nypixels) || (x<0) || (y<0) )
   {
      showStatus( "Array index out of bounds [x][y]=[" + x + "][" + y + "]" );
      return;
   }

   cellState[x][y] = 0;
   if ( onOffAlphaMode ) alpha[x][y] = offAlpha;

   update();

   return;
}

public void paintNonBuffered( Graphics2D g, Paper pa, int ixa, int iya, int nxAnodes, int nyAnodes, boolean printMono )
{

// Save the current transform

   AffineTransform tr = g.getTransform();

// Choose scaling to fit page

   double xscale = pa.getImageableWidth()/(nxAnodes*nxpixels*pixelWidth);
   double yscale = pa.getImageableHeight()/(nyAnodes*nypixels*pixelHeight);
   double scale;

   if ( xscale < yscale )
   {
       scale = xscale;
   }
   else
   {
       scale = yscale;
   }

// Translate and scale to fit page

   g.translate( pa.getImageableX(), pa.getImageableY() );
   g.scale( scale, scale );
   g.translate( (ixa*nxpixels*pixelWidth), (iya*nypixels*pixelHeight) );

// Draw background

   AlphaComposite ac3 = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,1.0f );
   g.setComposite(ac3);

   g.clearRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight );



// Paint hit pixels

   if ( printMono )
   {
     g.setColor(Color.white);
     g.fillRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight);
     g.setColor( Color.black );
   }
   else
   {
     g.setColor(bgColor);
     g.fillRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight);
     drawGrid( g, gridColor );
   }
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	AlphaComposite ac2 = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,alpha[x][y] );
	g.setComposite(ac2);
	if ( cellState[x][y] == 0 )
	{
          g.setColor(offFgColor);
	}
        else
        {
          g.setColor(onFgColor);
	}   
	g.fillRect(x*pixelWidth, y*pixelHeight, pixelWidth, pixelHeight);
      }
   }

// Draw a frame around the image

   g.setColor( Color.black );
   g.drawRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight );

// Restore the original transform

   g.setTransform( tr );

   return;
}

public void paintComponent(Graphics g)
{
   super.paintComponent(g); // paints the parent

// Draw background

   AlphaComposite ac3 = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,1.0f );
   bifgg2d.setComposite(ac3);

   bifgg2d.clearRect( 0, 0, nxpixels*pixelWidth, nypixels*pixelHeight );
   bifgg2d.drawImage( bifg, 0, 0, this );

// Paint hit pixels

   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	AlphaComposite ac2 = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,alpha[x][y] );
	bifgg2d.setComposite(ac2);
       
	if ( cellState[x][y] == 0 )
	{
          bifgg2d.setColor(offFgColor);
	}
        else
        {
          bifgg2d.setColor(onFgColor);
	}  
 
	bifgg2d.fillRect(x*pixelWidth, y*pixelHeight, pixelWidth, pixelHeight);
        bifgg2d.setColor(gridColor);
	bifgg2d.drawRect(x*pixelWidth, y*pixelHeight, pixelWidth-1, pixelHeight-1);
      }
   }

// Draw foreground

   g.drawImage( bifg, 0, 0, this );

   return;
}

public void getState( int state[][] )
{
   for ( int x=0; x<nxpixels; x++ )
   {
      for ( int y=0; y<nypixels; y++ )
      {
	 state[x][y] = cellState[x][y];
      }
   }
}
public void setOnOffAlphaMode( boolean flag ) { onOffAlphaMode = flag; return; }
public void showStatus( String s )
{
    return;
}

public Vector<Point> getSelectedCells()
{
  Vector<Point> cells = new Vector<Point>();
  for ( int x=0; x<nxpixels; x++ )
  {
    for ( int y=0; y<nypixels; y++ )
    {
	if (cellState[x][y] == 1) cells.add(new Point(x,y));
    }
  }
  return cells;
}

public void setRotation( int quadrant )
{
   quadrantRotation = quadrant;

   qrot = new AffineTransform(atfgg2d);
   qrot.quadrantRotate(quadrantRotation, nxpixels*pixelWidth/2.0, nypixels*pixelHeight/2.0);
   bifgg2d.setTransform(qrot);
   bibgg2d.setTransform(qrot);

   update();

   return;
}

}
//














