package cbsw.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.prefs.*;
import java.text.Format;

public class GuiContainer extends Container
{
protected Preferences prefs;
protected String prefsPrefix;

  public GuiContainer( String pfn, String pfx )
  {
    prefs = Preferences.userNodeForPackage(GuiContainer.class).node(pfn);
    prefsPrefix = pfx;
  }

  public GuiContainer( Preferences pfs, String pfx )
  {
    prefs = pfs;
    prefsPrefix = pfx;
  }

  public Preferences getPrefsUserNode() { return (prefs); }

  public Preferences setPrefsUserNode(String pfn, String pfx)
  {
    prefs = Preferences.userNodeForPackage(GuiContainer.class).node(pfn);
    prefsPrefix = pfx;
    return (prefs);
  }

  public Preferences setPrefsUserNode(Preferences pfs, String pfx)
  {
    prefs = pfs;
    prefsPrefix = pfx;
    return (prefs);
  }

// Creates a text field inside a title-bordered JPanel
// Name is used as the preference name and as the action command
// Returns the JTextField (use getParent() for the container JPanel)

  public JTextField makeTF(String name, String prefsDefault, int width, String borderLabel, String toolTip, boolean registerPrefsAction)
  {
    JTextField tf = new JTextField( prefs.get( prefsPrefix+name,prefsDefault ),width );
    tf.setName(prefsPrefix+name);
    if (registerPrefsAction)
    {
      tf.addActionListener( new ActionListener () { public void actionPerformed(ActionEvent e) { JTextField tf = (JTextField)e.getSource(); prefs.put(tf.getName(),tf.getText()); return; } } );
    }
    tf.setToolTipText( toolTip );
    if ( borderLabel != null )
    {
      JPanel jp = new JPanel();
      jp.setBorder( BorderFactory.createTitledBorder(borderLabel) );
      jp.add( tf );
    }

    return tf;
  }

// An alternative that uses JFormattedTextField
  public JTextField makeTF(Format fmt, String name, String prefsDefault, int width, String borderLabel, String toolTip, boolean registerPrefsAction)
  {
    JFormattedTextField tf = new JFormattedTextField(fmt);
    tf.setColumns(width);
    ((JTextField)tf).setText( prefs.get( prefsPrefix+name,prefsDefault ) );
    tf.setName(prefsPrefix+name);
    if (registerPrefsAction)
    {
      tf.addActionListener( new ActionListener () { public void actionPerformed(ActionEvent e) { JTextField tf = (JTextField)e.getSource(); prefs.put(tf.getName(),tf.getText()); return; } } );
    }
    tf.setToolTipText( toolTip );
    if ( borderLabel != null )
    {
      JPanel jp = new JPanel();
      jp.setBorder( BorderFactory.createTitledBorder(borderLabel) );
      jp.add( tf );
    }

    return tf;
  }

  public JCheckBox makeCheckBox(String name, String label, boolean def, boolean registerPrefsAction)
  {
    boolean pI = prefs.getBoolean(prefsPrefix+name,def);
    JCheckBox cb = new JCheckBox(label,pI);
    
    cb.setName(prefsPrefix+name);
    if (registerPrefsAction)
    {
      cb.addActionListener( new ActionListener () { public void actionPerformed(ActionEvent e) { JCheckBox cb = (JCheckBox)e.getSource(); prefs.putBoolean(cb.getName(),cb.isSelected()); return; } } );
    }

    return cb;
  }

  public static JPanel addBorder(JComponent component, String label )
  {
    JPanel jp = new JPanel();
    jp.setBorder( BorderFactory.createTitledBorder(label) );
    jp.add(component);
    return jp;
  }

  // Save settings for all JCheckBox and JTextField components of a Container.
  // The preferences name is taken from the component name.
  // Function is recursive
  public static void savePreferences(Preferences prefs, Container cn)
  {
    if (prefs == null) {System.out.println("GuiContainer.savePreferences(): Null prefs. Aborting...");return;}
    if (cn == null) {System.out.println("GuiContainer.savePreferences(): Null container. Aborting...");return;}
    for (Component component : cn.getComponents())
    {
      if (component == null) {System.out.println("GuiContainer.savePreferences(): Null component. Aborting...");continue;}
      if (component instanceof JCheckBox)
      {
	if (component.getName() == null) { System.out.println("CheckBox component name is null"); continue; }
	System.out.println(prefs.absolutePath()+"/"+component.getName()+"="+((JCheckBox)component).isSelected());
	prefs.putBoolean( component.getName(),((JCheckBox)component).isSelected());
      }
      else if (component instanceof JTextField) // also for JFormattedTextField
      {
	if (component.getName() == null) { System.out.println("TextField component name is null"); continue; }
	if (((JTextField)component).getText() == null) { System.out.println( "TextField text is null"); continue; }
	System.out.println(prefs.absolutePath()+"/"+component.getName()+"="+((JTextField)component).getText());
	prefs.put( component.getName(),((JTextField)component).getText());
      }
      else if (component instanceof Container) // Recursive
      {
	savePreferences(prefs, (Container)component);
      }
    }
  }

  public static void updateFields(Preferences prefs, Container cn)
  {
    if (prefs == null) {System.out.println("GuiContainer.savePreferences(): Null prefs. Aborting...");return;}
    if (cn == null) {System.out.println("GuiContainer.savePreferences(): Null container. Aborting...");return;}
    for (Component component : cn.getComponents())
    {
      if (component == null) {System.out.println("GuiContainer.updateFields(): Null component. Aborting...");continue;}
      if (component instanceof JCheckBox)
      {
	if (component.getName() == null) { System.out.println("CheckBox component name is null"); continue; }
	((JCheckBox)component).setSelected(prefs.getBoolean( component.getName(),false));
      }
      else if (component instanceof JTextField) // also for JFormattedTextField
      {
	if (component.getName() == null) { System.out.println("TextField component name is null"); continue; }
	if (((JTextField)component).getText() == null) { System.out.println( "TextField text is null"); continue; }
	((JTextField)component).setText(prefs.get( component.getName(),""));
      }
      else if (component instanceof Container) // Recursive
      {
	updateFields(prefs, (Container)component);
      }
    }
  }

  public static void updateUndefinedFields(Preferences prefs, Container cn)
  {
    if (prefs == null) {System.out.println("GuiContainer.savePreferences(): Null prefs. Aborting...");return;}
    if (cn == null) {System.out.println("GuiContainer.savePreferences(): Null container. Aborting...");return;}
    for (Component component : cn.getComponents())
    {
      if (component == null) {System.out.println("GuiContainer.updateFields(): Null component. Aborting...");continue;}
      if (component instanceof JCheckBox)
      {
	if (component.getName() == null) { System.out.println("CheckBox component name is null"); continue; }
        if (!prefs.get(component.getName(),"").equals("")) ((JCheckBox)component).setSelected(prefs.getBoolean( component.getName(),false));
      }
      else if (component instanceof JTextField) // also for JFormattedTextField
      {
	if (component.getName() == null) { System.out.println("TextField component name is null"); continue; }
	if (((JTextField)component).getText() == null) { System.out.println( "TextField text is null"); continue; }
	if (!prefs.get(component.getName(),"").equals("")) ((JTextField)component).setText(prefs.get( component.getName(),""));
      }
      else if (component instanceof Container) // Recursive
      {
	updateUndefinedFields(prefs, (Container)component);
      }
    }
  }

}

