/*
Network parameter panel

Steve Wotton, 2007-2015
Cavendish Lab (HEP)
*/

package cbsw.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.prefs.Preferences;
import cbsw.gui.GuiJPanel;

public class GuiDaqControl extends GuiJPanel implements ActionListener
{
JFrame frame;

JTextField ipProtocolTF;
JTextField ipTtlTF;

JTextField truncationHwmTF;
JTextField mepEventsTF;

//constructor
public GuiDaqControl( Preferences pf, JFrame f )
{
  super(pf,"DaqControl.");

  frame = f;

  // Now create the fields

  JPanel macPanel = makeMacAddressPanel();
  JPanel ipPanel = makeIpAddressPanel();
  JPanel ipJP = makeIpPanel();
  JPanel gbeJP = makeGbePanel();

// Add the subpanels to the main panel

  setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();

  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(2,2,2,2);

  gbC.gridwidth = 1;
  add( macPanel, gbC );

  gbC.gridwidth = GridBagConstraints.REMAINDER;
  add( ipPanel, gbC );

  gbC.gridwidth = 1;
  add( ipJP, gbC );

  gbC.gridwidth = GridBagConstraints.REMAINDER;
  add( gbeJP, gbC );


  JButton button;

  gbC.gridwidth = 1;
  button = new JButton( "Cancel" );
  button.addActionListener( this );
  button.setActionCommand( "cancel" );
  add( button, gbC );

  gbC.gridwidth = GridBagConstraints.REMAINDER;
  button = new JButton( "Save" );
  button.addActionListener( this );
  button.setActionCommand( "save" );
  add( button, gbC );

  return;
}

public JPanel makeMacAddressPanel()
{
  JPanel jP = new JPanel();
  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  jP.setBorder( BorderFactory.createTitledBorder("MAC address") );

  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(2,2,2,2);

  StringBuffer macLabel = new StringBuffer();
  for ( int i=0; i<6; i++ )
  {
    macLabel.append(prefs.get(String.format("net.mac.%d",5-i),"00"));
    if (i<5 ) macLabel.append( ":" );
  }
  jP.add( new JLabel(macLabel.toString()) );

  return jP;
}

public JPanel makeIpAddressPanel()
{
  JPanel jP = new JPanel();
  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  jP.setBorder( BorderFactory.createTitledBorder("IP address") );

  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(2,2,2,2);

  StringBuffer ipLabel = new StringBuffer();
  for ( int i=0; i<4; i++ )
  {
    ipLabel.append(prefs.get(String.format("net.ip.%d",3-i),"0"));
    if (i<3) ipLabel.append( "." );
  }
  jP.add( new JLabel(ipLabel.toString()) );

  return jP;
}

public JPanel makeIpPanel()
{
  JPanel jP = new JPanel();
  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  jP.setBorder( BorderFactory.createTitledBorder("IP parameters") );

  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(2,2,2,2);

  gbC.gridwidth = 1;
  jP.add( new JLabel("Protocol") );
  int ipProtocol = prefs.getInt("net.ip.protocol",0xf2);
  ipProtocolTF = new JTextField( String.format("%02X",ipProtocol),2 );
  ipProtocolTF.setName("net.ip.protocol");
  jP.add( ipProtocolTF );

  jP.add( new JLabel("TTL") );
  int ipTtl = prefs.getInt("net.ip_ttl",64);
  ipTtlTF = new JTextField( String.format("%02X",ipTtl), 2 );
  ipTtlTF.setName("net.ip.ttl");
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add( ipTtlTF, gbC );

  return jP;
}

public JPanel makeGbePanel()
{
  JPanel jP = new JPanel();
  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  jP.setBorder( BorderFactory.createTitledBorder("GBE parameters") );

  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(2,2,2,2);

  jP.add( new JLabel("Truncation size") );
  truncationHwmTF = new JTextField( prefs.get("mep.truncation-hwm","1000"), 4 );
  truncationHwmTF.setName("mep.truncation-hwm");
  truncationHwmTF.setToolTipText("MEP buffer high water mark (hex) bytes");
  gbC.gridwidth = 1;
  jP.add( truncationHwmTF, gbC );

  jP.add( new JLabel("MEP events") );
  mepEventsTF = new JTextField(prefs.get("mep.events","1"), 4 );
  mepEventsTF.setName("mep.events");
  mepEventsTF.setToolTipText("Number of events per MEP (decimal)");
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add( mepEventsTF, gbC );

  return jP;
}

public void actionPerformed( ActionEvent e )
{
   String cmd = e.getActionCommand();

   if ( cmd.equals( "save" ) )
   {
     savePreferences(prefs,this);
     frame.setVisible(false);
   }
   else if ( cmd.equals( "cancel" ) )
   {
     frame.setVisible(false);
   }

   return;
}

}
