/*
Simple histogrammer GUI

Steve Wotton, 2018-
Cavendish Lab (HEP)
*/

package cbsw.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Graphics2D.*;
import java.net.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.util.ArrayList;
import java.util.prefs.Preferences;
import org.jfree.data.xy.XYSeries;

public class JHistogrammer extends JPanel {

Preferences prefs;

DataThread dth;

URL dataURL;
Socket dataSocket;
BufferedReader dataInStream;
String dataSourceString;
    
// Control panel

JComboBox<String> dataSourceCombo;

Histogram hist;
Histogram2D hist2D;

JComboBox<String> keysCombo;

public JHistogrammer( JMenuBar menuBar ) {

    prefs = Preferences.userNodeForPackage(JHistogrammer.class).node("JHistogrammer");

// Make the "File" menu

        JMenu fileMenu = new JMenu( "Main" );

// Make "Exit..." menuitem

        JMenuItem exitItem = new JMenuItem( "Exit..." );
        exitItem.addActionListener( ae -> System.exit(0) );

        fileMenu.add( exitItem );

        menuBar.add( fileMenu );

// Make a panel to contain the graphics sub-parts

        JPanel graphics = new JPanel();
        graphics.setLayout( new GridBagLayout() );
        GridBagConstraints gbC = new GridBagConstraints();

	gbC.insets = new Insets(2,2,2,2);

// Make the histogram area

	hist = new Histogram("");	
        org.jfree.chart.ChartPanel chartPanel = new org.jfree.chart.ChartPanel(hist.getChart());
        chartPanel.setPreferredSize(new java.awt.Dimension(700, 300));
	chartPanel.setMouseZoomable(true, false);

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        graphics.add( chartPanel, gbC );

	// 2D histogram (not displayed yet)
	hist2D = new Histogram2D("");
	
// ComboBox for selection of data source

        JLabel label = new JLabel( "Data source URL" );

        gbC.fill = GridBagConstraints.NONE;
        gbC.anchor = GridBagConstraints.EAST;
        gbC.weightx = 1.0;
        gbC.weighty = 1.0;
        gbC.gridwidth = 1;

        graphics.add( label, gbC );

        gbC.weightx = 0.0;
        gbC.weighty = 0.0;

	// Get a list of files

        dataSourceCombo = new JComboBox<String>();
        dataSourceCombo.setEditable(true);
	//        dataSourceCombo.setMaximumRowCount(6);
	dataSourceCombo.addItem( prefs.get("JHistogrammer.dataSourcePath","file:hist.dat") );
        File dataSourceDirectory = new File( prefs.get("JHistogrammer.dataSourceDirectory","file:.") );
        java.io.FileFilter filter = new java.io.FileFilter() { public boolean accept(File f){ return f.toURI().toString().endsWith(".dat");} };
        File dataFiles [] = dataSourceDirectory.listFiles(filter);
        if ( dataFiles != null )
	{
	  for ( File sourceFile : dataFiles ) dataSourceCombo.addItem( sourceFile.toURI().toString() );
	}
        dataSourceCombo.addItem( new String("http://www.hep.phy.cam.ac.uk/lhcb/hist.dat") );
        dataSourceCombo.addItem( new String("file:/c:/Data/hist.dat") );
	dataSourceCombo.addItem( new String("http://localhost:7999/") );
        dataSourceCombo.addActionListener(ae -> updateDataURL());
        dataSourceCombo.setToolTipText( "URL of dataset" );
	//        int dataSourceComboIdx = prefs.getInt("JHistogrammer.dataSourceIdx",0);
	//        dataSourceCombo.setSelectedIndex(dataSourceComboIdx);

        gbC.fill = GridBagConstraints.HORIZONTAL;
        gbC.anchor = GridBagConstraints.CENTER;
        graphics.add( dataSourceCombo, gbC );

        JButton browseButton = new JButton( "Browse" );
        browseButton.addActionListener(ae->browse());

        gbC.fill = GridBagConstraints.VERTICAL;
        gbC.anchor = GridBagConstraints.CENTER;
        gbC.gridwidth = GridBagConstraints.REMAINDER;
        graphics.add( browseButton, gbC );

        gbC.gridwidth = 1;

// Series add/remove

	JPanel seriesJP = new JPanel();
	seriesJP.setBorder( BorderFactory.createTitledBorder("Add/remove series") );

	keysCombo = new JComboBox<String>();
	keysCombo.setEditable(true);
        seriesJP.add( keysCombo );

	JButton button = new JButton("Show");
        button.addActionListener(ae->hist.show((String)keysCombo.getSelectedItem()));
        seriesJP.add( button );
	
	button = new JButton("Hide");
        button.addActionListener(ae->hist.hide((String)keysCombo.getSelectedItem()));
        seriesJP.add( button );

	button = new JButton("Show all");
        button.addActionListener(ae->hist.show());
        seriesJP.add( button );

	button = new JButton("Hide all");
        button.addActionListener(ae->hist.hide());
        seriesJP.add( button );

	button = new JButton("Next");
        button.addActionListener(ae->hist.next());
        seriesJP.add( button );
	
	button = new JButton("Previous");
        button.addActionListener(ae->hist.previous());
        seriesJP.add( button );
	/*
	button = new JButton("XYZ");
        button.addActionListener(ae->{hist2D.readStream(null); hist2D.saveAsSVG();});
        seriesJP.add( button );
	*/
	button = new JButton("SVG");
        button.addActionListener(ae->hist.saveAsSVG());
        seriesJP.add( button );

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        graphics.add( seriesJP, gbC );

        gbC.gridwidth = 1;

        gbC.weightx = 0.0f;
        gbC.weighty = 0.0f;

// Add the graphics pane

        add(graphics, BorderLayout.SOUTH);

// Set preferences

        setPreferences();

};

//
// DataThread handles the IO and decoding
//

public class DataThread extends Thread
{
  public DataThread(){ ; }
  public void run()
  {

       try
       {

// Find out whether to open a socket or a normal URL

         int port = dataURL.getPort();

         if ( port == 7999 )
	 {
           dataSocket = new Socket( dataURL.getHost(), port );
           dataInStream = new BufferedReader(new InputStreamReader(dataSocket.getInputStream()));
	 }
         else
         {
	   dataInStream = new BufferedReader(new InputStreamReader(dataURL.openStream()));
         }

	 hist.readStream(dataInStream);

       }
       catch ( Histogram.EOFException eofe )
       {
	 keysCombo.removeAllItems();
	 for (Comparable key : hist.keys()) keysCombo.addItem((String)key);
	 hist.show();
       }
       catch ( IOException ioe ){ System.err.println("Warning: " + ioe.getMessage() ); }
       finally { closeStream(); }

  }
}

// Close and destroy the stream/socket if they already exist
public void closeStream()
{
  try
  {
    if ( dataInStream != null ) { dataInStream.close(); dataInStream = null; }
    if ( dataSocket != null ) { dataSocket.close(); dataSocket = null; }
  }
  catch ( IOException ioe ){ System.err.println("closeStream(): Warning: " + ioe.getMessage() ); }
}

public void updateDataURL()
{
// Make a URL representing the source

  dataSourceString = (String)dataSourceCombo.getSelectedItem();
  prefs.put("JHistogrammer.dataSourcePath",dataSourceString);
  System.out.println("Setting source " + dataSourceString);

  try
  {

    URL newDataURL = new URL(dataSourceString);
    if ( newDataURL == null ) System.err.println("Error forming data source URL: " + dataSourceString );

    dataURL = null;
    dataURL = newDataURL;

    hist.clear();
    startThread();

  }
  catch ( MalformedURLException ue )
  {
     System.err.println("Warning: " + ue.getMessage() );
  }

  return;
}

public void setPreferences()
{
  return;
}

public void browse()
{
  JFileChooser chooser = new JFileChooser();
  javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter() { public boolean accept(File f){ return f.getPath().endsWith(".dat");} public String getDescription(){return new String("Histogram files");}};

  chooser.setFileFilter( filter );
  int chooserc = chooser.showOpenDialog(null);
  if(chooserc == JFileChooser.APPROVE_OPTION)
  {
    File dataFile = chooser.getSelectedFile();
    dataSourceCombo.insertItemAt( dataFile.toURI().toString(),0 );
    dataSourceCombo.setSelectedIndex(0);
    prefs.put("JHistogrammer.dataSourcePath",dataFile.toURI().toString());
    prefs.put("JHistogrammer.dataSourceDirectory",dataFile.getParent());
    updateDataURL();
  }
}

public void startThread()
{
    dth = null;
    dth = new DataThread();
    dth.start();
}

public static void main( String[] args) {

// The pause here is for the splash screen which closes as soon as the first swing window is made visible.

  try
  {
    Thread.sleep(1000);
  }
  catch ( InterruptedException inte )
  {
    System.err.println("Warning: " + inte.getMessage() );
  }

// Force the System L&F

  String laf = UIManager.getSystemLookAndFeelClassName();
  try 
  {
    UIManager.setLookAndFeel(laf);
  }
  catch (UnsupportedLookAndFeelException exc)
  {
    System.err.println("Warning: UnsupportedLookAndFeel: " + laf);
  }
  catch (Exception exc)
  {
    System.err.println("Error loading " + laf + ": " + exc);
  }

  JFrame frame = new JFrame( "JHistogrammer" );
  frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

  Container cP = frame.getContentPane();
  cP.setLayout(new BorderLayout());

  JMenuBar mB = new JMenuBar();
  frame.setJMenuBar( mB );

  JHistogrammer hist = new JHistogrammer( mB );
  cP.add( hist, BorderLayout.NORTH );

  frame.pack();
  frame.setVisible(true);

  /*
  try { System.getProperties().storeToXML(System.err,"System properties"); }
  catch (IOException ioe){}
  */
  return;
}

}





































