package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAttribute;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.prefs.Preferences;

/**
<p>This is the abstract base class for the GBTX.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
abstract class GbtxParent
{
  @XmlAttribute(name="txBaseAddress")
  int txBaseAddress;
  @XmlAttribute(name="rxBaseAddress")
  int rxBaseAddress;
  @XmlAttribute(name="id")
  int gbtx;
    //  int pdmdb;
  cbsw.RegisterIo io;
  Preferences prefs;
  String name;
  BitSet csrBitSet;
  BitSet rdBitSet;

// Keep a list of the active e-ports and clocks. Updated when reading .gbt
  ArrayList<String> activeInEGroups;
  ArrayList<String> activeOutEGroups;
  ArrayList<String> activeEPorts;
  ArrayList<String> activePSClocks;
    
  JTextField puStateTF;
  JTextField cfgTF;
    //  byte [] gbtxBuf;
  byte [] gbtxReadBuf;
  JCheckBox gbldCB;

  JComboBox<String> regNameCB;
  JTextField regValueTF;

  public GbtxParent()
  {
    prefs = Preferences.userNodeForPackage(GbtxParent.class).node("GBTX");
    activeInEGroups = new ArrayList<String>();
    activeOutEGroups = new ArrayList<String>();
    activeEPorts = new ArrayList<String>();
    activePSClocks = new ArrayList<String>();
  }

  public GbtxParent(int id, int txBaseAddress, int rxBaseAddress)
  {
    gbtx = id;
    this.txBaseAddress = txBaseAddress;
    this.rxBaseAddress = rxBaseAddress;
    prefs = Preferences.userNodeForPackage(GbtxParent.class).node("GBTX");
    activeInEGroups = new ArrayList<String>();
    activeOutEGroups = new ArrayList<String>();
    activeEPorts = new ArrayList<String>();
    activePSClocks = new ArrayList<String>();
  }
    
  public void init(cbsw.RegisterIo io){ this.io = io; }
    
  public void setPreferencesRoot( Preferences prefs ){ this.prefs = prefs; }
  public void setName(String name){this.name = name; readConfigurationFile();}
  public BitSet getCsrBitSet(){return rdBitSet;}
  public int getGbtxId(){return gbtx;}

/**
<p>Convenience method that makes a GUI panel.
@return The panel.
**/
  public JPanel makePanel()
  {
    JPanel jp = new JPanel();
      
    jp.setName( name );
    jp.setBorder( BorderFactory.createTitledBorder(name) );

    jp.setLayout(new GridBagLayout());
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new java.awt.Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JButton button;
    
    button = new JButton("Save As...");
    button.addActionListener(a->GbtxRegisterList.write(csrBitSet));
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("Choose file");
    button.addActionListener(a->chooseConfigurationFile());
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    Preferences prefs = Preferences.userNodeForPackage(GbtxParent.class).node("GBTX");
    cfgTF = new JTextField(prefs.get(name+".file-name","gbtx.gbt"));
    cfgTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add( cfgTF, gbC );
    
    button = new JButton("Set");
    button.addActionListener(l->set());
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    regNameCB = new JComboBox<String>();
    regNameCB.setEditable(true);
    regNameCB.addItem("eport.pa.*.config");
    regNameCB.addItem("eport.pa.*.reset");
    regNameCB.addItem("eport.pa.*.0.phase");
    regNameCB.addItem("eport.pa.*.4.phase");
    regNameCB.addItem("ps.0.coarse-delay");
    regNameCB.addItem("ps.0.fine-delay");
    regNameCB.addItem("xpll.frequency-trim");
    regNameCB.addItem("efuse.update.enable");
    regNameCB.addItem("efuse.address");
    regNameCB.addItem("efuse.data");
    regNameCB.addItem("[bypassXpll]");
    regNameCB.addItem("[enableXpll]");
    regNameCB.addItem("[minimal]");
    regNameCB.addItem("[setGbld]");
    regNameCB.addItem("[ecEport]");
    regNameCB.addItem("[clear]");
    gbC.gridwidth = 1;
    jp.add(regNameCB,gbC);
    
    regValueTF = new JTextField("0");
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(regValueTF,gbC);

    button = new JButton("Reset");
    button.addActionListener(a->reset());
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    //    button = new JButton(" Program GBLD");
    //    button.addActionListener(a->programGBLD());
    //    gbC.gridwidth = 1;
    //    jp.add(button,gbC);

    button = new JButton("Configure");
    button.addActionListener(a->configure());
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    gbldCB =  new JCheckBox("GBLD",false);
    gbC.gridwidth = 1;
    jp.add( gbldCB, gbC );
  
    button = new JButton("Decode");
    button.addActionListener(a->Gbtx.decode(rdBitSet));
    //    button.addActionListener(a->Gbtx.decode(csrBitSet));
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("Verify");
    button.addActionListener(a->verify());
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(button,gbC);

    puStateTF = new JTextField(10);
    puStateTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add( puStateTF, gbC );
    
    return jp;
  }

public abstract void get(byte [] bb, int offset);
public abstract void put(byte [] bb, int offset);
public abstract void reset();
    
public void put(String regName, String regValue)
{
  GbtxRegisterDescriptor reg = Gbtx.map.get(regName);
  reg.set(csrBitSet,regValue);
  
  int offset = reg.getOffsetA();
  byte [] regBuf = reg.getBytes(csrBitSet,offset);
  put(regBuf,offset/8);
  
  if (reg.triplicated)
  {
    offset = reg.getOffsetB();
    regBuf = reg.getBytes(csrBitSet,offset);
    put(regBuf,offset/8);
    
    offset = reg.getOffsetC();
    regBuf = reg.getBytes(csrBitSet,offset);
    put(regBuf,offset/8);
  }
    
  return;
}
    
public String get(String regName)
{
  GbtxRegisterDescriptor reg = Gbtx.map.get(regName);
  
  return reg.get(this);
}
    
public void set(){  set((String)regNameCB.getSelectedItem(),regValueTF.getText()); }
    
// Set a register by name and hex-encoded string value
// Decode special case: keys starting eport.pa.*. are applied to all groups
    
  public void set(String key, String val)
  {
    if (key.equals("[bypassXpll]"))
    {
      bypassXpll();
    }
    else if (key.equals("[enableXpll]"))
    {
      enableXpll();
    }
    else if (key.equals("[minimal]"))
    {
      minimal();
    }
    else if (key.equals("[setGbld]"))
    {
      setGbld();
    }
    else if (key.equals("[ecEport]"))
    {
      ecEport();
    }
    else if (key.equals("[clear]"))
    {
      csrBitSet.clear();
    }
    else if (key.matches("eport\\.pa\\.\\*\\..*"))
    {
      for (int group=0;group<7;group++) put(key.replace("*",String.valueOf(group)),val);
    }
    else
    {
      put(key,val);
    }
  }
    
/**
<p>Update GBLD register bits of the cached GBTX registers.</p>
**/
  public void setGbld()
  {
    GbtxRegisterList.get("gbld.slave-address").set(csrBitSet,"7E");
    GbtxRegisterList.get("gbld.control").set(csrBitSet,"87");
    GbtxRegisterList.get("gbld.pre-emphasis.width").set(csrBitSet,"2");
    GbtxRegisterList.get("gbld.modulation-current").set(csrBitSet,"20");
    GbtxRegisterList.get("gbld.bias-current").set(csrBitSet,"20");
    GbtxRegisterList.get("gbld.pre-emphasis.rising-height").set(csrBitSet,"8");
    GbtxRegisterList.get("gbld.pre-emphasis.falling-height").set(csrBitSet,"8");
    GbtxRegisterList.get("gbld.pre-emphasis.mask").set(csrBitSet,"3");
    GbtxRegisterList.get("gbld.modulation-current.mask").set(csrBitSet,"3F");
    GbtxRegisterList.get("gbld.bias-current.mask").set(csrBitSet,"FF");
    //    GbtxRegisterList.get("gbld.pre-driver.state").set(csrBitSet,"0"); // Read-only
    GbtxRegisterList.get("gbld.pre-driver.bias").set(csrBitSet,"4");
  }
    
/**
<p>Update GBTX register bits that enable the Xpll.</p>
**/
  public void enableXpll()
  {
    GbtxRegisterList.get("xpll.enable").set(csrBitSet,true);
    GbtxRegisterList.get("xpll.phase-detector.enable").set(csrBitSet,true);
    GbtxRegisterList.get("xpll.auto-restart.enable").set(csrBitSet,false);
    GbtxRegisterList.get("xpll.control-override").set(csrBitSet,true);
    //    GbtxRegisterList.get("xpll.frequency-trim").set(csrBitSet,"50");
    GbtxRegisterList.get("xpll.gm-select").set(csrBitSet,"A");
    GbtxRegisterList.get("xpll.cp").set(csrBitSet,"B");
    GbtxRegisterList.get("cm.refclk.select").set(csrBitSet,"1");
    return;
  }
    
/**
<p>Update GBTX register bits that bypass the Xpll.</p>
**/
  public void bypassXpll()
  {
    GbtxRegisterList.get("xpll.enable").set(csrBitSet,false);
    GbtxRegisterList.get("xpll.phase-detector.enable").set(csrBitSet,false);
    GbtxRegisterList.get("xpll.auto-restart.enable").set(csrBitSet,false);
    GbtxRegisterList.get("xpll.control-override").set(csrBitSet,true);
    GbtxRegisterList.get("xpll.frequency-trim").set(csrBitSet,"5D");
    GbtxRegisterList.get("xpll.gm-select").set(csrBitSet,"A");
    GbtxRegisterList.get("xpll.cp").set(csrBitSet,"0");
    GbtxRegisterList.get("cm.refclk.select").set(csrBitSet,"1");
    return;
  }
    
/**
<p>Update GBTX register bits with the minimal set.</p>
<p>The minimal set should be enough to get the TCM link alive on power up.</p>
**/
  public void minimal()
  {
    csrBitSet.clear();
      //    GbtxRegisterList.get("xpll.enable").set(csrBitSet,false);
      //    GbtxRegisterList.get("xpll.phase-detector.enable").set(csrBitSet,false);
      //    GbtxRegisterList.get("xpll.auto-restart.enable").set(csrBitSet,false);
    GbtxRegisterList.get("watchdog.enable").set(csrBitSet,true);
    GbtxRegisterList.get("watchdog.startup-timeout-enable").set(csrBitSet,true);
    GbtxRegisterList.get("xpll.control-override").set(csrBitSet,true);
    GbtxRegisterList.get("xpll.frequency-trim").set(csrBitSet,"5D");
    GbtxRegisterList.get("xpll.gm-select").set(csrBitSet,"A");
    //    GbtxRegisterList.get("xpll.cp").set(csrBitSet,"0");
    GbtxRegisterList.get("cm.refclk.select").set(csrBitSet,"1");
    
    GbtxRegisterList.get("serialiser.lock-time").set(csrBitSet,"6");
    GbtxRegisterList.get("serialiser.unlock-time").set(csrBitSet,"3");
    GbtxRegisterList.get("serialiser.current").set(csrBitSet,"8");
    GbtxRegisterList.get("serialiser.resistor").set(csrBitSet,"2");
    GbtxRegisterList.get("tx.select-positive-edge").set(csrBitSet,"1");
    
    //    GbtxRegisterList.get("deserialiser.gain").set(csrBitSet,"0");
    GbtxRegisterList.get("deserialiser.resistor").set(csrBitSet,"d");
    GbtxRegisterList.get("rx.valid-headers").set(csrBitSet,"f");
    GbtxRegisterList.get("rx.max-invalid-headers").set(csrBitSet,"4");
    GbtxRegisterList.get("rx.min-valid-headers").set(csrBitSet,"8");
    GbtxRegisterList.get("rx.select-i1").set(csrBitSet,"2");
    
    GbtxRegisterList.get("rx.frequency-detector").set(csrBitSet,"2");
    GbtxRegisterList.get("rx.phase-detector").set(csrBitSet,"4");
    
    GbtxRegisterList.get("rx.switch.0").set(csrBitSet,"1");
    GbtxRegisterList.get("rx.switch.1").set(csrBitSet,"1");
    GbtxRegisterList.get("rx.switch.2").set(csrBitSet,"1");
    GbtxRegisterList.get("tx.switch.0").set(csrBitSet,"1");
    GbtxRegisterList.get("tx.switch.1").set(csrBitSet,"1");
    GbtxRegisterList.get("tx.switch.2").set(csrBitSet,"1");
    //    GbtxRegisterList.get("ps.pll.resetb").set(csrBitSet,"1");
    //    GbtxRegisterList.get("ps.pll.charge-pump").set(csrBitSet,"f");
    //    GbtxRegisterList.get("ps.pll.resistor").set(csrBitSet,"4");
    //    GbtxRegisterList.get("ps.pll.enable").set(csrBitSet,"1");
    GbtxRegisterList.get("efuse.update.enable").set(csrBitSet,"1");

    // Its probably very convenient for the EC e-port to come alive at power up too
    ecEport();

    //    GbtxRegisterList.get("cfg.do").set(csrBitSet,"aa");
    return;
  }

/**
<p>Update GBTX register bits that enable the EC ePort.</p>
**/
  public void ecEport()
  {
    GbtxRegisterList.get("eport.pa.ec.config").set(csrBitSet,"d");
    GbtxRegisterList.get("eport.pa.ec.enable").set(csrBitSet,"1");
    GbtxRegisterList.get("eport.pa.ec.termination").set(csrBitSet,"1");
    GbtxRegisterList.get("eport.pa.coarse-lock").set(csrBitSet,"1");
    GbtxRegisterList.get("eport.ec.mode").set(csrBitSet,"1");
  }

/**
<p>Select and cache the GBTX register file.</p>
**/
  public void chooseConfigurationFile()
  {
    Preferences prefs = Preferences.userNodeForPackage(GbtxParent.class).node("GBTX");
    JFileChooser chooser = new JFileChooser(prefs.get("chooser.gbtx.dir","."));
    FileNameExtensionFilter filter = new FileNameExtensionFilter( "GBTX config files (GBT)", "gbt" );
    chooser.setFileFilter(filter);
    int returnVal = chooser.showOpenDialog(null);
    if(returnVal != JFileChooser.APPROVE_OPTION) return;
    if (!chooser.getSelectedFile().isFile()) return;
    prefs.put("chooser.gbtx.dir",chooser.getSelectedFile().getParent());
    prefs.put(name+".file-name",chooser.getSelectedFile().getAbsolutePath());
    cfgTF.setText(prefs.get(name+".file-name","gbtx.gbt"));
    readConfigurationFile();
  }

/**
<p>Cache the GBTX register file and built lists of active e-links and clocks</p>
**/
  public void readConfigurationFile()
  {
    Preferences prefs = Preferences.userNodeForPackage(GbtxParent.class).node("GBTX");
    System.err.println("GbtxParent.readConfigurationFile(): "+prefs.get(name+".file-name","gbtx.gbt"));
    csrBitSet = GbtxRegisterList.read(prefs.get(name+".file-name","gbtx.gbt"));

// Update list of active input eGroups
    activeInEGroups.clear();
    for (int egroup=0;egroup<7;egroup++)
    {
      String key = "eport.pa."+egroup;
      if (!GbtxRegisterList.get(key+".rate").get(csrBitSet).equals("00") ) activeInEGroups.add(key);
    }
    for (String egroup : activeInEGroups) System.err.println("GbtxParent.readConfigurationFile(): Input eGroup "+egroup+" is active");

// Update list of active output eGroups
    activeOutEGroups.clear();
    for (int egroup=0;egroup<5;egroup++)
    {
      String key = "eport."+egroup;
      if (!GbtxRegisterList.get(key+".rate").get(csrBitSet).equals("00") ) activeOutEGroups.add(key);
    }
    for (String egroup : activeOutEGroups) System.err.println("GbtxParent.readConfigurationFile(): Output eGroup "+egroup+" is active");

// Update list of active ePorts
    activeEPorts.clear();
    for (int egroup=0;egroup<7;egroup++)
    {
      for (int eport=0;eport<8;eport++)
      {
	String key = "eport.pa."+egroup+"."+eport;
	if (GbtxRegisterList.get(key+".enable").get(csrBitSet).equals("01") ) activeEPorts.add(key);
      }
    }
    for (String eport : activeEPorts) System.err.println("GbtxParent.readConfigurationFile(): ePort "+eport+" is active");

// Update list of active phase shifter clocks
    activePSClocks.clear();
    for (int clock=0;clock<8;clock++)
    {
      String key = "ps."+clock;
      if (!GbtxRegisterList.get(key+".cset").get(csrBitSet).equals("0f") ) activePSClocks.add(key);
    }
    for (String clock : activePSClocks) System.err.println("GbtxParent.readConfigurationFile(): PS clock "+clock+" is active");
  }

/**
<p>Update the GBTX status field in the GUI panel.</p>
@return Status summary string
**/
  public String showStatus()
  {
    int stateInt = Integer.parseInt(GbtxRegisterList.get("powerup-fsm-state").get(rdBitSet),16);
    String state;
    try { state = Gbtx.fsmStates.get( stateInt ); }
    catch (IndexOutOfBoundsException oobe){state = "Unknown"; System.err.println("GbtxParent.showStatus: Unknown state "+stateInt);}
    String serialNo = GbtxRegisterList.get("fuse.serial").get(rdBitSet);
    String xPllLock = GbtxRegisterList.get("xpll.locked").get(rdBitSet);  
    puStateTF.setText(String.format("[0x%s] %s xPllLock=%s",serialNo,state,xPllLock));
    return "0x" + serialNo + "|" + state + "|" + xPllLock;
  }
    
/**
<p>Write the cached GBTX register bits to the hardware.</p>
**/
  void load()
  {

// Pack gbtx settings into array

    int buflen = 366; byte [] gbtxBuf = new byte [buflen];
    gbtxBuf = Arrays.copyOfRange(csrBitSet.toByteArray(),0,buflen);
    put(gbtxBuf,0);
        
    return;
  }
    
/**
<p>Configure GBTX.</p>
<p>Disables the watchdog and timeout feature duing the configuration.</p>
<p>Performs a soft reset.</p>
**/
public void configure()
{
  System.out.println("GbtxParent.configure(): Writing master GBTX configuration");
  
// The watchdog and timeout feature are disabled during soft configuration otherwise a timeout might cause the volatile registers to be reset
  put("watchdog.enable","0");
  put("watchdog.startup-timeout-enable","0");

  load();

  softReset();

// Put the downlinks into 320MBs mode for phase training
  put("epll.rx.cp","b");
  put("epll.rx.resistor","2");
  put("epll.rx.capacitor","2");
  put("epll.rx.phase","FF");
  for (String egroup : activeOutEGroups) put(egroup+".rate","3");
      
// Re-enable the watchdog and timeout feature
  put("watchdog.enable","1");
  put("watchdog.startup-timeout-enable","1");
}

/**
<p>Read configuration register bits from hardware.</p>
**/
  public void readConfiguration()
  {
    gbtxReadBuf = new byte [436];
    get(gbtxReadBuf,0);
    rdBitSet = BitSet.valueOf(gbtxReadBuf);
    return;
  }
    
/**
<p>Read and verify the GBTX registers against the cached values.</p>
@return Verification status.
**/
  public String verify()
  {

    boolean ok = true;
    try
    {
      readConfiguration();
      String status = showStatus();
      
// Compare gbtx settings

      byte [] gbtxBuf = Arrays.copyOfRange(csrBitSet.toByteArray(),0,367);

      int reg=0;
      for (byte val: gbtxBuf)
      {
        if (val != gbtxReadBuf[reg])
        {
	  ok = false;
	  System.err.println(String.format("[%03d] %02X!=%02X",reg,gbtxReadBuf[reg],val));
	}
        reg++;
      }

      // Now show all non-zero bytes
      reg=0;
      System.err.println("--- Non-zero bytes ---");
      for (byte val: gbtxBuf)
      {
        if (gbtxReadBuf[reg]!=0)
        {
	  System.err.println(String.format("[%03d,%03x] %02X",reg,reg,gbtxReadBuf[reg]));
	}
        reg++;
      }
      System.err.println("----------------------");
      
      return (ok ? "OK" : "FAIL") + "|" + status;
    }
    catch (Exception ex){ throw new RuntimeException("GbtxParent.verify(): "+ex.getMessage()); }
  }

/**
<p>Trigger the programming of the GBLD.</p>
<p>Note: The GBLD I2C address (0x7e) must be preloaded into GBTX reg 253.</p>
<p>Trigger the transfer of the GBLD registers from the GBTX registers.</p>
**/
  public void programGBLD()
  {
    byte [] putBuf = new byte [1];
    put(putBuf,388);
    try { Thread.sleep(500); } catch (InterruptedException ie) {};

    return;
  }

/**
<p>Trigger the reading of the GBLD.</p>
<p>Note: The GBLD I2C address (0x7e) must be preloaded into GBTX reg 253.</p>
<p>Trigger the transfer of the GBLD registers into the GBTX registers.</p>
**/
  public void readGBLD()
  {
    byte [] putBuf = new byte [1];
    put(putBuf,389);

    try { Thread.sleep(500); } catch (InterruptedException ie) {};

    return;
  }

/**
<p>A register read/write test</p>
**/
  public void test()
  {
    System.err.println("GBTX r/w test");

    for (byte i=0;i<5;i++)
    {

	byte [] bb = {i,i,i};
	put(bb,238); // Fusing registers
        try { Thread.sleep(50); } catch (InterruptedException ie) {};
// Check the values were written correctly

	byte [] chk = new byte [3];
        get(chk,238);
        try { Thread.sleep(50); } catch (InterruptedException ie) {};
	if (chk[0] != i) System.err.println("Fuse address LSB error "+i+" "+chk[0]);
	if (chk[1] != i) System.err.println("Fuse address MSB error "+i+" "+chk[0]);
	if (chk[2] != i) System.err.println("Fuse value error "+i+" "+chk[0]);
    }

  }

/**
<p>Perform a reset of the GBTX phase shifter logic.</p>
<p>When calling this it is advisable/mandatory to disable the watchdog/timeout feature.</p>
**/
public void resetPhaseShifter()
{
  put("ps.pll.resetb","0"); // Assert phase shifter PLL reset, active low
  for (String clock : activePSClocks) put(clock+".resetb","0"); // Assert phase shifter DLL reset, active low
  
  put("ps.pll.resetb","1"); // Deassert phase shifter PLL reset, active low

  try {Thread.sleep(10);} catch (InterruptedException ie){} // Time for PS PLL to lock before releasing DLL resets
  
  for (String clock : activePSClocks) put(clock+".resetb","1"); // Deassert Phase shifter DLL reset, active low
}

/**
<p>Perform a reset of the active GBTX ePorts.</p>
<p>When calling this it is advisable/mandatory to disable the watchdog/timeout feature.</p>
**/
public void resetEPorts()
{
  for (String egroup : activeInEGroups) put(egroup+".reset","1"); // Assert egroup PA DLL reset
  for (String eport : activeEPorts) put(eport+".reset","1"); // Assert eport reset
  put("eport.pa.ec.reset","1"); // Assert EC PA DLL reset
  
  for (String egroup : activeInEGroups) put(egroup+".reset","0"); // Deassert egroup PA DLL reset
  put("eport.pa.ec.reset","0"); // Deassert EC PA DLL reset

  try {Thread.sleep(10);} catch (InterruptedException ie){} // Time for PS PLL and PA DLLs to lock before releasing DLL resets

  for (String eport : activeEPorts) put(eport+".reset","0"); // Deassert eport reset
}

/**
<p>Perform a soft reset of the GBTX.</p>
<p>When fused with the minimal configuration the GBTX startup completes with many registers not yet programmed.
This means that when the remaining registers are set parts of the GBTX need to be manually reset.
<p>This method manually triggers the required resets.</p>
<p>When calling this it is advisable/mandatory to disable the watchdog/timeout feature.</p>
**/
public void softReset()
{
  resetPhaseShifter();
  resetEPorts();
}

}

