package cbsw.lhcb.pdmdb;

import java.util.BitSet;
import java.util.Arrays;
import javax.xml.bind.annotation.*;

/**
<p>This class describes a single GBTX register.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
public class GbtxRegisterDescriptor
{
  @XmlValue
  String description;
  @XmlAttribute
  String name;
// Triplicated location offsets
  @XmlAttribute
  int offsetA;
  @XmlAttribute
  int offsetB;
  @XmlAttribute
  int offsetC;
  @XmlAttribute
  int length;
  @XmlAttribute
  boolean triplicated;

  public GbtxRegisterDescriptor(){}

  public String getDescription(){return description;}
  public String getName(){return name;}
  public int getOffset(){return offsetA;}
  public int getOffsetA(){return offsetA;}
  public int getOffsetB(){return offsetB;}
  public int getOffsetC(){return offsetC;}
  public int getLength(){return length;}

// Return an array of bytes spanning the register bits
    
  public byte [] getBytes(BitSet regBits){ return getBytes(regBits,offsetA); }

  public byte [] getBytes(BitSet regBits, int offset)
  {
    int bitLow = offset&0xfffffff8;
    int bitHigh = ((offset+length-1)&0xfffffff8)+8;
    BitSet bits = regBits.get(bitLow,bitHigh);
    byte [] result = Arrays.copyOfRange(bits.toByteArray(),0,(bitHigh-bitLow)/8);
    return result;
  }

  public String get(BitSet regBits)
  {
    BitSet bits = regBits.get(offsetA,offsetA+length);
    byte [] result = Arrays.copyOfRange(bits.toByteArray(),0,(length+7)/8);
    String resultString = new String();
    for ( byte b: result ) resultString = String.format("%1$02x",b) + resultString;
    return resultString;
  }

// Direct read from hardware
  public String get(GbtxParent gbtx)
  {
    int byteLow = offsetA/8;
    int byteHigh = (offsetA+length-1)/8;
    byte [] rdBuf = new byte [byteHigh-byteLow+1];
    gbtx.get(rdBuf,byteLow);

// Shift the bits down to the low byte boundary
    int bitShiftDown = offsetA - 8*byteLow;
    BitSet rdBits = BitSet.valueOf(rdBuf);
    BitSet bits = rdBits.get(bitShiftDown,bitShiftDown+length);

// Convert to hexadecimal string
    StringBuffer sb = new StringBuffer();
    byte [] result = Arrays.copyOfRange(bits.toByteArray(),0,(length+7)/8);
    for ( byte b: result ) sb.insert(0,String.format("%02x",b));

    return sb.toString();
  }
    
  public void set(BitSet regBits, boolean flag)
  {
    if (length != 1) return;
    if (flag)
    {
      regBits.set(offsetA);
      if (triplicated)
      {
        regBits.set(offsetB);
        regBits.set(offsetC);
      }
    }
    else
    {
      regBits.clear(offsetA);
      if (triplicated)
      {
        regBits.clear(offsetB);
        regBits.clear(offsetC);
      }
    }
  }
    
  public void set(BitSet regBits, String regString)
  {
    int off = offsetA;
    int end = offsetA + length;

    regBits.clear(offsetA,offsetA+length);
    if (triplicated)
    {
      regBits.clear(offsetB,offsetB+length);
      regBits.clear(offsetC,offsetC+length);
    }

    int nibble=0;
    for (int c=regString.length()-1;c>=0;c--)
    {
      if (off >= end) break;
      int reg = Integer.parseInt(regString.substring(c,c+1),16);
      for (int i=0;i<4;i++)
      {
	if (off >= end) break;
        if ((reg>>i & 1) == 1)
        {
          regBits.set(offsetA+4*nibble+i);
          if (triplicated)
          {
            regBits.set(offsetB+4*nibble+i);
            regBits.set(offsetC+4*nibble+i);
          }
        }
        else
        {
          regBits.clear(offsetA+4*nibble+i);
          if (triplicated)
          {
            regBits.clear(offsetB+4*nibble+i);
            regBits.clear(offsetC+4*nibble+i);
	  }
        }
	off++;
      }
      nibble++;
    }
  }
    
}
