/*
GBTX register manipulation
Steve Wotton, 2018-
Cavendish Lab (HEP)

*/

package cbsw.lhcb.pdmdb;

import java.io.*;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.BitSet;
import java.util.Scanner;
import java.util.Formatter;
import javax.swing.JFileChooser;
import javax.swing.filechooser.*;

/**
<p>This class encapsulates a collection of GBTX register descriptors.</p>
**/
public class GbtxRegisterList
{
public GbtxRegisterList(){};

// Read the register array from file and create BitSet.
public static BitSet read(String filename)
{
  int buflen=0;
  byte [] csr = new byte [Gbtx.csrSize];
  try
  {
    Scanner sc = new Scanner(new File(filename));
    while ( sc.hasNext() ) csr[buflen++] = (byte)sc.nextInt( 16 );
    sc.close();
  }
  catch (IOException ioe) { System.out.println("GbtxRegisterList::read(): IOException"); System.out.println(ioe.getMessage()); return (new BitSet()); }

  return BitSet.valueOf(csr);
}
    
// Write the registers as ascii list of bytes.
public static void write(BitSet csrBitSet)
{
  byte [] csr = Arrays.copyOfRange(csrBitSet.toByteArray(),0,Gbtx.csrSize);
  
  JFileChooser chooser = new JFileChooser();
  FileNameExtensionFilter filter = new FileNameExtensionFilter( "GBTX configuration (GBT)", "gbt" );
  chooser.setFileFilter(filter);
  int returnVal = chooser.showOpenDialog(null);
  
  if(returnVal == JFileChooser.APPROVE_OPTION)
  {
    try
    {
      Formatter fm = new Formatter(chooser.getSelectedFile());
      for (byte b : csr) fm.format("%02x\n", b );
      fm.close();
    }
    catch (IOException ex) { System.err.println(ex.getMessage()); }
  }
  return;
}

public static GbtxRegisterDescriptor get (String key){ return Gbtx.map.get(key); }
	
}
