
package cbsw.gui;

import java.io.*;

import java.awt.Color;
import java.awt.Font;
import java.text.SimpleDateFormat;

import org.jfree.data.statistics.HistogramDataset;

public class Histogram
{
  public org.jfree.chart.JFreeChart chart;
  public org.jfree.chart.plot.XYPlot plot;
  public org.jfree.chart.renderer.xy.XYBarRenderer renderer;
  public org.jfree.data.xy.XYSeriesCollection xySeriesCollection;
  static org.jfree.chart.StandardChartTheme theme;
  public int currentIndex;

  static
  {
    theme = new org.jfree.chart.StandardChartTheme("CBSW");
    theme.setChartBackgroundPaint(Color.white);
    theme.setPlotBackgroundPaint(Color.white);
    theme.setDomainGridlinePaint(Color.black);
    theme.setRangeGridlinePaint(Color.black);
    theme.setAxisOffset(new org.jfree.ui.RectangleInsets(5.0, 5.0, 5.0, 5.0));
    theme.setSmallFont(new Font("SansSerif",Font.PLAIN,14));
    theme.setLargeFont(new Font("SansSerif",Font.PLAIN,20));
    theme.setExtraLargeFont(new Font("SansSerif",Font.PLAIN,24));
  }

  public Histogram(String title)
  {
    xySeriesCollection = new org.jfree.data.xy.XYSeriesCollection();

    renderer= new org.jfree.chart.renderer.xy.XYBarRenderer();
    renderer.setBarPainter(new org.jfree.chart.renderer.xy.StandardXYBarPainter());
    plot = new org.jfree.chart.plot.XYPlot(new HistogramDataset(),new org.jfree.chart.axis.NumberAxis(),new org.jfree.chart.axis.NumberAxis(), renderer);
    plot.setOrientation(org.jfree.chart.plot.PlotOrientation.VERTICAL);
    chart = new org.jfree.chart.JFreeChart(plot);

    theme.apply(chart);
    
    plot.setDomainCrosshairVisible(true);
    plot.setRangeCrosshairVisible(true);

    //    LogarithmicAxis y2Axis = new LogarithmicAxis("#");
    //    plot.setRangeAxis(y2Axis);
        
    plot.getRangeAxis().setAutoRange(true);
    //    plot.getRangeAxis().setRange(new org.jfree.data.Range(0.0,20.0));
    plot.getDomainAxis().setAutoRange(true);
    //    plot.getDomainAxis().setRange(-0.5,127.5);
  }

  public org.jfree.chart.JFreeChart getChart(){ return chart; }

  public HistogramDataset getDataset(){ return (HistogramDataset)plot.getDataset(); }

// Show series
  public void show(Comparable key)
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++)
    {
      if (getDataset().getSeriesKey(idx).equals(key))
      {
	renderer.setSeriesVisible(idx,true);
	//	System.err.println("Mean="+mean(key)+" rms="+rms(key));
      }
    }
  }

// Show all
  public void show()
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++) renderer.setSeriesVisible(idx,true);
  }
    
// Hide series
  public void hide(Comparable key)
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++)
    {
      if (getDataset().getSeriesKey(idx).equals(key)) renderer.setSeriesVisible(idx,false);
    }
  }

// Hide all
  public void hide()
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++) renderer.setSeriesVisible(idx,false);
  }

  public void next(int id){currentIndex = id; next();}
	
  public void next()
  {
    HistogramDataset dataset = getDataset();
    int seriesCount = dataset.getSeriesCount();
    currentIndex++;
    if (currentIndex >= dataset.getSeriesCount()) currentIndex = 0;
    if (currentIndex<0) currentIndex = 0;
    hide();
    renderer.setSeriesVisible(currentIndex,true);
  }
    
  public void previous()
  {
    HistogramDataset dataset = getDataset();
    int seriesCount = dataset.getSeriesCount();
    currentIndex--;
    if (currentIndex >= dataset.getSeriesCount()) currentIndex = dataset.getSeriesCount() - 1;
    if (currentIndex<0) currentIndex = dataset.getSeriesCount() - 1;
    hide();
    renderer.setSeriesVisible(currentIndex,true);
  }
    
  public Comparable [] keys()
  {
    org.jfree.data.general.AbstractSeriesDataset dataset = getDataset();
    int seriesCount = dataset.getSeriesCount();
    Comparable [] keyList = new Comparable [seriesCount];
    for (int idx=0;idx<seriesCount;idx++) keyList[idx] = dataset.getSeriesKey(idx);
    return keyList;
  }
    
  public void clear()
  {
      //    getDataset().removeAllSeries();
  }

  public File chooseFile(String fileExtension, String fileDescription)
  {
    File file = null;
    javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
    javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter() { public boolean accept(File f){ return f.getPath().endsWith(fileExtension);} public String getDescription(){return new String(fileDescription);}};

    chooser.setFileFilter( filter );
    int chooserc = chooser.showOpenDialog(null);
    if(chooserc == javax.swing.JFileChooser.APPROVE_OPTION) file = chooser.getSelectedFile();

    return file;
  }

  public void saveAsPNG()
  {
    File file = chooseFile(".png","PNG image");
    if (file != null) saveAsPNG(file);
  }
    
  public void saveAsPNG(File file)
  {
    try { org.jfree.chart.ChartUtilities.saveChartAsPNG(file, chart, 500, 500 ); }
    catch(IOException ioe){}
  }
    
  public void saveAsSVG()
  {
    File file = chooseFile(".svg","SVG image");
    if (file != null) saveAsSVG(file);
  }
    
  public void saveAsSVG(File file)
  {
    org.jfree.graphics2d.svg.SVGGraphics2D g2 = new org.jfree.graphics2d.svg.SVGGraphics2D(500, 500);

    // To do: Draw the histogram
    chart.draw(g2, new java.awt.geom.Rectangle2D.Double(0,0,500,500));
    g2.draw(new java.awt.geom.Rectangle2D.Double(2,2,498,498));
    // Write the file
    String svgElement = g2.getSVGElement();

    try { org.jfree.graphics2d.svg.SVGUtils.writeToSVG(file, svgElement, false ); }
    catch(IOException ioe){}
  }
    
  public class EOFException extends Exception
  {
    public EOFException(){;}
  };

// Compute the mean of the x-values in a series
// The y-value is used as the weight but is usually 1.0
  public double mean(Comparable key)
  {
    double [][] xyArray = xySeriesCollection.getSeries(key).toArray();
    double acc = 0.0;
    double weight = 0.0;
    for (int ix=0;ix<xyArray[0].length;ix++)
    {
      acc += xyArray[0][ix] * xyArray[1][ix];
      weight += xyArray[1][ix];
    }
    return acc/weight;
  }
    
// Compute the rms of the x-values in a series
// The y-value is used as the weight but is usually 1.0
  public double rms(Comparable key)
  {
    double [][] xyArray = xySeriesCollection.getSeries(key).toArray();
    double acc = 0.0;
    double acc2 = 0.0;
    double weight = 0.0;
    for (int ix=0;ix<xyArray[0].length;ix++)
    {
      acc += xyArray[0][ix] * xyArray[1][ix];
      acc2 += (xyArray[0][ix]*xyArray[0][ix]) * xyArray[1][ix];
      weight += xyArray[1][ix];
    }
    return Math.sqrt(acc2/weight - (acc*acc)/(weight*weight));
  }
    
// Read histogram data from comma separated values with header row
// The header row is used as the data series key.
// The XYSeries contains the raw data.
// It is converted to double[] to be added to the HistogramDataset
public void readStream(BufferedReader stream) throws IOException, EOFException
{
   String [] columnHeaders;
   
   try
   {
// First line contains histogram bin description
     String binDescription = stream.readLine();
     if (binDescription == null) {throw new EOFException();} // EOF
     String [] binParameters = binDescription.split(",");
     int bins = Integer.valueOf(binParameters[0]);
     double binMinimum = Double.valueOf(binParameters[1]);
     double binWidth = Double.valueOf(binParameters[2]);
     double binMaximum = binMinimum + bins*binWidth;
     plot.getDomainAxis().setRange(new org.jfree.data.Range(binMinimum,binMaximum));
     //     System.err.println("bins="+bins+" min="+binMinimum+" max="+binMaximum);
// Read the column headers (series keys)
     String headers = stream.readLine();
     if (headers == null) {throw new EOFException();} // EOF
     columnHeaders = headers.split(",");
     for (String columnHeader : columnHeaders) xySeriesCollection.addSeries(new org.jfree.data.xy.XYSeries(columnHeader));

// Read data and add to XYSeries
     String line;
     while ( null != (line = stream.readLine()) )
     {
       String [] columnData = line.split(",");
       if (columnHeaders.length != columnData.length) continue;
       int idx = 0;
       for (String columnHeader : columnHeaders)
       {
	 org.jfree.data.xy.XYSeries xy = xySeriesCollection.getSeries(columnHeader);
	 Double val = Double.valueOf(columnData[idx]);
	 xy.add(val,(Double)1.0);
	 idx++;
       }
     }
// Add Histogram data to chart HistogramDataset
     for (String columnHeader : columnHeaders)
     {
       double [][] xyArray = xySeriesCollection.getSeries(columnHeader).toArray();
       getDataset().addSeries(columnHeader,xyArray[0],bins,binMinimum,binMaximum);
     }
     
   }
   catch ( IOException ioe ){ throw ioe; }
}

public void setPaint(Color col)
{
  int seriesCount = getDataset().getSeriesCount();
  for (int idx=0;idx<seriesCount;idx++) renderer.setSeriesPaint(idx,col);
}
    
public void setPaint(int idx, Color col)
{
  renderer.setSeriesPaint(idx,col);
}
    
}
