
package cbsw.gui;

import java.io.*;

import java.awt.Color;
import java.awt.Font;
import java.text.SimpleDateFormat;

import org.jfree.data.xy.DefaultXYZDataset;

public class Histogram2D
{
  public org.jfree.chart.JFreeChart chart;
  public org.jfree.chart.plot.XYPlot plot;
  public org.jfree.chart.renderer.xy.XYBlockRenderer renderer;
  static org.jfree.chart.StandardChartTheme theme;
  public int currentIndex;

  static org.jfree.chart.renderer.LookupPaintScale paintScale;
  static
  {
    theme = new org.jfree.chart.StandardChartTheme("CBSW");
    theme.setChartBackgroundPaint(Color.white);
    theme.setPlotBackgroundPaint(Color.white);
    theme.setDomainGridlinePaint(Color.black);
    theme.setRangeGridlinePaint(Color.black);
    theme.setAxisOffset(new org.jfree.ui.RectangleInsets(5.0, 5.0, 5.0, 5.0));
    theme.setSmallFont(new Font("SansSerif",Font.PLAIN,14));
    theme.setLargeFont(new Font("SansSerif",Font.PLAIN,20));
    theme.setExtraLargeFont(new Font("SansSerif",Font.PLAIN,24));
    paintScale = createInfernoPaintScale(-5.5,1.5,1.4);
  }

  public Histogram2D(String title)
  {
    renderer = new org.jfree.chart.renderer.xy.XYBlockRenderer();
    plot = new org.jfree.chart.plot.XYPlot(new DefaultXYZDataset(),new org.jfree.chart.axis.NumberAxis(),new org.jfree.chart.axis.NumberAxis(), renderer);
    plot.setOrientation(org.jfree.chart.plot.PlotOrientation.VERTICAL);
    chart = new org.jfree.chart.JFreeChart(plot);

    theme.apply(chart);
    
    plot.setDomainCrosshairVisible(true);
    plot.setRangeCrosshairVisible(true);

    plot.getRangeAxis().setAutoRange(true);
    //    plot.getRangeAxis().setRange(new org.jfree.data.Range(0.0,20.0));
    plot.getDomainAxis().setAutoRange(true);
    //    plot.getDomainAxis().setRange(-0.5,127.5);
    
    // create a paint-scale and its legend
    /*
    ((org.jfree.chart.renderer.xy.XYBlockRenderer)plot.getRenderer()).setPaintScale(paintScale);

    org.jfree.chart.title.PaintScaleLegend psl = new org.jfree.chart.title.PaintScaleLegend(paintScale, new org.jfree.chart.axis.NumberAxis());
    psl.setPosition(org.jfree.ui.RectangleEdge.RIGHT);
    psl.setAxisLocation(org.jfree.chart.axis.AxisLocation.TOP_OR_RIGHT);
    psl.setMargin(50.0, 20.0, 80.0, 0.0);
    chart.addSubtitle(psl);
    */
  }

// create/set a paint-scale and its legend
  public void setPaintScale(double lowerBound, double upperBound, double saturationBound)
  {
    org.jfree.chart.renderer.LookupPaintScale paintScale = createInfernoPaintScale(lowerBound, upperBound, saturationBound);

    ((org.jfree.chart.renderer.xy.XYBlockRenderer)plot.getRenderer()).setPaintScale(paintScale);

    org.jfree.chart.title.PaintScaleLegend psl = new org.jfree.chart.title.PaintScaleLegend(paintScale, new org.jfree.chart.axis.NumberAxis());
    psl.setPosition(org.jfree.ui.RectangleEdge.RIGHT);
    psl.setAxisLocation(org.jfree.chart.axis.AxisLocation.TOP_OR_RIGHT);
    psl.setMargin(50.0, 20.0, 80.0, 0.0);
    chart.addSubtitle(psl);
  }

  public org.jfree.chart.JFreeChart getChart(){ return chart; }

  public DefaultXYZDataset getDataset(){ return (DefaultXYZDataset)plot.getDataset(); }
    /*
// Show series
  public void show(Comparable key)
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++)
    {
      if (getDataset().getSeriesKey(idx).equals(key))
      {
	renderer.setSeriesVisible(idx,true);
	System.err.println("Mean="+mean(key)+" rms="+rms(key));
      }
    }
  }

// Show all
  public void show()
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++) renderer.setSeriesVisible(idx,true);
  }
    
// Hide series
  public void hide(Comparable key)
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++)
    {
      if (getDataset().getSeriesKey(idx).equals(key)) renderer.setSeriesVisible(idx,false);
    }
  }

// Hide all
  public void hide()
  {
    int seriesCount = getDataset().getSeriesCount();
    for (int idx=0;idx<seriesCount;idx++) renderer.setSeriesVisible(idx,false);
  }

  public void next(int id){currentIndex = id; next();}
	
  public void next()
  {
    HistogramDataset dataset = getDataset();
    int seriesCount = dataset.getSeriesCount();
    currentIndex++;
    if (currentIndex >= dataset.getSeriesCount()) currentIndex = 0;
    if (currentIndex<0) currentIndex = 0;
    hide();
    renderer.setSeriesVisible(currentIndex,true);
  }
    
  public void previous()
  {
    HistogramDataset dataset = getDataset();
    int seriesCount = dataset.getSeriesCount();
    currentIndex--;
    if (currentIndex >= dataset.getSeriesCount()) currentIndex = dataset.getSeriesCount() - 1;
    if (currentIndex<0) currentIndex = dataset.getSeriesCount() - 1;
    hide();
    renderer.setSeriesVisible(currentIndex,true);
  }
    */  
  public Comparable [] keys()
  {
    org.jfree.data.general.AbstractSeriesDataset dataset = getDataset();
    int seriesCount = dataset.getSeriesCount();
    Comparable [] keyList = new Comparable [seriesCount];
    for (int idx=0;idx<seriesCount;idx++) keyList[idx] = dataset.getSeriesKey(idx);
    return keyList;
  }
    
  public void clear()
  {
      //    getDataset().removeAllSeries();
  }

  public File chooseFile(String fileExtension, String fileDescription)
  {
    File file = null;
    javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
    javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter() { public boolean accept(File f){ return f.getPath().endsWith(fileExtension);} public String getDescription(){return new String(fileDescription);}};

    chooser.setFileFilter( filter );
    int chooserc = chooser.showOpenDialog(null);
    if(chooserc == javax.swing.JFileChooser.APPROVE_OPTION) file = chooser.getSelectedFile();

    return file;
  }

  public void saveAsPNG()
  {
    File file = chooseFile(".png","PNG image");
    if (file != null) saveAsPNG(file);
  }
    
  public void saveAsPNG(File file)
  {
    try { org.jfree.chart.ChartUtilities.saveChartAsPNG(file, chart, 500, 500 ); }
    catch(IOException ioe){}
  }
    
  public void saveAsSVG()
  {
    File file = chooseFile(".svg","SVG image");
    if (file != null) saveAsSVG(file);
  }
    
  public void saveAsSVG(File file)
  {
    org.jfree.graphics2d.svg.SVGGraphics2D g2 = new org.jfree.graphics2d.svg.SVGGraphics2D(500, 500);

    // To do: Draw the histogram
    chart.draw(g2, new java.awt.geom.Rectangle2D.Double(0,0,500,500));
    g2.draw(new java.awt.geom.Rectangle2D.Double(2,2,498,498));
    // Write the file
    String svgElement = g2.getSVGElement();

    try { org.jfree.graphics2d.svg.SVGUtils.writeToSVG(file, svgElement, false ); }
    catch(IOException ioe){}
  }
    
  public class EOFException extends Exception
  {
    public EOFException(){;}
  };

public void readStream(BufferedReader stream) throws IOException, EOFException
{
   org.jfree.data.xy.DefaultXYZDataset dataset = getDataset();
   java.util.ArrayList<String[]> xyzList = new java.util.ArrayList<String[]>();

   try
   {
       //     stream = new BufferedReader(new FileReader("xyz.dat")); // test
// First line is the series key
     String key = stream.readLine();
     if (key == null){throw new EOFException();} // EOF
// Read XYZ data into array and add to dataset
     String line;
     while ( null != (line = stream.readLine()) )
     {
       String [] columnData = line.split(",");
       if (columnData.length < 4) continue;

       xyzList.add(columnData);
     }
     double [][] xyzArray = new double [3][xyzList.size()];
     int idx = 0;
     for(String [] xyz : xyzList)
     {
       xyzArray[0][idx] = Double.valueOf(xyz[0]);
       xyzArray[1][idx] = Double.valueOf(xyz[1]);
       //       xyzArray[2][idx] = Double.valueOf(xyz[3])/Double.valueOf(xyz[2]);
       if ( (Integer.valueOf(xyz[3]) == 0xffff) || (Integer.valueOf(xyz[2]) == 0) ) xyzArray[2][idx] = 1.45;
       else if ( Integer.valueOf(xyz[3]) == 0 ) xyzArray[2][idx] = -10.0;
       else xyzArray[2][idx] = Math.log10(Double.valueOf(xyz[3])/Double.valueOf(xyz[2]));
       idx++;
     }
     dataset.addSeries(key,xyzArray);
   }
   catch ( IOException ioe ){ throw ioe; }
}

    static org.jfree.chart.renderer.LookupPaintScale createInfernoPaintScale(double lowerBound, double upperBound, double saturationBound)
{
// https://www.kennethmoreland.com/color-advice/ (Inferno)
  Color [] colors =
  {
    new Color( 1, 0, 4 ),
    new Color( 6, 3, 19 ),
    new Color( 16, 7, 37 ),
    new Color( 28, 9, 57 ),
    new Color( 42, 8, 77 ),
    new Color( 56, 4, 93 ),
    new Color( 70, 2, 103 ),
    new Color( 83, 4, 108 ),
    new Color( 95, 10, 110 ),
    new Color( 107, 16, 111 ),
    new Color( 119, 22, 110 ),
    new Color( 131, 28, 108 ),
    new Color( 143, 33, 105 ),
    new Color( 155, 38, 101 ),
    new Color( 167, 44, 96 ),
    new Color( 179, 50, 89 ),
    new Color( 190, 57, 81 ),
    new Color( 201, 65, 73 ),
    new Color( 211, 74, 64 ),
    new Color( 220, 85, 54 ),
    new Color( 228, 97, 44 ),
    new Color( 234, 110, 34 ),
    new Color( 239, 123, 22 ),
    new Color( 242, 138, 10 ),
    new Color( 244, 153, 1 ),
    new Color( 245, 169, 6 ),
    new Color( 243, 185, 23 ),
    new Color( 241, 201, 44 ),
    new Color( 237, 217, 68 ),
    new Color( 233, 233, 97 ),
    new Color( 234, 246, 131 ),
    new Color( 245, 255, 163 )
  };

  org.jfree.chart.renderer.LookupPaintScale paintScale = new org.jfree.chart.renderer.LookupPaintScale(lowerBound, upperBound, Color.black);
	 
  double lowValue = lowerBound;
  double increment = (saturationBound-lowerBound)/32.0;
  for (Color col : colors) { paintScale.add( lowValue, col ); lowValue += increment; }
  paintScale.add( saturationBound, new Color( 245, 255, 163 ));

  return paintScale;
}

}
