package cbsw.lhcb.chimaera2;

import java.io.File;
import javax.swing.*;
import javax.xml.bind.annotation.*;

/**
<p>Chimaera2v3 basic CSR Panel.</p>
<p>Access through Adept Depp USB interface.</p>
@author Steve Wotton, Cavendish Lab (HEP), 2015-
@see https://twiki.cern.ch/twiki/bin/view/LHCb/RichChimaera2 
*/
@XmlRootElement(name="deppGui")
@XmlAccessorType(XmlAccessType.NONE)
public class JChimaera2Depp
{
Timer timer;

@XmlElementWrapper(name="deppTables")
@XmlElement(name="deppTable")
java.util.ArrayList<cbsw.RegisterIoTable> tableList;
@XmlElement(name="miim")
MiimDepp miim;
public cbsw.RegisterIo io;
JButton phyLinkJB, phy10JB, phy100JB, phy1000JB, phyFullDuplexJB, phyHalfDuplexJB;
    
public JChimaera2Depp(){};

public void init(cbsw.RegisterIo io)
{
  this.io = io;
  for (cbsw.RegisterIoTable dT : tableList ) dT.init(io);

  try
  {
    miim.init(io);

  //  miim.writeRegister(0,0x1140);
  // Force the clock master/slave mode 
  // miim.writeRegister(9,0x1e00); // MASTER 0x1e00, SLAVE 0x1600
  //  System.out.println("[000] "+miim.readRegister(0));
  //  System.out.println("[009] "+miim.readRegister(9));

  // Adjust the RX & TX clock skew (extended register)
  //  miim.writeRegister(12,0xf7f3);
  //  miim.writeRegister(11,0x8104);

// Soft reset of PHY
//  miim.writeRegister(31,0x0202); // Assert
//  miim.writeRegister(31,0x0200); // Deassert

  // Create a timer

    timer = new Timer( 1000, al->{if (miim != null) miim.update();} );
    timer.setInitialDelay(1000);
    timer.setCoalesce( false );
    timer.start();
  }
  catch (NullPointerException npe){}

  return;
}

public JMenu makeMainMenu(String title)
{
  JMenu menuBarMenu = new JMenu(title);

  JMenuItem menuItem = new JMenuItem("Exit");
  menuItem.addActionListener(ae -> { io.close(); System.exit(0); });
  menuBarMenu.add(menuItem);

  return menuBarMenu;
}

public JPanel makePanel(String typeName)
{
  JPanel jP = new JPanel();
  jP.setLayout(new java.awt.BorderLayout());

  jP.add(cbsw.RegisterIoTable.makePanel(tableList),java.awt.BorderLayout.NORTH);

  if (miim != null) jP.add(miim.makePanel(),java.awt.BorderLayout.SOUTH);

  return jP;
}

public static void main( String[] args ) {

// The pause here is for the splash screen which closes as soon as the first swing window is made visible.

  try { Thread.sleep(1000); } catch ( Exception ex ) { System.err.println("JChimaera2Depp.main(): " + ex ); }

// Force SwingApplet to come up in the System L&F

  String laf = UIManager.getSystemLookAndFeelClassName();
  try { UIManager.setLookAndFeel(laf); }
  catch (UnsupportedLookAndFeelException exc){ System.err.println("JChimaera2Depp.main(): UnsupportedLookAndFeel: " + laf); }
  catch (Exception exc){ System.err.println("JChimaera2Depp.main(): Error loading " + laf + ": " + exc); }

  String [] deviceTokens = cbsw.adept.JniAdeptDepp.selectConnectedDevice();
  if (deviceTokens==null || deviceTokens.length < 2)
  {
    System.out.println("No connected device found. Exiting...");
    System.exit(0);
  }
  System.out.println("Connecting to "+deviceTokens[1]);
  cbsw.adept.JniAdeptDepp mainDepp = new cbsw.adept.JniAdeptDepp(deviceTokens[1]);

  try { mainDepp.open(); cbsw.adept.JniAdeptDepp.depp.enable(); } catch (Exception ex) {System.err.println("JChimaera2Depp.main(): "+ex);}

  JChimaera2Depp csr;

  java.util.prefs.Preferences prefs = java.util.prefs.Preferences.userNodeForPackage(JChimaera2Depp.class).node("JChimaera2Depp");

// A prefix is added to the local file name to avoid conflict with other GUIs that use Cbsw.unmarshal()
  String localName = "chimaera2-" + deviceTokens[0];
  File file = new File(localName+".xml");
  if (!file.canRead())
  {
    try
    {
      java.net.URL xml = new java.net.URL(prefs.get("unmarshall.xml","https://www.hep.phy.cam.ac.uk/lhcb/Chimaera2/xml/"+deviceTokens[0]+".xml"));
      System.err.println("Unmarshal (URL): "+xml);
      java.io.InputStream is = xml.openStream();
      java.nio.file.Files.copy(is,java.nio.file.Paths.get("www-"+localName+".xml"),java.nio.file.StandardCopyOption.REPLACE_EXISTING);
    }
    catch ( Exception ex ){ System.err.println("JChimaera2Depp.main(): " + ex ); }
    file = new File("www-"+localName+".xml");
  }

  try
  {
    javax.xml.bind.JAXBContext jaxbContext = javax.xml.bind.JAXBContext.newInstance(JChimaera2Depp.class);
    javax.xml.bind.Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
    jaxbUnmarshaller.setEventHandler(new javax.xml.bind.helpers.DefaultValidationEventHandler());
    System.err.println("Unmarshal (local): "+file.getAbsolutePath());
    csr = (JChimaera2Depp) jaxbUnmarshaller.unmarshal(file);
    csr.init(mainDepp);

// Put it in a JFrame
    
    JFrame frame = cbsw.gui.GuiJPanel.makeFrame(csr.makePanel(deviceTokens[0]),"JChimaera2Depp");

    frame.getJMenuBar().add(csr.makeMainMenu("Main"));
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.pack();
    frame.setVisible(true);
  }
  catch (Exception ex) { System.err.println("JChimaera2Depp.main(): " + ex ); ex.printStackTrace(); }

  return;
}

}
