package cbsw.lhcb.chimaera2;

import java.awt.*;
import javax.swing.*;
import java.util.BitSet;
import java.util.ArrayList;
import java.util.HashMap;
import cbsw.*;
import javax.xml.bind.annotation.*;

/**
<p>MIIM register manipulation.</p>
@author Steve Wotton, Cavendish Lab (HEP), 2018-
*/
@XmlAccessorType(XmlAccessType.NONE)
public class MiimDepp
{
RegisterIo io;
@XmlAttribute
int baseAddress;
@XmlAttribute
int csrSize;
@XmlElement(name="register")
@XmlElementWrapper(name="registers")
ArrayList<RegisterDescriptor> registerDescriptors;
HashMap<String,RegisterDescriptor> registerMap;
byte [] csrArray;
BitSet csrBitSet;

JButton phyLinkJB, phy10JB, phy100JB, phy1000JB, phyFullDuplexJB, phyHalfDuplexJB;
JLabel modeSelectJL;

public MiimDepp(){};

public void init(RegisterIo io)
{
  this.io = io;
  csrArray = new byte [csrSize];

// Make a HashMap for the register descriptors

  registerMap = new HashMap<String,RegisterDescriptor>();
  for (RegisterDescriptor reg : registerDescriptors){ registerMap.put(reg.getName(),reg); }

  return;
}

/**
<p>Reads the register array from hardware into local cache.</p>
<p>For MIIM registers we must read 16-bit regs one at a time</p>
*/
public void cache()
{
  byte [] miim = new byte [2];
  for (int addr=0;addr<(csrSize/2);addr++)
  {
    putMiimAddress(addr);
    miim = getMiimData();
    csrArray[2*addr] = miim[0]; csrArray[2*addr+1] = miim[1];
  }
  csrBitSet =  BitSet.valueOf(csrArray);

  return;
}

void update()
{
  cache();

  String modeSelect;

  if (get("phy.select.auto-negotiation").equals("01"))
  {
    modeSelect = "Requested mode: Auto";
  }
  else
  {
    if (get("phy.select.duplex").equals("01"))
    {
      modeSelect = "Requested mode: Full duplex";
    }
    else
    {
      modeSelect = "Requested mode: Half duplex";
    }

    if (get("phy.select.speed.msb").equals("01"))
    {
      if (get("phy.select.speed.lsb").equals("01"))
      {
	modeSelect = modeSelect + ", Reserved";
      }
      else
      {
	modeSelect = modeSelect + ", 1000Mb/s";
      }
    }
    else
    {
      if (get("phy.select.speed.lsb").equals("01"))
      {
	modeSelect = modeSelect + ", 100Mb/s";
      }
      else
      {
	modeSelect = modeSelect + ", 10Mb/s";
      }
    }
  }

  modeSelectJL.setText(modeSelect);

  if (get("phy.link-status").equals("01"))
  {
    phyLinkJB.setBackground(Color.green);
  }
  else
  {
    phyLinkJB.setBackground(Color.lightGray);
  }

  if (get("phy.duplex").equals("01"))
  {
    phyFullDuplexJB.setBackground(Color.green);
    phyHalfDuplexJB.setBackground(Color.lightGray);
  }
  else
  {
    phyFullDuplexJB.setBackground(Color.lightGray);
    phyHalfDuplexJB.setBackground(Color.green);
  }

  if (get("phy.10baset").equals("01"))
  {
    phy10JB.setBackground(Color.green);
  }
  else
  {
    phy10JB.setBackground(Color.lightGray);
  }

  if (get("phy.100basetx").equals("01"))
  {
    phy100JB.setBackground(Color.green);
  }
  else
  {
    phy100JB.setBackground(Color.lightGray);
  }

  if (get("phy.1000baset").equals("01"))
  {
    phy1000JB.setBackground(Color.green);
  }
  else
  {
    phy1000JB.setBackground(Color.lightGray);
  }

  // DeppTable statusTable = tableList.get(0);
  // System.out.println(statusTable.get("chimaera.dna"));
  
}

public String get(String key){ return registerMap.get(key).get(csrBitSet); }

public void putMiimAddress(int addr)
{
  byte [] buf = new byte [1];
  java.nio.ByteBuffer bb = java.nio.ByteBuffer.wrap(buf);
  bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);

  int miimAddr = addr<<1;
  
  try
  {
// Address LSB
    bb.clear();
    bb.put((byte)miimAddr);
    bb.flip();
    io.put(baseAddress,bb);

// Address MSB
    bb.clear();
    bb.put((byte)(miimAddr>>8));
    bb.flip();
    io.put(baseAddress+1,bb);
    
  } catch (RegisterIoException re){System.err.println("MiimDepp.putMiimAddress(): RegisterIo exception:"+re.getMessage());}
}
    
public void putMiimData(int data)
{
  byte [] buf = new byte [2];
  java.nio.ByteBuffer bb = java.nio.ByteBuffer.wrap(buf);
  bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);

  try
  {
// 16bit data
    bb.clear();
    bb.putShort((short)data);
    bb.flip();
    io.put(baseAddress+2,bb);
  } catch (RegisterIoException re){System.err.println("MiimDepp.putMiimData(): RegisterIo exception:"+re.getMessage());}
}

public byte [] getMiimData()
{
  byte [] buf = new byte [2];
    
  try
  {
// 16bit data
    io.get(baseAddress+2,buf);
  } catch (RegisterIoException re){System.err.println("MiimDepp.getMiimData(): RegisterIo exception:"+re.getMessage());}

  return buf;
}

public void writeRegister(int addr, int mode)
{
  putMiimAddress(addr);
  putMiimData(mode); 
}

public String readRegister(int addr)
{
  putMiimAddress(addr);
  byte [] miim = getMiimData();
  return String.format("%02X%02X", miim[1],miim[0]);
}

public JPanel makePanel()
{
  JPanel jP = new JPanel();
  modeSelectJL = new JLabel("Requested mode: Unknown");
  jP.add(modeSelectJL);
  phyLinkJB = new JButton("Link");
  phyLinkJB.setEnabled(false);
  jP.add(phyLinkJB);
  phyFullDuplexJB = new JButton("Full Duplex");
  phyFullDuplexJB.setEnabled(false);
  jP.add(phyFullDuplexJB);
  phyHalfDuplexJB = new JButton("Half Duplex");
  phyHalfDuplexJB.setEnabled(false);
  jP.add(phyHalfDuplexJB);
  phy10JB = new JButton("10Base-T");
  phy10JB.setEnabled(false);
  jP.add(phy10JB);
  phy100JB = new JButton("100Base-TX");
  phy100JB.setEnabled(false);
  jP.add(phy100JB);
  phy1000JB = new JButton("1000Base-T");
  phy1000JB.setEnabled(false);
  jP.add(phy1000JB);

  return jP;
}


}
