package cbsw.lhcb;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.gui.*;

public class JM25Pxx extends GuiJPanel implements ActionListener
{
  final int pageSize=256;
  final int txPages=4;

  M25PxxIo m25;

  javax.swing.Timer timer;

  JButton programButton;
    
  JTextField progressTF;
  JTextField statusTF;
  JTextField rawTF;

  JTextArea textArea;
  JScrollPane textSP;
    
ActionListener timerAL = new ActionListener () { public void actionPerformed(ActionEvent e)
{
  // Read the progress register
  byte [] buf = new byte [2];
  m25.getRxBuffer(0,buf);
  statusTF.setText(String.format("%1$02X%2$02X",buf[1],buf[0]));
}
};

  public JM25Pxx( JMenuBar mB, cbsw.RegisterIo io )
  {
    super(Preferences.userNodeForPackage(JM25Pxx.class).node("M25Pxx"),"");
    
    JMenu menuBarMenu = new JMenu("Main");

    JMenuItem menuItem = new JMenuItem("Exit");
    menuItem.addActionListener( l->System.exit(0) );
    menuBarMenu.add(menuItem);

    mB.add(menuBarMenu);

    m25 = new M25PxxIo(io);

    setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    programButton = new JButton("Program");
    programButton.addActionListener(this);
    programButton.setActionCommand("program");
    programButton.setBackground(Color.green);
    programButton.setToolTipText("Green: Ready to program; Orange: Write/erase in progress.");
    gbC.gridwidth = 1;
    add(programButton,gbC);

    JPanel dJp = new JPanel();
    dJp.setBorder( BorderFactory.createTitledBorder("Progress") );
    progressTF = new JTextField(20);
    progressTF.setToolTipText("Address (hex) of last page written.");
    dJp.add(progressTF);
    statusTF = new JTextField(4);
    statusTF.setToolTipText("Status (hex) of flash memory.");
    dJp.add(statusTF);

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    add(dJp,gbC);

    rawTF = new JTextField("000000",20);
    rawTF.setToolTipText("Address (hex) of read/write operation.");
    gbC.gridwidth = 1;
    add(rawTF,gbC);

    JButton button = new JButton("Show");
    button.addActionListener(this);
    button.setActionCommand("show");
    gbC.gridwidth = 1;
    add(button,gbC);

    button = new JButton("READ");
    button.addActionListener(l->m25.read(Integer.parseInt(rawTF.getText(),16)));
    button.setActionCommand("read");
    gbC.gridwidth = 1;
    add(button,gbC);

    button = new JButton("Sector Erase");
    button.addActionListener(l->m25.sectorErase(Integer.parseInt(rawTF.getText(),16)));
    button.setActionCommand("sector");
    gbC.gridwidth = 1;
    add(button,gbC);

    button = new JButton("PP");
    button.addActionListener(l->m25.pp(Integer.parseInt(rawTF.getText(),16)));
    button.setActionCommand("pp");
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    add(button,gbC);

    button = new JButton("Bulk Erase");
    button.addActionListener(l->m25.bulkErase(true,progressTF));
    button.setActionCommand("erase");
    gbC.gridwidth = 1;
    add(button,gbC);

    button = new JButton("WREN");
    button.addActionListener(this);
    button.setActionCommand("wren");
    gbC.gridwidth = 1;
    add(button,gbC);

    button = new JButton("WRDI");
    button.addActionListener(this);
    button.setActionCommand("wrdi");
    gbC.gridwidth = 1;
    add(button,gbC);

    button = new JButton("RDSR");
    button.addActionListener(this);
    button.setActionCommand("rdsr");
    gbC.gridwidth = 1;
    add(button,gbC);

    button = new JButton("RDID");
    button.addActionListener(this);
    button.setActionCommand("rdid");
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    add(button,gbC);

    textArea = new JTextArea(4,10); // Minimum width. Will expand to fill available width.
    textArea.setEditable(false);
    textSP = new JScrollPane(textArea);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    gbC.fill = GridBagConstraints.HORIZONTAL;
    add(textSP,gbC);

    m25.setTextArea(textArea);
    
    // Create a timer

    timer = new javax.swing.Timer( 1000, timerAL );
    timer.addActionListener( timerAL );
    timer.setInitialDelay(1000);
    timer.setCoalesce( false );
    timer.start();

  }

  public void actionPerformed( ActionEvent e )
  {
    Object ob = e.getSource();
    String command = e.getActionCommand();

    if ( command.equals("wren") )
    {
      m25.clear();
      m25.append(M25PxxIo.WREN,(short)0x0001);
      m25.op(0,true);
    }
    else if ( command.equals("wrdi") )
    {
      m25.clear();
      m25.append(M25PxxIo.WRDI,(short)0x0001);
      m25.op(0,true);
    }
    else if ( command.equals("rdsr") )
    {
      m25.clear();
      m25.append(M25PxxIo.RDSR,(short)0x0002, new byte[1]);
      m25.op(0,true);
    }
    else if ( command.equals("rdid") )
    {
      m25.clear();
      m25.append(M25PxxIo.RDID,(short)0x0015, new byte[20]);
      m25.op(0,true);
    }
    else if ( command.equals("show") )
    {
      byte [] buf = new byte[64];
      m25.showRxBuffer( 0,buf );
    }
    else if ( command.equals("program") )
    {
      ProgramTask programTask = new ProgramTask();
      programTask.execute();
    }

  }
    
// Load the bin file and return byte array with file contents
  byte [] loadBin()
  {
    File file = cbsw.Cbsw.chooseFile(prefs.get("bin.path","."),new FileNameExtensionFilter( "Flash bin files (BIN)", "bin" ));
    prefs.put("bin.path",file.getParent());
    int length = (int)file.length();
    byte [] buf = new byte [length];
      
    try
    {
      FileInputStream fi = new FileInputStream(file);
      fi.read( buf );
      fi.close();
    }
    catch (IOException ioe) { System.out.println("loadBin(): IOException"); System.out.println(ioe.getMessage()); }
    
    m25.showText("loadBin(): Buffered "+buf.length+" bytes from "+file);

    return buf;
  }
    
  class ProgramTask extends javax.swing.SwingWorker<Boolean, String>
  {
    @Override
    protected Boolean doInBackground()
    {
      byte [] buf = loadBin();
      if ( buf == null ) return Boolean.valueOf(false);

      programButton.setBackground(Color.orange);

      m25.bulkErase(true,progressTF);

      programButton.setBackground(Color.yellow);

      m25.program(buf,progressTF);

      return Boolean.valueOf(true);
    }
      
    @Override
    protected void done(){ try { if (get()) programButton.setBackground(Color.green); else programButton.setBackground( Color.orange ); } catch (Exception e){} }

    @Override
    protected void process( java.util.List<String> chunks ){for ( String pass : chunks ) progressTF.setText(pass);}
  }
    
public static void main( String[] args ) {

// The pause here is for the splash screen which closes as soon as the first swing window is made visible.

  try
  {
    Thread.sleep(1000);
  }
  catch ( InterruptedException inte )
  {
    System.err.println("Warning: " + inte.getMessage() );
  }

// Force SwingApplet to come up in the System L&F

  String laf = UIManager.getSystemLookAndFeelClassName();
  try 
  {
    UIManager.setLookAndFeel(laf);
  }
  catch (UnsupportedLookAndFeelException exc)
  {
    System.err.println("Warning: UnsupportedLookAndFeel: " + laf);
  }
  catch (Exception exc)
  {
    System.err.println("Error loading " + laf + ": " + exc);
  }

  JFrame frame = new JFrame( "M25PXX" );
  frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

  Container cP = frame.getContentPane();
  cP.setLayout(new BorderLayout());

  JMenuBar mB = new JMenuBar();
  frame.setJMenuBar( mB );

  String [] deviceTokens = cbsw.adept.JniAdeptDepp.selectConnectedDevice();
  if (deviceTokens==null || deviceTokens.length < 2)
  {
    System.out.println("No connected device found. Exiting...");
    System.exit(0);
  }
  System.out.println("Connecting to "+deviceTokens[1]);
  cbsw.adept.JniAdeptDepp mainDepp = new cbsw.adept.JniAdeptDepp(deviceTokens[1]);

  try { mainDepp.open(); cbsw.adept.JniAdeptDepp.depp.enable(); } catch (cbsw.adept.JniAdeptDepp.DeppException de) {System.err.println("JM25Pxx.main(): Depp exception. "+de.getDescription());}
    
  JM25Pxx csr = new JM25Pxx( mB, mainDepp );
  cP.add( csr, BorderLayout.NORTH );

  frame.pack();
  frame.setVisible(true);

  return;
}

}
