/*
PDMDB basic CSR Panel
Access through Adept Depp USB interface
Steve Wotton, 2016-
Cavendish Lab (HEP)
*/

package cbsw.lhcb.pdmdb;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.*;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.JniAdeptDepp;
import cbsw.gui.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
 
/**
<p>PDMDB GUI main class.</p>
**/
public class JPDMDBDepp
{
JniAdeptDepp depp;
MuDaq muDaq;
JFrame programJF;
JFrame muDaqJF;
TesterSequence testerSequence;
JFrame testerJF;
Preferences prefs;
JTextField fusingTF;

/**
<p>Constructor - called from main() to build GUI.</p>
**/
  public JPDMDBDepp()
  {
    prefs = Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp");

    String [] deviceTokens = JniAdeptDepp.selectConnectedDevice();
    if (deviceTokens==null || deviceTokens.length < 2)
    {
      System.out.println("No connected device found. Exiting...");
      System.exit(0);
    }
    System.out.println("Connecting to "+deviceTokens[1]);
    depp = new JniAdeptDepp(deviceTokens[1]);

    try { depp.open(); JniAdeptDepp.depp.enable(); } catch (DeppException de) {System.err.println("JPDMDBDepp.JPDMDBDepp(): Depp exception. "+de.getDescription());}

    muDaq = (MuDaq) Cbsw.unmarshal(prefs.get("unmarshal.xml","https://www.hep.phy.cam.ac.uk/lhcb/muDAQ/xml/mudaq.xml"),MuDaq.class);
    MuDaq.setStaticMuDaq(muDaq);
    muDaq.init(depp,prefs);
    muDaqJF = GuiJPanel.makeFrame(muDaq.makePanel(),"muDaq");
    muDaqJF.pack();
    
    testerSequence = (TesterSequence)Cbsw.unmarshal("https://www.hep.phy.cam.ac.uk/lhcb/PDMDB/xml/testerSequence.xml",TesterSequence.class);
/*
    testerJF = GuiJPanel.makeFrame(testerSequence.makePanel(),"Tester sequence");
    testerJF.pack();
    testerJF.setVisible(true);
*/
    cbsw.adept.xilinx.XilinxDjtg xilinx = new cbsw.adept.xilinx.XilinxDjtg(1,0);
    xilinx.setConnectedDevice(depp);
    programJF = GuiJPanel.makeFrame(xilinx.makePanel(),"Xilinx programmer");
    programJF.pack();
  }

/**
<p>Convenience function that builds a main panel.</p>
@return The panel.
**/
  public JPanel makePanel()
  {

    JPanel jP = new JPanel();
    jP.setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(2,2,2,2);
    gbC.weightx = 1.0f;
    //    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JPanel fusingJP = new JPanel();
    fusingJP.setBorder(BorderFactory.createTitledBorder("Fusing token"));
    fusingTF = new JTextField(prefs.get("fusing-expiry","2000-01-01T12:00:00"));
    fusingTF.setEditable(false);
    fusingJP.add(fusingTF);
    JButton extendJB = new JButton("Extend");
    extendJB.addActionListener(ae->extendFusingToken());
    fusingJP.add(extendJB);
    JButton discardJB = new JButton("Discard");
    discardJB.addActionListener(ae->discardFusingToken());
    fusingJP.add(discardJB);
    
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jP.add(fusingJP,gbC);

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jP.add(testerSequence.makePanel(),gbC);

    return jP;

  }

/**
<p>Extends token to allow e-fusing. Sets token expiry 9 hours in the future.</p>
**/
  public void extendFusingToken()
  {
    prefs.put("fusing-expiry", java.time.LocalDateTime.now().plusHours(9).toString());
    fusingTF.setText(prefs.get("fusing-expiry","2000-01-01T12:00:00"));
  }

/**
<p>Discards token to prevent e-fusing. Sets token expiry 1 hour into the past.</p>
**/
  public void discardFusingToken()
  {
    prefs.put("fusing-expiry", java.time.LocalDateTime.now().minusHours(1).toString());
    fusingTF.setText(prefs.get("fusing-expiry","2000-01-01T12:00:00"));
  }

ActionListener socketUrlAL = new ActionListener () { public void actionPerformed(ActionEvent e)
{
  String url = JOptionPane.showInputDialog("Please enter socket URL",prefs.get("mtb.socket","http://:7999"));
  if (url != null) prefs.put("mtb.socket",url);
  return;
}
};

/**
<p>Convenience function that builds a main menu.</p>
@param title The menu title.
@return The menu.
**/
  public JMenu makeMainMenu(String title)
  {
    JMenu menuBarMenu = new JMenu(title);

// Add some standard DEPP menuitems
//    JniAdeptDepp.addMenuItems(menuBarMenu);

// Add GUI Import/Export menuitems
    GuiPreferences.addMenuItems(menuBarMenu);

    menuBarMenu.addSeparator();

    JMenuItem menuItem = new JMenuItem("MuDaq...");
    menuItem.addActionListener(a->muDaqJF.setVisible(true));
    menuBarMenu.add(menuItem);
    
    menuItem = new JMenuItem("MuDaq status...");
    menuItem.addActionListener(a->muDaq.getStatusJF().setVisible(true));
    menuBarMenu.add(menuItem);
    
    menuItem = new JMenuItem("Master GBTX...");
    menuItem.addActionListener(a->muDaq.getMasterConfigurationJF().setVisible(true));
    menuBarMenu.add(menuItem);
    
    menuItem = new JMenuItem("Master GBTX (I2C)...");
    menuItem.addActionListener(a->muDaq.getMasterI2cConfigurationJF().setVisible(true));
    menuBarMenu.add(menuItem);
    
    menuItem = new JMenuItem("Master link...");
    menuItem.addActionListener(a->muDaq.getMasterLinkJF().setVisible(true));
    menuBarMenu.add(menuItem);
    
    menuItem = new JMenuItem("Tester commands...");
    menuItem.addActionListener(a->muDaq.getTesterJF().setVisible(true));
    menuBarMenu.add(menuItem);
    
    menuBarMenu.addSeparator();

    menuItem = new JMenuItem("Program FPGA...");
    menuItem.addActionListener(a->programJF.setVisible(true));
    menuBarMenu.add(menuItem);
    
    menuBarMenu.addSeparator();

    menuItem = new JMenuItem("Set MTB socket...");
    menuItem.addActionListener(socketUrlAL);
    menuBarMenu.add(menuItem);
    
    menuBarMenu.addSeparator();
    
    menuItem = new JMenuItem("Exit");
    menuItem.addActionListener(a->System.exit(0));
    menuBarMenu.add(menuItem);
    
    return menuBarMenu;
  }

/**
<p>Launches the GUI.</p>
@param args The command line arguments
**/
public static void main( String[] args ) {

// The pause here is for the splash screen which closes as soon as the first swing window is made visible.

  try
  {
    Thread.sleep(1000);
  }
  catch ( InterruptedException inte )
  {
    System.err.println("Warning: " + inte.getMessage() );
  }

// Force SwingApplet to come up in the System L&F

//  String laf = UIManager.getSystemLookAndFeelClassName();
  String laf = UIManager.getCrossPlatformLookAndFeelClassName();
  try 
  {
    UIManager.setLookAndFeel(laf);
  }
  catch (UnsupportedLookAndFeelException exc)
  {
    System.err.println("Warning: UnsupportedLookAndFeel: " + laf);
  }
  catch (Exception exc)
  {
    System.err.println("Error loading " + laf + ": " + exc);
  }

// Minor tweaks to the LAF
  
  UIManager.put("Button.background",new Color(220,220,220));
  UIManager.put("TextField.background",new Color(240,240,240));
  UIManager.put("TextArea.background",new Color(240,240,240));
  UIManager.put("Table.background",new Color(240,240,240));
  
  JPDMDBDepp pdmdbDepp = new JPDMDBDepp();

  JFrame frame = GuiJPanel.makeFrame(pdmdbDepp.makePanel(),"JPDMDBDepp");
  frame.getJMenuBar().add(pdmdbDepp.makeMainMenu("Main"));
  frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  frame.pack();
  frame.setVisible(true);


  return;
}

}
