package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.lang.reflect.*;

/**
<p>This class encapsulates a static method invoked through Java reflection.</p>
*/
@XmlAccessorType(XmlAccessType.NONE)
public class Command
{
/**<p>The label that appears on the button associated with the command.</p>*/
  @XmlAttribute
  String label;
/**<p>The method to call.</p>*/
  @XmlAttribute
  private String targetMethod;
/**<p>The static method's class.</p>*/
  @XmlAttribute
  private String targetClass;
/**<p>Flag to control behaviour on a FAIL response.</p>*/
  @XmlAttribute
  private boolean stopOnFail;
/**<p>A flag to signal that a sequence should pause after the current step.</p>*/
  @XmlAttribute
  private boolean pause;
/**<p>The target method parameters.</p>*/
  @XmlAttribute
  private String data;
  @XmlElement(name="response")
  private java.util.ArrayList<TesterResponse> responseList;
    
  private javax.swing.JTextField statusTF;
  private javax.swing.JButton button;
  
/**
<p>No-argument constructor called by the unmarshaller.</p>
*/
  public Command(){}

/**
<p>Calls the static exec method of the target class.</p>
<p>Allows multiple groups of target method args (separated by ? delimiters) that are applied repeatedly to the same method.</p>
@throws Exception Thrown for various conditions.
*/
  public void go() throws Exception
  {
    try
    {

// Split the groups of target method args at the ? character
    
      String [] dataTokenised = (data==null) ? new String []{""} : data.split("\\?");
  
      if (button!=null) button.setBackground(java.awt.Color.YELLOW); // Command send

      for (String targetMethodData : dataTokenised)
      {
//      System.out.println(targetMethodData);
	if (statusTF!=null && targetClass.equals("PdmdbModuleTester")) statusTF.setText((PdmdbModuleTester.getCmdSeq()+1)+">");

	if (!targetClass.equals("PdmdbModuleTester")) TesterSequence.logMessage(">["+java.time.LocalTime.now().toString()+"] "+targetClass+"."+targetMethod+"("+targetMethodData+")\n");

	String [] targetMethodArgs = targetMethodData.split("\\|");

	String rv = (String)Command.exec(Class.forName("cbsw.lhcb.pdmdb."+targetClass), targetMethod, targetMethodArgs);
	TesterSequence.logMessage("<["+java.time.LocalTime.now().toString()+"] "+rv+"\n");
	String cmdStatus = rv.split("\\|")[0];
	if (!cmdStatus.equals("OK") && (!cmdStatus.equals("FAIL") || stopOnFail)) throw new RuntimeException(rv);

	if (responseList == null) continue; // No registered responses
  
	if (button!=null) button.setBackground(java.awt.Color.ORANGE); // Waiting for reponses.
// Get all responses to this command
	for ( TesterResponse response : responseList )
	{
	  if (response.incrementSequenceNumber) PdmdbModuleTester.rxSeq++;
	  String [] msgTokens = PdmdbModuleTester.readResponse(response.timeout);
// Check the token array exists. Bail out if null.
	  if (msgTokens == null) throw new NullPointerException("readResponse() returned null");
//	for (String msgToken : msgTokens) TesterSequence.logMessage(" "+msgToken);
// Check the token array has at least 2 elements. Bail out if not.
	  if (msgTokens.length < 2) throw new java.util.NoSuchElementException("readResponse() returned insufficient tokens ");

	  if (statusTF!=null) statusTF.setText(statusTF.getText()+"|"+msgTokens[0]+">"+msgTokens[1]);
      
// Check for out of sequence response
	  if (Integer.parseInt(msgTokens[0]) != PdmdbModuleTester.rxSeq) throw new RuntimeException("Out-of-sequence response to targetMethod "+targetMethod+" "+msgTokens[0]+"|"+msgTokens[1]);
// Check for error acknowledge
	  if (msgTokens[1].equals("ACK_ERROR")) throw new RuntimeException("ACK_ERROR received for targetMethod "+targetMethod);
// Check that the response targetMethod is the expected one.
	  if (!msgTokens[1].equals(response.opcode)) throw new UnsupportedOperationException("Unexpected response "+msgTokens[1]+"!="+response.opcode);
// Check for ERROR status
	  if (msgTokens[msgTokens.length-1].startsWith("TEST_STATUS=ERROR")) throw new RuntimeException("TEST_STATUS=ERROR for targetMethod "+targetMethod);

	  if (response.postProcessor != null)
	  {
	    String post = (String)Command.exec(Class.forName("cbsw.lhcb.pdmdb.TesterSequence"), response.postProcessor, msgTokens);
	    String postStatus = post.split("\\|")[0];
	    if (!postStatus.equals("OK") && (!postStatus.equals("FAIL") || stopOnFail)) throw new RuntimeException(rv);
	  }
	}
      }
      if (button!=null) button.setBackground(java.awt.Color.GREEN); // All done.
    }
    catch (Exception ex) {throw new Exception("Command.go(): "+ex+" for "+targetMethod+"["+data+"]");}
  }

/**
<p>Invokes the static method of the target class.</p>
@param targetClass The static method's class.
@param targetMethod The method to call.
@param targetArgs Target method parameters.
@return The method's return value (often String).
*/
  public static Object exec(Class<?> targetClass, String targetMethod, String [] targetArgs)
  {
//  System.err.println(String.format("Command.exec(): %s.%s()",targetClass,targetMethod));
    Object rv = null;
    try
    {
      java.lang.reflect.Method method = targetClass.getMethod(targetMethod,String[].class);
      rv = method.invoke(targetClass,new Object[] {targetArgs});
    }
    catch (Exception ex) {System.out.println(String.format("Command.exec(): %s for %s.%s()",ex,targetClass,targetMethod));}
    return rv;
  }
    
/**
<p>Invokes an OS shell script.</p>
<p>Creates a process whose stdout is connected to an input channel.</p>
<p>Process completion is signalled by the process sending a completion response message. This method blocks until it receives the completion response message therefore the process should not write anything other than its completion response to its stdout or this method will return prematurely.</p>
@param args {0=script-path,1...=command-parameters}
@return The command response.
*/
  static public String shell(String [] args)
  {
    String msg = "FAIL|Command.shell";
    java.nio.ByteBuffer bb = java.nio.ByteBuffer.allocate(1024);
    bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);

    try
    {
      Process pr = java.lang.Runtime.getRuntime().exec(args,null,TesterSequence.testDirectory.toFile());
//    ostream = pr.getOutputStream();
//    ochannel = Channels.newChannel(pr.getOutputStream());
      java.nio.channels.ReadableByteChannel ichannel = java.nio.channels.Channels.newChannel(pr.getInputStream());
      ichannel.read(bb);
      bb.flip();
      byte [] rdMsg = new byte[bb.limit()];
      bb.get(rdMsg);
      msg = new String(rdMsg);
//    pr.waitFor(5L,java.util.concurrent.TimeUnit.SECONDS);
    }
    catch (Exception ex){ System.err.println("Command.shell(): "+ex); msg = "FAIL|Command.shell|"+ex; }

    return msg;
  }

/**
<p>Registers a text field used to display messages.</p>
@param statusTF The text field.
*/
  public void setStatusTextField(javax.swing.JTextField statusTF){this.statusTF = statusTF;}

/**
<p>Registers the button that executes the command.</p>
@param button The button.
*/
  public void setButton(javax.swing.JButton button){this.button = button;}

/**
<p>Clears the text field.</p>
*/
  public void clearStatus()
  {
    if (statusTF!=null) statusTF.setText("");
    if (button!=null) button.setBackground(new java.awt.Color(220,220,220));
  }
}
