package cbsw.lhcb.pdmdb;

import java.util.prefs.Preferences;
import javax.xml.bind.annotation.*;
import java.util.*;
import cbsw.RegisterDescriptor;

/**
<p>This class encapsulates the GBTX register set.</p>
<p>GBTX register descriptors are unmarshalled into statically accessible members of this Gbtx class.</p>
<p>Note that the class members are static but unfortunately not immutable because of the need to use JAXB setters.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="gbtx")
class Gbtx
{
public static int csrSize;
public static ArrayList<RegisterDescriptor> descriptors;
public static ArrayList<String> fsmStates;
public static HashMap<String,RegisterDescriptor> map;

  @XmlAttribute
  public void setCsrSize(int csrSize){Gbtx.csrSize = csrSize;}

  @XmlElement(name="register")
  public void setDescriptors(ArrayList<RegisterDescriptor> descriptors)
  {
    Gbtx.descriptors = descriptors;
  }
  public ArrayList<RegisterDescriptor> getDescriptors(){return descriptors;}
    
  @XmlElement(name="fsmState")
  public void setFsmStates(ArrayList<String> fsmStates) { Gbtx.fsmStates = fsmStates; }
  public ArrayList<String> getFsmStates(){return fsmStates;}
    
  static
  {
    cbsw.Cbsw.unmarshal("https://www.hep.phy.cam.ac.uk/lhcb/PDMDB/xml/gbtx2.xml",Gbtx.class);
    Gbtx.map = new HashMap<String,RegisterDescriptor>();
    for (RegisterDescriptor reg : Gbtx.descriptors){ Gbtx.map.put(reg.getName(),reg); }
  }

/**
<p>No-argument constructor called by unmarshaller</p>
 */
  public Gbtx(){}

/**
<p>Display the current bit-field settings on System.out</p>
@param csrBitSet The register bitfield values encoded as a BitSet
 */
  public static void decode(BitSet csrBitSet)
  {
    if (csrBitSet==null) return;
    for (RegisterDescriptor reg : Gbtx.descriptors)
    {
      System.out.println(reg.getName()+": "+reg.get(csrBitSet));
    } 
    return;
  }
    
/**
<p>Read the register array from file and create BitSet.</p>
@param filename The name of the file from where the values will be read
@return The register values encoded as a BitSet
 */
  public static java.util.BitSet read(String filename)
  {
    int buflen=0;
    byte [] csr = new byte [Gbtx.csrSize];
    try
    {
      Scanner sc = new Scanner(new java.io.File(filename));
      while ( sc.hasNext() ) csr[buflen++] = (byte)sc.nextInt( 16 );
      sc.close();
    }
    catch (java.io.IOException ioe) { System.out.println("Gbtx.read(): "+ioe.toString()); return (new java.util.BitSet()); }

    return java.util.BitSet.valueOf(csr);
  }
    
/**
<p>Write the registers as ascii list of bytes.</p>
@param csrBitSet The register values encode in a BitSet
 */
  public static void write(java.util.BitSet csrBitSet)
  {
    byte [] csr = Arrays.copyOfRange(csrBitSet.toByteArray(),0,Gbtx.csrSize);
  
    javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
    javax.swing.filechooser.FileNameExtensionFilter filter = new javax.swing.filechooser.FileNameExtensionFilter( "GBTX configuration (GBT)", "gbt" );
    chooser.setFileFilter(filter);
    int returnVal = chooser.showOpenDialog(null);
  
    if(returnVal == javax.swing.JFileChooser.APPROVE_OPTION)
    {
      try
      {
        Formatter fm = new Formatter(chooser.getSelectedFile());
        for (byte b : csr) fm.format("%02x\n", b );
        fm.close();
      }
      catch (java.io.IOException ex) { System.err.println("Gbtx.write(): "+ex.toString()); }
    }
    return;
  }

/**
<p>Get the descriptor for the given key</p>
@param key The register bit-field key
@return The descriptor
 */
  public static cbsw.RegisterDescriptor get (String key){ return map.get(key); }
    
}

