package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;

/**
<p>Concrete class for IC interface to master GBTX.</p>
<p>Implements the put() and get() abstract methods from GbtxParent for the IC interface.</p>
@author Steve Wotton, Cavendish Lab (HEP), 2016-
**/
@XmlAccessorType(XmlAccessType.NONE)
public class GbtxIcDepp extends GbtxParent
{
/**
<p>No-argument constructor used during unmarshalling</p>
<p>Call init after unmarshalling to complete initialisation.</p>
**/
  public GbtxIcDepp(){ super(); }
    
/**
<p>Constructor used during unmarshalling by GbtxAdapter</p>
<p>Call init after unmarshalling to complete initialisation.</p>
@param id The GBTX logical ID.
@param mgt The FPGA MGT connected to this GBTX.
@param slaveAddress The I/O interface slave address (e.g. I2C).
@param txBaseAddress The IO interface base address.
@param rxBaseAddress Set this equal to txBaseAddress.
**/
  public GbtxIcDepp(int id, int mgt, int slaveAddress, int txBaseAddress, int rxBaseAddress){ super(id, mgt, txBaseAddress, rxBaseAddress); };
    
/**
<p>Writes a byte array to the hardware.</p>
@param offset Offset to location where to start writing.
@param regBuf Array of register values.
**/
  public void put(int offset, byte [] regBuf)
  {
    java.nio.ByteBuffer ic = java.nio.ByteBuffer.allocate(12+regBuf.length);
    ic.order(java.nio.ByteOrder.LITTLE_ENDIAN);

// Set header bytes
    ic.putShort((short)(8+regBuf.length)); // Transaction length
    ic.putShort((short)0); // Transaction address
    ic.put((byte)0); // Dummy zero
    ic.put((byte)0x02); // GBTX address & write op
    ic.put((byte)0x01); // GBTX command, value is ignored by GBTX
    ic.putShort((short)regBuf.length); // Length
    ic.putShort((short)offset); // Register address
    ic.put(regBuf); // value
    byte parity = 0;
    for (int b=6;b<ic.position();b++) parity = (byte)(parity ^ ic.get(b));
    ic.put(parity); // Parity

    try { io.put(txBaseAddress,ic); }
    catch (Exception ex) { throw new RuntimeException("GbtxIcDepp::put(): "+ex.toString()); }
  }
    
/**
<p>Writes a byte array to the hardware.</p>
@param bb Buffer of register values.
**/
  public void put(java.nio.ByteBuffer bb){ put(0,bb.array()); }

/**
<p>Reads a byte array from the hardware through the DEPP interface.</p>
@param offset Offset to location where to start reading.
@param regBuf Array of register values.
**/
  public void get(int offset, byte [] regBuf)
  {
    try
    {
      java.nio.ByteBuffer ic = java.nio.ByteBuffer.allocate(12);
      ic.order(java.nio.ByteOrder.LITTLE_ENDIAN);

// Set header bytes
      ic.putShort((short)8); // Transaction length
      ic.putShort((short)0); // Transaction address
      ic.put((byte)0); // Dummy zero
      ic.put((byte)0x03); // GBTX address & read op
      ic.put((byte)0x01); // GBTX command, value is ignored by GBTX
      ic.putShort((short)regBuf.length); // Length
      ic.putShort((short)offset); // Register address
      byte parity = 0;
      for (int b=6;b<ic.position();b++) parity = (byte)(parity ^ ic.get(b));
      ic.put(parity); // Parity

      io.put(txBaseAddress,ic);

      Thread.sleep(200);

      byte [] rxBuffer = new byte [512];
      io.get(rxBaseAddress,0,rxBuffer);

// Start of registers is offset by 7 bytes in the RX buffer
      System.arraycopy(rxBuffer,7,regBuf,0,regBuf.length);
    }
    catch (Exception ex) { throw new RuntimeException("GbtxIcDepp.get(): "+ex.toString()); }
  }
    
/**
<p>Reset the GBTX.</p>
<p>TCM does not have a master GBTX external reset. Does soft reset instead.</p>
**/
  public void reset()
  {
// The watchdog and timeout feature are disabled during soft configuration otherwise a timeout might cause the soft registers to be reset
    put("watchdog.enable","0");
    put("watchdog.startup-timeout-enable","0");
    
    softReset();
    
// Re-enable the watchdog and timeout feature
    put("watchdog.enable","1");
    put("watchdog.startup-timeout-enable","1");

/*test();*/
  }
    
}

