package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.xml.stream.*;
import java.io.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.*;
import cbsw.gui.*;

/**
<p>This class encapsulates the interactions with the muDaq FPGA MGTs.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
public class MGTControl implements ActionListener
{
RegisterIo io;
Preferences prefs;
Csr cfg;
@XmlAttribute(name="id")
int iMGT;
@XmlAttribute(name="txInvertPolarity")
boolean txInvertPolarity;
@XmlAttribute(name="rxInvertPolarity")
boolean rxInvertPolarity;

public MGTControl(){}

public int getId(){return iMGT;}

public GuiJPanel makePanel()
{
  GuiJPanel jP = new GuiJPanel(prefs,"");

  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(0,0,0,0);
  gbC.weightx = 1.0f;
  gbC.weighty = 1.0f;

  // Check boxes

  JCheckBox cb = jP.makeCheckBox("rxon","rxOn",false,true);
  gbC.gridwidth = 1;
  jP.add( cb, gbC );

  cb = jP.makeCheckBox("rxreset","rxReset",false,true);
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add( cb, gbC );

  cb = jP.makeCheckBox("txon","txOn",false,true);
  gbC.gridwidth = 1;
  jP.add( cb, gbC );

  cb = jP.makeCheckBox("txreset","txReset",false,true);
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add( cb, gbC );

  JTextField tf = jP.makeTF("precursor","5",6,"Precursor","Precursor",true);
  gbC.gridwidth = 1;
  jP.add( tf.getParent(), gbC );

  tf = jP.makeTF("postcursor","5",6,"Postcursor","Postcursor",true);
  gbC.gridwidth = 1;
  jP.add( tf.getParent(), gbC );

  tf = jP.makeTF("diffctrl","0",6,"Diff ctrl","Differential control",true);
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add( tf.getParent(), gbC );

  cb = jP.makeCheckBox("loopback","Loopback",false,true);
  gbC.gridwidth = 1;
  jP.add( cb, gbC );

  cb = jP.makeCheckBox("bitslip.enable","Bitslip enable",true,true);
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add( cb, gbC );

  JButton jb = new JButton("Send");
  jb.addActionListener(this);
  jb.setActionCommand("send");
  gbC.gridwidth = 1;
  jP.add( jb, gbC );
  
  return jP;
}
    
public void init(RegisterIo io, Csr cfg)
{
  prefs = Preferences.userNodeForPackage(MGTControl.class).node("MuDaq").node("gt."+iMGT);
  this.io=io;
  this.cfg = cfg;
  setStateFromPreferences();
}

public void setStateOff()
{
  cfg.set("gt."+iMGT+".rxreset", true);
  cfg.set("gt."+iMGT+".txreset", true);
  cfg.set("gt."+iMGT+".rxon", false);
  cfg.set("gt."+iMGT+".txon", false);
  return;
}

public void setStateFromPreferences()
{
  cfg.set("gt."+iMGT+".rxreset",prefs.getBoolean("rxreset",false));
  cfg.set("gt."+iMGT+".txreset",prefs.getBoolean("txreset",false));
  cfg.set("gt."+iMGT+".rxon",prefs.getBoolean("rxon",false));
  cfg.set("gt."+iMGT+".txon",prefs.getBoolean("txon",false));
  cfg.set("gt."+iMGT+".rxpolarity",rxInvertPolarity);
  cfg.set("gt."+iMGT+".txpolarity",txInvertPolarity);
  cfg.set("gt."+iMGT+".bitslip.enable",prefs.getBoolean("bitslip.enable",true));
  cfg.set("gt."+iMGT+".loopback",prefs.getBoolean("loopback",false));
  cfg.set("gt."+iMGT+".precursor",prefs.get("precursor","5"));
  cfg.set("gt."+iMGT+".postcursor",prefs.get("postcursor","5"));
  cfg.set("gt."+iMGT+".diffctrl",prefs.get("diffctrl","0"));

  return;
}

public void actionPerformed(ActionEvent e)
{
    Object ob = e.getSource();
    String command = e.getActionCommand();

    if ( command.equals("send") )
    {
      setStateFromPreferences();
// Note: The descriptor gt.n encompasses all 64 bits of the MGT registers
      try { cfg.put("gt."+iMGT,io); }
      catch ( Exception ex ){ System.out.println("MGTControl.actionPerformed(): "+ex.toString()); }
    }

   return;
};

}
