package cbsw.lhcb.pdmdb;

import java.awt.*;
import javax.swing.*;
import javax.xml.stream.*;
import java.io.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.*;
import cbsw.gui.*;
import javax.xml.bind.annotation.*;

/**
<p>The MuDaq class ecapsulates the interactions with a PDMDB module through the muDaq hardware interface</p>
**/
@XmlRootElement(name="mudaq")
@XmlAccessorType(XmlAccessType.NONE)
public class MuDaq
{
  Preferences prefs;
/**<p>The DEPP interface</p>*/
  cbsw.adept.JniAdeptDepp depp;
/**<p>The RegisterIo interface</p>*/
  RegisterIo io;
/**<p>List of tables that display FPGA register content</p>*/
  @XmlElementWrapper(name="deppTables")
  @XmlElement(name="deppTable")
  ArrayList<RegisterIoTable> tableList;
/**<p>List of FPGA MGT instances</p>*/
  @XmlElement(name="mgt")
  ArrayList<MGTControl> mgts;
/**<p>The PDMDB instance</p>*/
  @XmlElement(name="pdmdb")
  PdmdbMaster pdmdbMaster;
/**<p>E-fusing interface to the master GBTX through I2C</p>*/
  @XmlElement(name="gbtxI2c")
  GbtxDepp gbtxI2c;
/**<p>Normal interface to the master GBTX through either I2C or IC depending on XML configuration</p>*/
  @XmlElement(name="gbtx")
  @javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter(GbtxAdapter.class)
  GbtxParent gbtx;
/**<p>Controls for the FPGA DRP port</p>*/    
  @XmlElement(name="drp")
  DrpPanel drp;

  private JComboBox<String> regNameCB;
  private JTextField regValueTF;
  JFrame masterI2cConfigurationJF;
  JFrame masterConfigurationJF;
  JFrame masterLinkJF;
  PdmdbModuleTester tester;
  JFrame testerJF;
  private JMenuBar menuBar;
  private GuiJPanel jP;

/**<p>Static member containing the MuDaq instance referenced by exec()</p>*/
  static MuDaq muDaq;
    
  ScanControl scanControl;
  JFrame statusFrame;

/**<p>Configuration register mirror</p>*/
  static Csr cfg;
/**<p>Status register mirror</p>*/
  static Csr status;
  static
  {
    MuDaq.cfg = Csr.unmarshall(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp").get("csr-xml","https://www.hep.phy.cam.ac.uk/lhcb/muDAQ/xml/mudaq-csr-config.xml"));
    MuDaq.status = Csr.unmarshall(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp").get("status-xml","https://www.hep.phy.cam.ac.uk/lhcb/muDAQ/xml/mudaq-csr-status.xml"));
  }

/**
<p>No-argument constructor called by the unmarshaller</p>
**/
  public MuDaq()
  {
    scanControl = new ScanControl();
  }

/**
<p>Convenience method that builds a GUI panel</p>
@return The panel.
**/
  public GuiJPanel makePanel()
  {
    jP = new GuiJPanel(prefs,"");
    
    jP.setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.NONE;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;

    JTabbedPane mgtTP = new JTabbedPane();
    int i=0;
    for (MGTControl mgt: mgts)
    {
      GuiJPanel mgtP = mgt.makePanel();
      mgtP.setName("MGT."+mgt.getId());
      mgtTP.add(mgtP);
      i++;
    }
    gbC.gridwidth = 2;
    jP.add(mgtTP, gbC);

    gbC.gridwidth = 2;
//  jP.add(new ControlPanel(mB,depp), gbC);
    ControlPanel cP = new ControlPanel();
    jP.add(cP.makePanel(), gbC);

    return jP;
  }

/**
<p>Called after unmarshalling to initialise the instance and its children</p>
@param depp The I/O interface.
@param prefs The preference path.
**/
  public void init(cbsw.adept.JniAdeptDepp depp, Preferences prefs)
  {
    this.depp = depp;
    this.io = depp;
    this.prefs = prefs;
  
    for (RegisterIoTable dT: tableList) dT.init(io);
    statusFrame = GuiJPanel.makeFrame((new StatusPanel()).makePanel(),"Status");
    statusFrame.pack();
  
    pdmdbMaster.init(io); PdmdbMaster.pdmdbMaster=pdmdbMaster;
    for (MGTControl mgt: mgts) mgt.init(io,cfg);
  
    gbtxI2c.setName("tcm.gbtx.i2c");
    gbtxI2c.setPreferencesRoot(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp"));
    masterI2cConfigurationJF = GuiJPanel.makeFrame(gbtxI2c.makePanel(),"TCM GBTX I2C");
    masterI2cConfigurationJF.pack();
    gbtxI2c.init(io);

    gbtx.setName("tcm.gbtx");
    gbtx.setPreferencesRoot(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp"));
    masterConfigurationJF = GuiJPanel.makeFrame(gbtx.makePanel(),"TCM GBTX IC");
    masterConfigurationJF.pack();
    gbtx.init(io);

    drp.init(depp); // DrpPanel not yet converted to RegisterIo

    scanControl.init(io);

    masterLinkJF = GuiJPanel.makeFrame(pdmdbMaster.makePanel(),"Master Link");
    masterLinkJF.pack();

    tester = new PdmdbModuleTester();
    tester.init(prefs);
/*
testerJF = GuiJPanel.makeFrame(tester.makePanel(),"Tester commands");
testerJF.pack();
*/
  }

/**
<p>Convenience fuction that creates a menu</p>
@param title The menu title.
@return The menu.
**/
  public JMenu makeMasterLinkMenu(String title)
  {
    JMenu menu = new JMenu(title);

    JMenuItem menuItem = new JMenuItem("Master configuration (I2C)...");
    menuItem.addActionListener( ae->masterI2cConfigurationJF.setVisible(true) );
    menu.add(menuItem);

    menuItem = new JMenuItem("Master configuration...");
    menuItem.addActionListener( ae->masterConfigurationJF.setVisible(true) );
    menu.add(menuItem);

    return menu;
  }
	
/**
<p>Resets muDaq</p>
*/
  public void reset()
  {
    try
    {
      cfg.put("daq-reset",true,io);
      cfg.put("daq-reset",false,io);
      
      cfg.put("counter-reset",true,io);
      cfg.put("counter-reset",false,io);
      
      cfg.put("gbt.reset-all",true,io);
      cfg.put("gbt.reset-all",false,io);
    }
    catch ( Exception ex ){ throw new RuntimeException("MuDaq.reset(): "+ex);}
    return;
  }

/**
<p>Updates the cached configuration register with setting from preferences</p>
**/
  void setStateConfigure()
  {
    setStateFromPreferences();
    cfg.set("tfc.enable",true);
    cfg.set("dcms.enable",true);

    return;
  };

/**
<p>Puts MuDaq into the low power shutdown state</p>
*/
  void setStateShutdown()
  {
    cfg.set("tfc.enable",false);
    cfg.set("dcms.enable",false);
    for ( MGTControl mgt : mgts ) mgt.setStateOff();

    return;
  };

/**
<p>Reads saved settings from Preferences and updates the configuration register mirror</p>
<p>Does not write the settings to the hardware.</p>
*/
  public void setStateFromPreferences()
  {

    System.out.println("MuDaq.setStateFromPreferences(): Setting state from prefs path "+prefs.absolutePath());

    try
    {
      prefs.sync();
    } catch (BackingStoreException ex) { ex.printStackTrace(); }

// DAQ parameters
    cfg.set("daq.latency-compensation", prefs, "1b");

// Enable TFC commands
    cfg.set("tfctog-enable", prefs, "f");

// GT settings

    for ( MGTControl mgt : mgts ) mgt.setStateFromPreferences();

// Pulser settings
    cfg.set("pulser.count", prefs, 80);
    cfg.set("pulser.delay", prefs, 1000);

// TX test pattern
    cfg.set("gt.txpattern",prefs,"AA");

    return;
  };

/**
<p>Triggers GBTX e-fusing</p>
**/
  public void fuseGbtx()
  {
    java.lang.Runnable pulse = () -> toggle("gbtx.efuse-program");
    gbtxI2c.fuse(pulse);
    return;
  }

/**
<p>Toggles a configuration register (on, then off)</p>
@param regName The register name as described in XML.
**/
  public void toggle(String regName)
  {
    try
    {
      cfg.put(regName,true,io);
      cfg.set(regName,false);
    }
    catch ( Exception ex ){ throw new RuntimeException("GbtxDepp.write(): "+ex); }
  }

/**
<p>Prepares for a bit error rate test</p>
<p>Enables a fixed data pattern and resets the bitflip counters.</p>
*/
  public void prepareBER()
  {
    try
    {
      
// Set a fixed pattern on the down links
      cfg.put("tfctog-enable","0",io);
      cfg.put("tfc-d","A",io);

// Reset the bitflip counters
      cfg.put("counter-reset",true,io);
      cfg.put("counter-reset",false,io);
    }
    catch (Exception ex){ throw new RuntimeException("MuDaq.prepareBER|"+ex); }
  }

/**
<p>Reads the bitflip counters from hardware</p>
@return Byte buffer containing the counters
*/
  public java.nio.ByteBuffer getBER()
  {
    try
    {
      byte [] rdBuf = new byte [192];
      io.get(status.getBaseAddress(),256,rdBuf);
      java.nio.ByteBuffer bb = java.nio.ByteBuffer.wrap(rdBuf);
      bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);
      return bb;
    }
    catch (Exception ex){ throw new RuntimeException("MuDaq.getBER|"+ex); }
  }

  public class ControlPanel
  {
    public ControlPanel(){}
    
    public JPanel makePanel()
    {
      JPanel cP = new JPanel();
      
      cP.setLayout( new GridBagLayout() );
      GridBagConstraints gbC = new GridBagConstraints();
      gbC.fill = GridBagConstraints.VERTICAL;
      gbC.anchor = GridBagConstraints.CENTER;
      gbC.insets = new Insets(2,2,2,2);
      gbC.weightx = 1.0f;
      gbC.weighty = 1.0f;
      gbC.fill = GridBagConstraints.BOTH;

      JButton button = null;

      JPanel jp = new JPanel();
      jp.setBorder( BorderFactory.createTitledBorder("Configuration") );

      regNameCB = new JComboBox<String>();
      regNameCB.setEditable(true);
      regNameCB.addItem("daq.latency-compensation");
      regNameCB.addItem("tfctog-enable");
      gbC.gridwidth = 1;
      jp.add(regNameCB,gbC);
    
      regValueTF = new JTextField(8);
      gbC.gridwidth = 1;
      jp.add(regValueTF,gbC);

      button = new JButton("Set");
      button.addActionListener(l->set());
      gbC.gridwidth = GridBagConstraints.REMAINDER;
      jp.add(button,gbC);

      gbC.gridwidth = GridBagConstraints.REMAINDER;
      cP.add( jp, gbC );

      button = new JButton( "Shutdown" );
      button.setName("shutdown");
      button.addActionListener( buttonAL );
      gbC.gridwidth = 1;
      cP.add( button, gbC );

      button = new JButton( "Init" );
      button.setName("init");
      button.addActionListener( buttonAL );
      gbC.gridwidth = 1;
      cP.add( button, gbC );

      button = new JButton( "Enable" );
      button.setName("enable");
      button.addActionListener( buttonAL );
      gbC.gridwidth = 1;
      cP.add( button, gbC );

      button = new JButton( "Disable" );
      button.setName("disable");
      button.addActionListener( buttonAL );
      gbC.gridwidth = GridBagConstraints.REMAINDER;
      cP.add( button, gbC );

      button = new JButton( "Reset" );
      button.addActionListener(ae->reset());
      gbC.gridwidth = 1;
      cP.add( button, gbC );

      button = new JButton( "Efuse" );
      button.addActionListener(ae->fuseGbtx());
      gbC.gridwidth = 1;
      cP.add( button, gbC );

      button = new JButton( "Trigger" );
      button.addActionListener(ae->toggle("trigger-tog"));
      gbC.gridwidth = GridBagConstraints.REMAINDER;
      cP.add( button, gbC );

      jp = new JPanel();
      jp.setBorder( BorderFactory.createTitledBorder("Pulser") );

      jp.add(new JLabel("Count"));
      JTextField tf = jP.makeTF("pulser.count", "100", 10, "Pulser count", "Pulser count",true);
      tf.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 11));
      jp.add( tf );

      jp.add(new JLabel("Delay"));
      tf = jP.makeTF("pulser.delay", "10000", 10, "Pulser delay", "Pulser delay",true);
      tf.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 11));
      jp.add( tf );

      button = new JButton( "Pulse" );
      button.setName("pulse");
      button.addActionListener( buttonAL );
      jp.add( button );

      gbC.gridwidth = GridBagConstraints.REMAINDER;
      cP.add(jp,gbC);

      gbC.gridwidth = GridBagConstraints.REMAINDER;
      cP.add(scanControl.makePanel(),gbC);

//     tf = makeTF("gt.txpattern", "112233445566778899aabbccddee", 28, "Loopback pattern", "Loopback pattern", false);
//     tf.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 11));
//     gbC.gridwidth = GridBagConstraints.REMAINDER;
//     cP.add( tf.getParent(),gbC );

      jp = new JPanel();
      jp.setBorder( BorderFactory.createTitledBorder("TFC") );

      button = new JButton( "Trigger" );
      button.addActionListener( ae->toggle("tfc-trigger") );
      jp.add( button );

      button = new JButton( "SYNC" );
      button.addActionListener( ae->toggle("tfc-sync") );
      jp.add( button );

      button = new JButton( "Test pulse" );
      button.addActionListener( ae->toggle("tfc-tp") );
      jp.add( button );

      button = new JButton( "BX reset" );
      button.addActionListener( ae->toggle("tfc-bxreset") );
      jp.add( button );

      gbC.gridwidth = GridBagConstraints.REMAINDER;
      cP.add( jp, gbC );

      jp = new JPanel();
      jp.setBorder( BorderFactory.createTitledBorder("DRP") );
      jp.add(drp.makePanel());
      gbC.gridwidth = GridBagConstraints.REMAINDER;
      cP.add( jp, gbC );

      return cP;
    };

/**
<p>Called from GUI panel to set register values.</p>
<p>The register name in the text field can be a regular expression in
which case the same value is applied to all registers that match the
expression.</p>
*/
    public void set()
    {
      try
      {
	String regExp = (String)regNameCB.getSelectedItem();
//    cfg.put(regName,regValueTF.getText(),io);

	cfg.getDescriptors().stream().filter(reg->reg.getName().matches(regExp))
	  .collect(java.util.stream.Collectors.toList())
	  .forEach(reg->{try{cfg.put(reg.getName(),regValueTF.getText(),io);}catch(RegisterIoException ex){}});

      }
      catch ( Exception ex ){ throw new RuntimeException("MuDaq.set(): "+ex); }
    }

    private java.awt.event.ActionListener buttonAL = ae ->
      {
	JButton jb = (JButton)ae.getSource();
	String buttonName = jb.getName();
  
	try
	{
	  if ( buttonName.equals("enable") )
	  {
	    cfg.put("ttc.enable",true,io);
// Reset the GBT buffer write pointer
	    cfg.put("daq.gbtbuf-reset",true,io);
	    cfg.set("daq.gbtbuf-reset",false);
	  }
	  else if ( buttonName.equals("disable") )
	  {
	    cfg.put("ttc.enable",false,io);
	  }
	  else if ( buttonName.equals("init") )
	  {
	    setStateConfigure();
	    io.put(cfg.getBaseAddress(),0,cfg.toByteArray(4));
	  }
	  else if ( buttonName.equals("shutdown") )
	  {
	    setStateShutdown();
	    io.put(cfg.getBaseAddress(),0,cfg.toByteArray(4));
	  }
	  else if ( buttonName.equals("pulse") )
	  {
	    cfg.put("pulser.count",prefs,"80",io);
	    cfg.put("pulser.delay",prefs,"1000",io);

// Issue a pulser reset in case the delay pipeline has been changed
	    cfg.put("pulser.reset",true,io);
	    cfg.put("pulser.reset",false,io);
      
	    cfg.put("pulser-go",true,io);
	    cfg.set("pulser-go",false);
	  }
	}
	catch ( Exception ex ){ throw new RuntimeException("MuDaq.buttonAL: "+ex); }
      };

  }

  public class StatusPanel
  {
    JPanel statusPanel;
    
    public StatusPanel(){}
    public JPanel makePanel()
    {
      statusPanel = new JPanel();
      
      statusPanel.setLayout( new GridBagLayout() );
      GridBagConstraints gbC = new GridBagConstraints();
      gbC.fill = GridBagConstraints.VERTICAL;
      gbC.anchor = GridBagConstraints.CENTER;
      gbC.insets = new Insets(2,2,2,2);
      gbC.weightx = 1.0f;
      gbC.weighty = 1.0f;
      gbC.fill = GridBagConstraints.BOTH;
    
      gbC.gridwidth = GridBagConstraints.REMAINDER;
      statusPanel.add(RegisterIoTable.makePanel(tableList),gbC);
      tableList.get(0).addDecodeAction(e->{try{status.cache(io);status.decode();}catch (RegisterIoException re){System.err.println("MuDaq: "+re);}});
      tableList.get(1).addDecodeAction(e->cfg.decode());
  
      return statusPanel;
    }

  }
}
