package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.GridBagConstraints;
import java.awt.Color;
import javax.swing.*;
import java.io.*;
import java.util.*;
import java.util.prefs.Preferences;
import cbsw.gui.*;

/**
<p>This class encapsulates the interactions with the PDMDB.</p>
**/
@XmlRootElement(name="pdmdb")
@XmlAccessorType(XmlAccessType.NONE)
public class PdmdbMaster
{
  cbsw.RegisterIo io;
  @XmlElement(name="sca")
  ScaDepp sca;
  @XmlElement(name="db")
  PdmdbControl pdmdb;
  @XmlElement(name="ec")
  List<EcControl> ecList;
  @XmlElement(name="dtm")
  List<DtmControl> dtmList;

  static PdmdbMaster pdmdbMaster;
   
  JTextField claroTF;
  JButton spiTestButton;
  JCheckBox spiTestLoopCB;
  JTextField spiTestTF;
  public static boolean loop = false;
    
/**
<p>No-argument constructor used during unmarshalling.</p>
**/
  public PdmdbMaster(){}

/**
<p>Called after unmarshalling to complete initialisation of this and children.</p>
@param io The I/O interface to the hardware.
**/
  public void init( cbsw.RegisterIo io )
  {
    this.io = io;
    sca.init(io);
    ScaControl.staticSca=sca;
    pdmdb.init(sca);
    for (DtmControl dtm : dtmList.toArray(new DtmControl[0])){ dtm.init(sca); }
    for (EcControl ec : ecList.toArray(new EcControl[0])){ ec.init(sca); }
  }

/**
<p>Convenience function to make a GUI panel.</p>
@return The panel.
**/
  public GuiJPanel makePanel()
  {
    GuiJPanel jp = new GuiJPanel(Preferences.userNodeForPackage(PdmdbMaster.class).node("SCA"),"");

    jp.setLayout( new java.awt.GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new java.awt.Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(sca.makePanel(),gbC);

    JButton button = new JButton("Save CLARO file");
    button.addActionListener(l->saveClaroConfigurationFile());
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("Choose CLARO file");
    button.addActionListener(l->chooseConfigurationFile());
    gbC.gridwidth = GridBagConstraints.RELATIVE;
    jp.add(button,gbC);

    claroTF = new JTextField(Preferences.userNodeForPackage(ClaroControl.class).node("SCA").get("claro.filename","claro.spi"));
    claroTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add( claroTF, gbC );
    
    spiTestButton = new JButton("CLARO SPI test");
    spiTestButton.addActionListener(e->{((JButton)e.getSource()).setBackground(Color.yellow); (new SpiTestTask()).execute();});
    gbC.gridwidth = 1;
    jp.add(spiTestButton,gbC);

    spiTestLoopCB = new JCheckBox("Loop",false);
    spiTestLoopCB.addActionListener(l->PdmdbMaster.loop=spiTestLoopCB.isSelected());
    gbC.gridwidth = 1;
    jp.add(spiTestLoopCB,gbC);

    gbC.gridwidth = GridBagConstraints.RELATIVE;
    jp.add(new JLabel("Status",JLabel.RIGHT),gbC);
    
    spiTestTF = new JTextField(10);
    spiTestTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(spiTestTF,gbC);

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(pdmdb.makePanel(),gbC);

    JTabbedPane tp = new JTabbedPane();
    for ( EcControl ec : ecList.toArray(new EcControl[0]) ){ tp.add(ec.makePanel()); }
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(tp,gbC);

    tp = new JTabbedPane();
    for (DtmControl dtm : dtmList.toArray(new DtmControl[0])){ tp.add(dtm.makePanel()); }
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(tp,gbC);
    
    return jp;
  }

/**
<p>Select and load the CLARO configuration registers.</p>
**/
  public void chooseConfigurationFile()
  {
    Preferences prefs = Preferences.userNodeForPackage(ClaroControl.class).node("SCA");
    File file = cbsw.Cbsw.chooseFile(prefs.get("claro.dir","."),new javax.swing.filechooser.FileNameExtensionFilter( "CLARO configuration (SPI)", "spi" ));
    prefs.put("claro.dir",file.getParent());
    prefs.put("claro.filename",file.getAbsolutePath());
    claroTF.setText(prefs.get("claro.filename","claro.spi"));

    loadClaroConfiguration();

    return;
  }
    
/**
<p>Load the CLARO configuration registers.</p>
**/
  public void loadClaroConfiguration()
  {
    Preferences prefs = Preferences.userNodeForPackage(ClaroControl.class).node("SCA");
    File cf = new File(prefs.get("claro.filename","claro.spi"));
    for (EcControl ec: ecList)
    {
      ArrayList<Claro> claroList = ec.claro.claros;
      for (Claro claro: claroList) claro.set(cf);
    }
  }

/**
<p>Create a CLARO configuration file from the current settings.</p>
**/
  public void saveClaroConfigurationFile()
  {
    Preferences prefs = Preferences.userNodeForPackage(ClaroControl.class).node("SCA");
    File file = cbsw.Cbsw.chooseFile(prefs.get("claro.dir","."),new javax.swing.filechooser.FileNameExtensionFilter( "CLARO configuration (SPI)", "spi" ));
    if(file == null) return;
    
    try
    {
      FileOutputStream fo = new FileOutputStream(file);
      for (EcControl ec: ecList)
      {
        ArrayList<Claro> claroList = ec.claro.claros;
        for (Claro claro: claroList){ fo.write(claro.toByteArray()); }
      }
      fo.close();
    }
    catch (IOException ex) { System.err.println("PdmdbMaster.saveClaroConfigurationFile(): "+ex.toString()); }
  }

// Loop test of CLARO configuration registers

  class SpiTestTask extends SwingWorker<Boolean, String>
  {
    @Override
    protected Boolean doInBackground()
    {
      boolean pass = true;
      int trial=0;
      int errors=0;
      while ( loop  )
      {
	int irand = (int)(0xff&Math.round(256 * Math.random()));
	for (EcControl ec: ecList)
	{
	  if (!ec.claro.testEnable) continue;
	  ArrayList<Claro> claroList = ec.claro.claros;
	  for (Claro claro: claroList) claro.setRandom(irand);

          sca.allocateTxRxBuffers(10*claroList.size()); // 10 SCA commands per CLARO
          for ( Claro claro : claroList ) claro.prepareSpi(true);
          sca.putBytes(0,true);

          if (!ClaroControl.verify(sca,claroList))
	  {
	    errors++;
	    System.err.println("PdmdbMaster.doInBackground(): verification failed for trial "+trial+" retrying...");
	    if (!ClaroControl.verify(sca,claroList))
	    {
	      pass = false;
 	      System.err.println("PdmdbMaster.doInBackground(): retry failed");
	    }
	    else
	    {
 	      System.err.println("PdmdbMaster.doInBackground(): retry OK");
	    }
	  }

          if (trial%10==0) publish(String.format("Trials: %d Errors: %d",trial,errors));
          trial++;
	}
      }
      return Boolean.valueOf(pass);
    }
      
    @Override
    protected void done(){ try { if (get()) spiTestButton.setBackground(Color.green); else spiTestButton.setBackground( Color.orange ); } catch (Exception e){} }

    @Override
    protected void process( List<String> chunks ){for ( String pass : chunks ) spiTestTF.setText(pass);}
  }
}
