package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.file.*;
import java.nio.file.attribute.*;
import java.awt.GridBagConstraints;
import javax.swing.*;
import java.util.prefs.*;
import java.util.*;
import java.time.LocalTime;

/**
<p>This class encapsulates the interactions with the Module Tester Board (MTB) hardware.</p>
@author Steve Wotton, Cavendish Lab (HEP), 2018-
**/
public class PdmdbModuleTester
{

Preferences prefs;

static cbsw.SocketTxRx txrx;
boolean threadStopFlag = false;
    
JFrame programJF;
JTextField cmdTF;
static JTextArea textArea;
JScrollPane textSP;
    
static int cmdSeq=0;
static int rxSeq=0;

static String tcmQR;
static String [] dtmQR;
static String tcmGbtxId;
static String [][] dtmGbtxId;

private void socketDialog()
{
  String url = JOptionPane.showInputDialog("Please enter socket URL",prefs.get("mtb.socket","http://:7999"));
  if (url != null) prefs.put("mtb.socket",url);
  return;
}

static
{
  dtmQR = new String [3];
  dtmGbtxId = new String [3][2];
  
  Preferences mtbPrefs = Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp");

  txrx = new cbsw.SocketTxRx();
  try { txrx.makeSocket(new java.net.URL(mtbPrefs.get("mtb.socket","http://:7999"))); }
  catch (Exception ex){System.err.println("PdmdbModuleTester.PdmdbModuleTester(): " + ex.toString());}
}
    
public PdmdbModuleTester(){}

public void init(Preferences prefs)
{
  this.prefs = prefs;
  System.err.println("Prefs path: "+prefs.absolutePath());
}

    /*
public JPanel makePanel()
{
  JPanel jP = new JPanel();
// Set layout style

  jP.setLayout( new java.awt.GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  gbC.fill = GridBagConstraints.VERTICAL;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new java.awt.Insets(2,2,2,2);
  gbC.weightx = 1.0f;
  //  gbC.weighty = 1.0f;
  gbC.fill = GridBagConstraints.BOTH;


  cmdTF = jP.makeTF("command","",80,"Command","Enter command",false);
  cmdTF.addActionListener(this);
  cmdTF.setFont(new java.awt.Font("Monospaced",java.awt.Font.PLAIN,12));
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add(cmdTF.getParent(),gbC);
  
  textArea = new JTextArea(10,10); // Minimum width. Will expand to fill available width.
  textArea.setEditable(false);
  textArea.setFont(new java.awt.Font("Monospaced",java.awt.Font.PLAIN,12));
  textSP = new JScrollPane(textArea);
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  gbC.fill = GridBagConstraints.HORIZONTAL;
  jP.add(textSP,gbC);

  return jP;
}
	
public void actionPerformed( ActionEvent e )
{
  String cmd = e.getActionCommand();
  java.awt.Component src = (java.awt.Component)e.getSource();
  if ( src.getName().equals( "command" ) )
  {
    textArea.setText(null); // Clear the text area ready for the next command
    exec(((JTextField)src).getText());
  }
 return;
}
    */

public static String [] readResponse(int timeout)
{

  try
  {
    String msg = txrx.rx(timeout);
    TesterSequence.logMessage("<["+LocalTime.now().toString()+"] "+msg);
    return msg.split("\\|");
  }
  catch (Exception ex) {System.out.println("PdmdbModuleTester.readResponse(): "+ex.toString());}

  return null;
}

/**
<p>Cleans out any pending responses.</p>
 */
public static void readFlush()
{
  try { txrx.rxFlush(); } catch (Exception ex){}
  rxSeq = cmdSeq;
}

/**
<p>Write XML data</p>
@param args Command line arguments
@return Command response
*/
static String xml(String [] args)
{
  try
  {
    BufferedWriter tcmXml = Files.newBufferedWriter(TesterSequence.testDirectory.resolve(Path.of("tcm.xml")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);
    tcmXml.write("<tcm qr=\""+PdmdbModuleTester.tcmQR+"\" gbtx=\"0x"+PdmdbModuleTester.tcmGbtxId+"\" />");
    tcmXml.flush();
    tcmXml.close();
  
    BufferedWriter dtmXml = Files.newBufferedWriter(TesterSequence.testDirectory.resolve(Path.of("dtm.xml")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);
    for (int dtm=0;dtm<3;dtm++)
    {
      dtmXml.write("<dtm qr=\""+PdmdbModuleTester.dtmQR[dtm]+"\" gbtx0=\""+PdmdbModuleTester.dtmGbtxId[dtm][0]+"\" gbtx1=\""+PdmdbModuleTester.dtmGbtxId[dtm][1]+"\" position=\""+dtm+"\" />");
    }
    dtmXml.flush();
    dtmXml.close();
  }
  catch (IOException ioe){return "FAIL|PdmdbModuleTester.xml|IO Exception";}

  return "OK|PdmdbModuleTester.xml";
}

// Called by Command.Method.invoke()
public static String exec(String [] args)
{
  cmdSeq++;

  if (args==null) return "FAIL|PdmdbModuleTester.exec";

  String cmd = Integer.toString(cmdSeq);
  for (String s : args) cmd = cmd + "|" + s;
  cmd = cmd + "\n";

  TesterSequence.logMessage(">["+LocalTime.now().toString()+"] "+cmd);
  try { txrx.tx(cmd); }
  catch (Exception ex) {System.err.println("PdmdbModuleTester.exec(String[]): "+ex.toString()+" for "+cmd); return "FAIL|PdmdbModuleTester.exec";}

  return "OK|PdmdbModuleTester.exec";
}

public static void exec(String msg)
{
  cmdSeq++;
  
  String cmd = cmdSeq + "|" + msg + "\n";
  TesterSequence.logMessage(">["+LocalTime.now().toString()+"] "+cmd);
  try
  { txrx.tx(cmd); }
  catch (Exception ex) {System.err.println("PdmdbModuleTester.exec(String): "+ex.toString()+" for "+msg);}
  
  return;
}

static String scanModule(String name, String indicatorPattern)
{
  PdmdbModuleTester.exec("INDICATE|"+indicatorPattern);
  PdmdbModuleTester.rxSeq++;
  PdmdbModuleTester.readResponse(3000);
  String [] dummy = PdmdbModuleTester.readResponse(3000);
  JOptionPane pane = new JOptionPane(new String("Scan "+name),JOptionPane.PLAIN_MESSAGE,JOptionPane.DEFAULT_OPTION,null,null);
  //  pane.setBackground(java.awt.Color.BLUE);
  pane.setWantsInput(true);
  JDialog dialog = pane.createDialog(null, name);
  dialog.setVisible(true);
  String qr = ((String)pane.getInputValue()).toUpperCase();

  if (qr.matches("^4RXCAMDB[DT][TC][0-1][0-9][0-9][0-9]")) return qr;
  else return "";
}

/**
<p>Scan TCM/DTM barcodes</p>
<p>Callable by {@link Command}</p>
@param args Command args
@return Command response
*/
public static String scan(String [] args)
{
  try
  {
    String dtm0 = scanModule("DTM0","1000");
    if (dtm0.equals("")) throw new RuntimeException("Invalid scanned code (DTM0)");
    TesterSequence.xml(String.format("<index testerModule=\"dtm0\" code=\"%s\" directory=\"%s\" />\n",dtm0,TesterSequence.testSubdirectory));
  
    String tcm = scanModule("TCM","0001");
    if (tcm.equals("")) throw new RuntimeException("Invalid scanned code (TCM)");
    TesterSequence.xml(String.format("<index testerModule=\"tcm\" code=\"%s\" directory=\"%s\" />\n",tcm,TesterSequence.testSubdirectory));

    String dtm1 = scanModule("DTM1","0100");
    if (dtm1.equals("")) throw new RuntimeException("Invalid scanned code (DTM1)");
    TesterSequence.xml(String.format("<index testerModule=\"dtm1\" code=\"%s\" directory=\"%s\" />\n",dtm1,TesterSequence.testSubdirectory));
  
    String dtm2 = scanModule("DTM2","0010");
    if (dtm2.equals("")) throw new RuntimeException("Invalid scanned code (DTM2)");
    TesterSequence.xml(String.format("<index testerModule=\"dtm2\" code=\"%s\" directory=\"%s\" />\n",dtm2,TesterSequence.testSubdirectory));

    PdmdbModuleTester.exec("INDICATE|0000");
    PdmdbModuleTester.rxSeq++;
    PdmdbModuleTester.readResponse(3000);
    String [] dummy = PdmdbModuleTester.readResponse(3000);
  
    return "OK|PdmdbModuleTester.scan|tcm="+tcm+" dtm0="+dtm0+" dtm1="+dtm1+" dtm2="+dtm2;
  }
  catch (Exception ex) { return "FAIL|PdmdbModuleTester.scan|"+ex.toString(); }

}

/**
<p>Acquire DAC test samples</p>
<p>Callable by {@link Command}</p>
@param args []=samples, bins, binMinimum, binWidth, binMaximum, rmsMaximum, svg
@return Command response
*/
public static String readDacs(String [] args)
{
  int samples;
  int bins;
  double binMinimum;
  double binWidth;
  double binMaximum;
  double maximumRMS;
  boolean svg;
  try
  {
    samples = Integer.decode(args[0]);
    bins = Integer.valueOf(args[1]);
    binMinimum = Double.valueOf(args[2]);
    binWidth = Double.valueOf(args[3]);
    binMaximum = binMinimum + bins*binWidth;
    maximumRMS = Double.valueOf(args[4]);
    svg = Boolean.valueOf(args[5]);
  }
  catch (Exception ex){ return "FAIL|PdmdbModuleTester.readDacs|"+ex.toString(); }

  if (samples < 1) return "FAIL|PdmdbModuleTester.readDacs|Invalid argument";

  String timestamp = java.time.LocalDateTime.now().format(java.time.format.DateTimeFormatter.ofPattern("yyyyMMdd-A"));

  double dacVal [] = new double[4];

  try
  {
    BufferedWriter writer = Files.newBufferedWriter(TesterSequence.testDirectory.resolve(Path.of("dac-"+timestamp+".dat")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);

// Bin description
    writer.write(""+bins+","+binMinimum+","+binWidth); // #bins, low limit, bin width
    writer.newLine();
// Column headers
    for (int dac=0;dac<3;dac++) writer.write("dac"+Integer.toString(dac)+",");
    writer.write("dac"+Integer.toString(3));
    writer.newLine();

// Data
    for (int i=0;i<samples;i++)
    {
      PdmdbModuleTester.exec("READ_DACS");
      PdmdbModuleTester.rxSeq++;
      PdmdbModuleTester.readResponse(3000); // ACK
      String [] response = PdmdbModuleTester.readResponse(3000);
      for (String token: response)
      {
	String [] dacResponse = token.split("=");
	if (dacResponse==null || dacResponse.length!=2) continue;
	if (!dacResponse[0].startsWith("DAC")) continue;
	int dacId = Integer.parseInt(dacResponse[0].substring(3,4));
	if (dacId<0 || dacId>3) break;
	dacVal[dacId] = Double.valueOf(dacResponse[1]);
      }
      for (int dac=0;dac<3;dac++) writer.write(Double.toString(dacVal[dac])+",");
      writer.write(Double.toString(dacVal[3]));
      writer.newLine();
    }
    writer.flush();
    writer.close();

// Create the histograms (get data from the file we've just created)
    BufferedReader reader = Files.newBufferedReader(TesterSequence.testDirectory.resolve(Path.of("dac-"+timestamp+".dat")));
    plot(reader,svg,"dac");
  }
  catch (Exception ex){return "FAIL|PdmdbModuleTester.readDacs|"+ex.toString();}
    
  return "OK|PdmdbModuleTester.readDacs";
}

/**
<p>Create histograms from reader</p>
<p>Creates distributions and writes summary to XML</p>
@param reader The reader from where to get the data
@param svg Save plot as SVG file if true
@param element The name of the element written to XML
@throws IOException I/O exception
*/
  static void plot(BufferedReader reader, boolean svg, String element) throws IOException
  {
    String timestamp = java.time.LocalDateTime.now().format(java.time.format.DateTimeFormatter.ofPattern("yyyyMMdd-A"));

    try
    {
      cbsw.gui.Histogram hist = new cbsw.gui.Histogram("");
      try { hist.readStream(reader); } catch(cbsw.gui.Histogram.EOFException eof){}
      reader.close();
      
// Set the Paint for all series to be the same block colour
      hist.setPaint(new java.awt.Color(0x3333cc));

// Output the plots and dataset statistics
      for ( Comparable key : hist.keys() )
      {
        hist.hide(); // Hide any series that are currently visible
        hist.show(key); // Draw this series
        if (svg) hist.saveAsSVG(TesterSequence.testDirectory.resolve(Path.of((String)key+".svg")).toFile()); // Write as an SVG file
        double mean = hist.mean(key); // Unbinned mean
        double rms = hist.rms(key); // Unbinned RMS
        TesterSequence.xml(String.format("<%s id=\"%s\" mean=\"%.2f\" rms=\"%.2f\" timestamp=\"%s\"/>\n",element,(String)key,mean,rms,timestamp));
      }
    }
    catch (IOException ioe){throw ioe;}
  }

public static int getCmdSeq(){return cmdSeq;}

public static void makeSocket(java.net.URL url)
{
  try
  {
      txrx.makeSocket(url);
      readFlush();
  }
  catch ( Exception ex ) { System.err.println( "PdmdbModuleTester.makeSocket(): [" + url.toString() + "] " + ex.toString() ); }

  return;
}

}
