package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.util.*;

/**
<p>This class encapsulates a collection of the SCA commands.</p>
<p>Unmarshall SCA TX register descriptors into statically accessible members of this Sca class.</p>
*/
@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="scacommands")
class Sca
{
public static ArrayList<ScaCommandDescriptor> descriptors;
public static HashMap< String, ScaCommandDescriptor > map;

  @XmlElement(name="SCACommand")
  public void setDescriptors(ArrayList<ScaCommandDescriptor> descriptors) { Sca.descriptors = descriptors; }
  public ArrayList<ScaCommandDescriptor> getDescriptors(){return descriptors;}
    
  static
  {
    cbsw.Cbsw.unmarshal("https://www.hep.phy.cam.ac.uk/lhcb/PDMDB/sca/SCATXCommands.xml",Sca.class);

    Sca.map = new HashMap< String, ScaCommandDescriptor >();

    Sca.descriptors.forEach(reg->map.put( reg.getName(), reg ));
  }
    
  public Sca(){}

  public static Byte getI2cChannel(String command)
  {

    String [] tokens = command.split("[_/]");
    String [] cmdch = command.split("/");
 
    if (tokens[0].equals("I2C"))
    {
      return (byte)(Integer.decode(cmdch[cmdch.length-1]).intValue()+3); // I2C commands have bus number appended after "/"
    }
    else
    {
      return null; // For all other commands return null
    }

  }

  public static ScaCommandDescriptor getDescriptor(String command)
  {

    String [] tokens = command.split("[_/]");
    String [] cmdch = command.split("/");
 
    ScaCommandDescriptor commandDescriptor;

    byte scaChannel;
    if (tokens[0].equals("I2C"))
    {
      commandDescriptor = map.get(cmdch[0]);      // I2C commands have bus number appended after "/"
    }
    else
    {
      commandDescriptor = map.get(command);    // For all other commands get channel from map
    }

    return commandDescriptor;
  }
    
  public static void main( String[] args )
  {
    Sca.descriptors.forEach(reg->System.out.println(reg.getName()));  
  }

}

