package cbsw.lhcb.pdmdb;

import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.file.*;
import java.nio.file.attribute.*;
import java.util.prefs.*;
import java.util.*;
import java.time.LocalTime;

/**
<p>A class that tests a socket connection.</p>
@author Steve Wotton, Cavendish Lab (HEP), 2018-
**/
public class TesterTester
{
Preferences prefs;

static final int bufferSize = 4096;
static ByteBuffer readBuffer;
static ByteBuffer writeBuffer;

static ServerSocketChannel serverSocketChannel;
static SocketChannel socketChannel;
static ReadableByteChannel ichannel;
static WritableByteChannel ochannel;
    
static
{
  // Proxy message buffer
  readBuffer = ByteBuffer.allocate(bufferSize);
  readBuffer.order(ByteOrder.LITTLE_ENDIAN);
  writeBuffer = ByteBuffer.allocate(bufferSize);
  writeBuffer.order(ByteOrder.LITTLE_ENDIAN);
}
    
public TesterTester(){}

/**
<p>Reads a message from the input channel.</p>
@param timeout Socket timeout in milliseconds. 0 for no timeout.
@return The message split into tokens at the | character
 */
public static String [] readMessage(int timeout)
{

  try
  {
    socketChannel.socket().setSoTimeout(timeout);
    readBuffer.clear();
    ichannel.read(readBuffer);
    readBuffer.flip();
    byte [] rdMsg = new byte[readBuffer.limit()];
    readBuffer.get(rdMsg);
    String msg = new String(rdMsg);
    System.err.println("<["+LocalTime.now().toString()+"] "+msg);
    return msg.split("\\|");
  }
  catch (Exception ex) {throw new RuntimeException("TesterTester.readMessage(): "+ex.toString());}
}

/**
<p>Cleans out any pending messages.</p>
<p>Reads socket until it times out indicating no more data.</p>
 */
public static void readFlush()
{
  while (true)
  {
     String [] msg = readMessage(500);
     if (msg == null) break;
     StringBuffer str = new StringBuffer();
     for (String token : msg) str.append(token+" ");
     str.append("\n");
     System.err.println("TesterTester.readFlush(): "+str);
  }
}

/**
<p>Creates a server socket channel.</p>
@param url A URL to pass the port number
 */
public static void makeServerSocket(java.net.URL url)
{
  try
  {
    int port = url.getPort();
    if ( port < 1024 ) return; // Disallow privileged port numbers

    serverSocketChannel = ServerSocketChannel.open();
    serverSocketChannel.bind(new java.net.InetSocketAddress(port));
  }
  catch ( Exception ex ) { System.err.println( "TesterTester.makeServerSocket(): [" + url.toString() + "] " + ex.toString() ); }

  return;
}

/**
<p>An example program that replies ACK_OK to any message.</p>
<p>Creates and listens on a server socket. When a connection is made,
responds with ACK_OK to all messages.</p>
<p>When the connection is broken, waits for a new connection.</p>
@param args Command-line parameters
*/
public static void main( String[] args )
{
  try
  {
    TesterTester tester = new TesterTester();
    tester.makeServerSocket(new java.net.URL("http://:7999"));
    while (true)
    {
      try
      {
        socketChannel = serverSocketChannel.accept();

        ichannel = Channels.newChannel(socketChannel.socket().getInputStream());
        ochannel = Channels.newChannel(socketChannel.socket().getOutputStream());

        while (true)
        {
          String [] response = readMessage(0);
	  int seq = Integer.valueOf(response[0]);
          writeBuffer.clear();
	  writeBuffer.put((seq+"|ACK_OK").getBytes());
	  writeBuffer.flip();
	  ochannel.write(writeBuffer);
        }
      }
      catch ( Exception ex ) { System.err.println( "TesterTester.main(): [while] " + ex.toString() ); }  
    }
  }
  catch ( Exception ex ) { System.err.println( "TesterTester.main(): " + ex.toString() ); }
}

}
