// API for Digilent Adept interface

package cbsw.adept;

import javax.swing.*;
import java.awt.event.*;
import javax.swing.Timer;
import java.util.concurrent.locks.*;
import java.util.prefs.*;
import java.util.prefs.Preferences;
import java.util.prefs.BackingStoreException;
import java.util.*;
import java.io.*;
import java.net.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import cbsw.TapBuilder;
import cbsw.RegisterIoException;

public class JniAdeptDepp implements cbsw.RegisterIo
{

private BitSet regBits;
static private ReentrantLock djtgLock;
static private ReentrantLock deppLock;

static
{
  System.loadLibrary("JniAdeptDepp");

  djtgLock = new ReentrantLock();
  deppLock = new ReentrantLock();
}

@Deprecated
static ActionListener importHWAL = new ActionListener () { public void actionPerformed(ActionEvent e)
{
   importHardware("http://www.hep.phy.cam.ac.uk/lhcb/RICHEC/xml/XilinxHardwarePrefs.xml");
   return;
}
};

static ActionListener enumerateAL = new ActionListener () { public void actionPerformed(ActionEvent e)
{
   showEnumeratedDevices();
   return;
}
};

  public static void addMenuItems(JMenu m)
  {
/*
    JMenuItem menuItem = new JMenuItem("Import hardware...");
    menuItem.addActionListener( importHWAL );
    m.add(menuItem);
*/
      
    JMenuItem menuItem = new JMenuItem("Enumerate");
    menuItem.addActionListener( ae -> showEnumeratedDevices() );
    m.add(menuItem);
  }

public class DeppException extends cbsw.RegisterIoException
{
  public DeppException()
  {
      super(JniAdeptDepp.dmgr.getLastError(),JniAdeptDepp.dmgr.getLastErrorName(JniAdeptDepp.dmgr.getLastError()),JniAdeptDepp.dmgr.getLastErrorDescription(JniAdeptDepp.dmgr.getLastError()));
  }
};

private Timer timer;
private String deviceString;

ActionListener timerAL = new ActionListener () { public void actionPerformed(ActionEvent e)
{
// Open the device if not already open

  try
  {
    close();
    open();
  }
  catch ( DeppException de ){System.out.println("JniAdeptDepp.timerAL(): Exception opening "+deviceString);}

  return;
}
};

//    ReentrantLock lock;

    public JniAdeptDepp( String dev )
    {
        regBits = new BitSet(2048);
        deviceString = dev;

	//	lock = new ReentrantLock();

        timer = new Timer(1000, timerAL );
        timer.setInitialDelay(0);
        timer.setCoalesce( true );
        
    }
    public JniAdeptDepp( String dev, ActionListener al )
    {
        regBits = new BitSet(2048);
        deviceString = dev;

	//	lock = new ReentrantLock();

        timer = new Timer(1000, al );
        timer.setInitialDelay(0);
        timer.setCoalesce( true );
        
    }

    public void lock(){deppLock.lock();}
    public void unlock(){deppLock.unlock();}

// DMGR functions

    public static class dmgr
    {
      public static native int open( String dev );
      public static native int close();
      public static native int enumerate();
      public static native String getEnumeratedDevice( int idx );
      public static native String getCapabilities( int idx );
      public static native int getLastError();
      public static native String getLastErrorName(int erc);
      public static native String getLastErrorDescription(int erc);
    }

// DEPP functions

    public static class depp
    {
      public static native int enable();
      public static native int disable();
      public static native int getSet( byte [] addr, byte [] data );
      public static native int getRepeat( byte addr, byte [] buf );
      public static native int putSet( byte [] buf );
      public static native int putRepeat( byte addr, byte [] buf );
    }

// DJTG functions

    public static class djtg
    {
      public static native boolean enable();
      public static native boolean disable();
      public static native String getProperties(int port);
      public static native boolean putTmsTdi(int bits, byte [] snd, byte [] rcv);
      public static native boolean putTmsTdi(int bits, byte [] snd);
      public static native boolean putTms(int bits, byte [] snd, byte [] rcv, boolean tdi);
      public static native boolean putTms(int bits, byte [] snd, boolean tdi);
      public static native boolean putTdi(int bits, byte [] snd, byte [] rcv, boolean tms);
      public static native boolean putTdi(int bits, byte [] snd, boolean tms);
      public static native boolean clock(boolean tms, boolean tdi, int clks);
      public static byte [] packAndGo(TapBuilder tap, boolean rcv)
      {
        int bits = tap.length();
        if (bits < 1) return null;

	//        tap.setDebug(true);
        tap.show();

        ByteBuffer bb = tap.tmsTdoToByteBuffer();
	tap.setDebug(false);
        boolean rc = false;
        byte [] tdi = null;
        if (rcv)
	{
          tdi = new byte [bb.capacity()/2];
          rc = JniAdeptDepp.djtg.putTmsTdi(bits,bb.array(),tdi);
	}
        else
	{
          rc = JniAdeptDepp.djtg.putTmsTdi(bits,bb.array());
	}
        if (!rc) System.err.println("JniAdeptDepp.djtg.packAndGo(): putTmsTdi() failed.");
        return tdi;
      }
    }

    public boolean OK(){ return true; }

    public static void showEnumeratedDevices()
    {
      int ndepp = JniAdeptDepp.dmgr.enumerate();
      if (ndepp == 0) System.out.println("JniAdeptDepp.showEnumeratedDevices(): No devices found.");
      for ( int idepp=0;idepp<ndepp;idepp++)
      {
        System.out.println(JniAdeptDepp.dmgr.getEnumeratedDevice(idepp));
      }
    }
    
    public void connect()
    {
      timer.start();
      return;
    }

    public void disconnect()
    {
      timer.stop();
      close();
      return;
    }

    public void open() throws DeppException
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          if ( JniAdeptDepp.dmgr.open( deviceString ) != 1 ){throw new DeppException();}
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

    public void close()
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          int rc = JniAdeptDepp.dmgr.close();
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

// Get single byte

    public void get( byte addr, byte [] buf ) throws DeppException
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          byte [] addrArray = new byte[1]; addrArray[0] = addr;
          if ( JniAdeptDepp.depp.getSet( addrArray, buf ) != 1 ){throw new DeppException();}
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

// Get multiple data

    public void get( byte [] addr, byte [] data ) throws DeppException
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          if ( JniAdeptDepp.depp.getSet( addr, data ) != 1 ){throw new DeppException();}
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

// Put single addr,byte

    public void put( byte addr, byte data ) throws DeppException
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          byte [] buf = new byte[2]; buf[0] = addr; buf[1] = data;
          if ( JniAdeptDepp.depp.putSet( buf ) != 1 ){throw new DeppException();}
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

// Buffer contains addr,data pairs

    public void put( byte [] buf ) throws DeppException
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          if ( JniAdeptDepp.depp.putSet( buf ) != 1 ){throw new DeppException();}
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

    public void put( int base, int addr, int val ) throws RegisterIoException
    {
      byte [] buf = new byte[4];
      buf [0] = (byte)(val&0xff);
      buf [1] = (byte)((val>>8)&0xff);
      buf [2] = (byte)((val>>16)&0xff);
      buf [3] = (byte)((val>>24)&0xff);

      try
      {
        put( base, addr, buf );
      }
      catch ( RegisterIoException ioe ) { throw ioe; }
    }

    public void put( int base, int addr, short val ) throws RegisterIoException
    {
      byte [] buf = new byte[2];
      buf [0] = (byte)(val&0xff);
      buf [1] = (byte)((val>>8)&0xff);

      try
      {
        put( base, addr, buf );
      }
      catch ( RegisterIoException ioe ) { throw ioe; }
    }

    public void put( int base, int addr, byte val ) throws RegisterIoException
    {
      byte [] buf = new byte[1];
      buf [0] = val;

      try
      {
        put( base, addr, buf );
      }
      catch ( RegisterIoException ioe ) { throw ioe; }
    }

// Push for depp2 transaction
   public void push(int base, int len, int addr, byte [] buf, boolean waitFlag) throws DeppException
   {
     buf[0] = (byte)(len&0xff); buf[1] = (byte)((len>>8)&0xff); // Length
     buf[2] = (byte)(addr&0xff); buf[3] = (byte)((addr>>8)&0x7f); // Address
     if (waitFlag) buf[3] |= (byte)0x80;
     try { push((byte)(base&0xff),buf); } catch (DeppException de ){ throw de; }
   }

// Push buf[0],buf[1],...,buf[buf.legth-1] to addr

    public void push( byte base, byte [] buf ) throws DeppException
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          if (JniAdeptDepp.depp.putRepeat( base, buf ) != 1) {throw new DeppException();}
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

// Pop buf[0],buf[1],...,buf[buf.legth-1] from addr

    public void pop( byte base, byte [] buf ) throws DeppException
    {
        try
	{
	  JniAdeptDepp.deppLock.lock();
          if ( JniAdeptDepp.depp.getRepeat( base, buf ) != 1 ){throw new DeppException();}
	}
	finally { JniAdeptDepp.deppLock.unlock(); }

        return;
    }

    public void put1(int base, int addr, byte [] regs) throws DeppException
    {
      byte [] hdr = {(byte)(base&0xff), (byte)(addr&0xff), (byte)((base+1)&0xff), (byte)((addr>>8)&0xff)};
      try
      {
        JniAdeptDepp.deppLock.lock();
        if (JniAdeptDepp.depp.putSet( hdr ) != 1){ throw new DeppException(); }
        if (JniAdeptDepp.depp.putRepeat( (byte)((base+2)&0xff), regs ) != 1){ throw new DeppException(); }
      }
      finally { JniAdeptDepp.deppLock.unlock(); }
      return;
    }

// Depp2 version of put (old put renamed put1)
    public void put(int base, int addr, byte [] regs) throws cbsw.RegisterIoException
    {
    // Write the header at the beginning of the buffer
      int idx = 0;
      int dataLength = regs.length - 4;
      regs[idx++] = (byte)( dataLength & 0xff );
      regs[idx++] = (byte)( (dataLength>>8)&0xff );
      regs[idx++] = (byte)( addr & 0xff );
      regs[idx++] = (byte)( (addr>>8)&0xff );

      try
      {
        JniAdeptDepp.deppLock.lock();
        if (JniAdeptDepp.depp.putRepeat( (byte)(base&0xff), regs ) != 1){ throw new DeppException(); }
      }
      finally { JniAdeptDepp.deppLock.unlock(); }
      return;
    }

// Depp2 version of put
    public void put(int base, java.nio.ByteBuffer bb) throws cbsw.RegisterIoException
    {
      try
      {
        JniAdeptDepp.deppLock.lock();
        if (JniAdeptDepp.depp.putRepeat( (byte)(base&0xff), bb.array() ) != 1){ throw new DeppException(); }
      }
      finally { JniAdeptDepp.deppLock.unlock(); }
      return;
    }

    public void getBytes(int base, int addr, byte [] regs) throws DeppException
    {
      byte [] hdr = {(byte)(base&0xff), (byte)(addr&0xff), (byte)((base+1)&0xff), (byte)((addr>>8)&0xff)};
      try
      {
        JniAdeptDepp.deppLock.lock();
        if (JniAdeptDepp.depp.putSet( hdr ) != 1){ throw new DeppException(); }
        if (JniAdeptDepp.depp.getRepeat( (byte)((base+2)&0xff), regs ) != 1){ throw new DeppException(); }
      }
      finally { JniAdeptDepp.deppLock.unlock(); }
      return;
    }

// Depp2 version of getBytes

    public void get(int base, int addr, byte [] regs) throws cbsw.RegisterIoException
    {
      byte [] hdr = new byte [4];
      int idx = 0;
    // Write the header
      int dataLength = regs.length;
      hdr[idx++] = (byte)( dataLength & 0xff );
      hdr[idx++] = (byte)( (dataLength>>8)&0xff );
      hdr[idx++] = (byte)( addr & 0xff );
      hdr[idx++] = (byte)( (addr>>8)&0xff );
    
      try
      {
        JniAdeptDepp.deppLock.lock();
        if (JniAdeptDepp.depp.putRepeat( (byte)(base&0xff), hdr ) != 1){ throw new DeppException(); }
        if (JniAdeptDepp.depp.getRepeat( (byte)(base&0xff), regs ) != 1){ throw new DeppException(); }
      }
      finally { JniAdeptDepp.deppLock.unlock(); }

      return;
    }

// This version of get is intended to be called after the header has been sent
    public void get(int base, byte [] regs) throws cbsw.RegisterIoException
    {
      try
      {
        JniAdeptDepp.deppLock.lock();
        if (JniAdeptDepp.depp.getRepeat( (byte)(base&0xff), regs ) != 1){ throw new DeppException(); }
      }
      finally { JniAdeptDepp.deppLock.unlock(); }

      return;
    }

  public byte [] getByteArray(int size)
  {
   byte [] cfg = new byte [size];
   System.arraycopy(regBits.toByteArray(),0,cfg,0,regBits.toByteArray().length);   return cfg;
  };

  public byte [] getByteArray(int size, int off)
  {
   byte [] cfg = new byte [size];
   int bsLen = regBits.toByteArray().length;
   //   System.out.println("size="+size+" off="+off+" bsLen="+bsLen );
   if (bsLen < off) return cfg; // All bits zero
   int copySize = (bsLen < (size+off)) ? bsLen-off : size;
   //   System.out.println("copySize="+copySize );
   System.arraycopy(regBits.toByteArray(),off,cfg,0,copySize);
   return cfg;
  };

  static public void importHardware(String url)
  {

    System.out.println("JniAdeptDepp.importHardware(): Importing "+url);
    
// Make a URL representing the source

    try
    {

      URL hwURL = new URL(url);
      if ( hwURL == null )
      {
        System.err.println("Error forming data source URL: " + url );
        return;
      }
      InputStream hwStream = hwURL.openStream();
      Preferences.userRoot().importPreferences(hwStream);
      hwStream.close();

    }
    catch ( MalformedURLException ue ){ System.err.println("Warning: " + ue.getMessage() ); }
    catch ( EOFException eofe ){ System.err.println( "End of file" ); }
    catch ( IOException ioe ){ System.err.println("Warning: " + ioe.getMessage() ); }
    catch (InvalidPreferencesFormatException ex) { ex.printStackTrace(); }

    return;
  }

@Deprecated
  static public JMenu makeDeviceMenu(String title, ActionListener aL)
  {
    JMenu menu = new JMenu(title);
    try
    {
      for ( String dna : Preferences.userRoot().node("Hardware/Xilinx").childrenNames() )
      {
        JMenuItem menuItem = new JMenuItem(dna);
        menuItem.addActionListener(aL);
        menu.add(menuItem);
      }
    } catch (BackingStoreException ex) {}

    return menu;
  }

@Deprecated
  static public String selectDevice(String filter)
  {
    ArrayList<String> hwList = new ArrayList<String>();
    if (filter == null) filter = new String("chimaera2v3");
    try
    {
      for ( String dna : Preferences.userRoot().node("Hardware/Xilinx").childrenNames() )
      {
	Preferences hwPrefs = Preferences.userRoot().node("Hardware/Xilinx").node(dna);
	if (hwPrefs.get("typeName","chimaera2v3").equals(filter))
	{
	  hwList.add(dna);
	}
      }
    } catch (BackingStoreException ex) {}

    Object [] hwArray = hwList.toArray();

    return (String) JOptionPane.showInputDialog(null, "Select hardware", "EC tester", JOptionPane.QUESTION_MESSAGE, null, hwArray, hwArray[0]);
  }

  static public String [] selectConnectedDevice()
  {
    ArrayList<String> hwList = new ArrayList<String>();
    int ndepp = JniAdeptDepp.dmgr.enumerate();
    for ( int idepp=0;idepp<ndepp;idepp++)
    {
      hwList.add(JniAdeptDepp.dmgr.getEnumeratedDevice(idepp));
      //      System.out.println(JniAdeptDepp.dmgr.getCapabilities(idepp));
    }

    Object [] hwArray = hwList.toArray();

    if (hwArray.length < 1) return null;

    String selectedDevice = (String) JOptionPane.showInputDialog(null, "Select hardware", "Enumerated devices", JOptionPane.QUESTION_MESSAGE, null, hwArray, hwArray[0]);

    if (selectedDevice==null) return null;

    String [] deviceTokens = selectedDevice.split("/");

    if (deviceTokens.length != 2) return null;

    return deviceTokens;
  }

@Deprecated
  static public JComboBox<String> makeDeviceComboBox(String selectedName, ActionListener aL, String filter)
  {
    if (filter == null) filter = new String("chimaera2v3");
    JComboBox<String> cB = new JComboBox<String>();
    if (aL != null) cB.addActionListener(aL);
    try
    {
      for ( String dna : Preferences.userRoot().node("Hardware/Xilinx").childrenNames() )
      {
	Preferences hwPrefs = Preferences.userRoot().node("Hardware/Xilinx").node(dna);
	if (hwPrefs.get("typeName","chimaera2v3").equals(filter))
	{
	  cB.addItem(dna);
	  if (dna.equals(selectedName))
	  {
	    cB.setSelectedItem(dna);
	  }
	}
      }
    } catch (BackingStoreException ex) {}

    return cB;
  }

@Deprecated
  static public Preferences getDevicePreferences(Class cl, String node)
  {
    Preferences mainPrefs = Preferences.userNodeForPackage(cl).node(node);
    String hw = mainPrefs.get("hardwareId","default");
    return Preferences.userRoot().node("Hardware/Xilinx").node(hw);
  }
}
