package cbsw.lhcb.pdmdb;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.xml.stream.*;
import java.io.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.*;
import cbsw.lhcb.pdmdb.*;
import cbsw.gui.*;
import javax.xml.bind.annotation.*;
import java.lang.reflect.*;

/**
<p>The MuDaq class ecapsulates the interactions with a PDMDB module through the muDaq hardware interface.</p>
 **/
@XmlRootElement(name="mudaq")
@XmlAccessorType(XmlAccessType.NONE)
public class MuDaq
{
Preferences prefs;
Preferences typePrefs;
JniAdeptDepp depp;
RegisterIo io;
JComboBox<String> regNameCB;
JTextField regValueTF;
@XmlElementWrapper(name="deppTables")
@XmlElement(name="deppTable")
ArrayList<RegisterIoTable> tableList;
@XmlElement(name="mgt")
ArrayList<MGTControl> mgts;
@XmlElement(name="pdmdb")
PdmdbMaster pdmdbMaster;
JFrame masterI2cConfigurationJF;
JFrame masterConfigurationJF;
JFrame masterLinkJF;
PdmdbModuleTester tester;
JFrame testerJF;
JMenuBar menuBar;
GuiJPanel jP;
@XmlElement(name="gbtxI2c")
GbtxDepp gbtxI2c; // I2C master configuration
@XmlElement(name="gbtx")
@javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter(GbtxAdapter.class)
GbtxParent gbtx; // I2C or IC configuration (set in xml)    
@XmlElement(name="drp")
DrpPanel drp;

// Static member containing the MuDaq instance referenced by exec()
static MuDaq muDaq;
    
ScanControl scanControl;
JFrame statusFrame;

static Csr cfg;
static Csr status;
static
{
  MuDaq.cfg = Csr.unmarshall(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp").get("csr-xml","https://www.hep.phy.cam.ac.uk/lhcb/muDAQ/xml/mudaq-csr-config.xml"));
  MuDaq.status = Csr.unmarshall(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp").get("status-xml","https://www.hep.phy.cam.ac.uk/lhcb/muDAQ/xml/mudaq-csr-status.xml"));
}

/**
<p>No-argument constructor called by the unmarshaller.</p>
**/
public MuDaq()
{
  scanControl = new ScanControl();
}

/**
<p>Convenience method that builds a GUI panel.</p>
@return The panel.
**/
public GuiJPanel makePanel()
{
  jP = new GuiJPanel(prefs,"");
    
  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  gbC.fill = GridBagConstraints.NONE;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(1,1,1,1);
  gbC.weightx = 1.0f;
  gbC.weighty = 1.0f;

  JTabbedPane mgtTP = new JTabbedPane();
  int i=0;
  for (MGTControl mgt: mgts)
  {
    GuiJPanel mgtP = mgt.makePanel();
    mgtP.setName("MGT."+mgt.getId());
    mgtTP.add(mgtP);
    i++;
  }
  gbC.gridwidth = 2;
  jP.add(mgtTP, gbC);

  gbC.gridwidth = 2;
  //  jP.add(new ControlPanel(mB,depp), gbC);
  ControlPanel cP = new ControlPanel();
  jP.add(cP.makePanel(), gbC);

  return jP;
}

public void setGbtx(GbtxParent gbtx){this.gbtx = gbtx;}

/**
<p>Called after unmarshalling to initialise the instance and its children.</p>
@param depp The I/O interface.
@param prefs The preference path.
**/
public void init(JniAdeptDepp depp, Preferences prefs)
{
  this.depp = depp;
  this.io = depp;
  this.prefs = prefs;
  
  for (RegisterIoTable dT: tableList) dT.init(io);
  statusFrame = GuiJPanel.makeFrame((new StatusPanel()).makePanel(),"Status");
  statusFrame.pack();
  
  pdmdbMaster.init(io); PdmdbMaster.setStaticPdmdbMaster(pdmdbMaster);
  for (MGTControl mgt: mgts) mgt.init(io,cfg);
  
  gbtxI2c.setName("tcm.gbtx.i2c");
  gbtxI2c.setPreferencesRoot(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp"));
  masterI2cConfigurationJF = GuiJPanel.makeFrame(gbtxI2c.makePanel(),"TCM GBTX I2C");
  masterI2cConfigurationJF.pack();
  gbtxI2c.init(io);

  gbtx.setName("tcm.gbtx");
  gbtx.setPreferencesRoot(Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp"));
  masterConfigurationJF = GuiJPanel.makeFrame(gbtx.makePanel(),"TCM GBTX IC");
  masterConfigurationJF.pack();
  gbtx.init(io);

  drp.init(depp); // DrpPanel not yet converted to RegisterIo

  scanControl.init(io);

  masterLinkJF = GuiJPanel.makeFrame(pdmdbMaster.makePanel(),"Master Link");
  masterLinkJF.pack();

  tester = new PdmdbModuleTester();
  tester.init(prefs);
  /*
  testerJF = GuiJPanel.makeFrame(tester.makePanel(),"Tester commands");
  testerJF.pack();
  */
}

public static void setStaticMuDaq(MuDaq muDaq){MuDaq.muDaq = muDaq;}

/**
<p>Configures the muDaq registers.</p>
<p>Called by reflection as part of a sequence.</p>
@param args Command arguments.
@return Command response.
**/
public static String configure(String [] args)
{
  MuDaq.muDaq.setMgtStateFromPreferences();
  MuDaq.muDaq.setStateConfigure();
  try { MuDaq.muDaq.io.put(cfg.getBaseAddress(),0,cfg.toByteArray(4)); }
  catch(Exception ex){System.err.println("Mudaq.configure() Exception "+ex.getMessage()); return "FAIL|MuDaq.configure";}
  reset(null);

  try {Thread.sleep(100);}catch(InterruptedException e){}
  // Check the protocol register to decide whether configuration was successful
  //  MuDaq.muDaq.cfg.cache(MuDaq.muDaq.io);
  //  if (!cfg.get("net.ip.protocol",csr).equals("f2")) return "FAIL|MuDaq.configure";
  return "OK|MuDaq.configure";
}

/**
<p>Reset muDaq.</p>
<p>Called by reflection as part of a sequence.</p>
@param args Command arguments.
@return Command response.
**/
  public static String reset(String [] args)
  {
    try
    {
      cfg.put("daq-reset",true,MuDaq.muDaq.io);
      cfg.put("daq-reset",false,MuDaq.muDaq.io);
      
      cfg.put("counter-reset",true,MuDaq.muDaq.io);
      cfg.put("counter-reset",false,MuDaq.muDaq.io);
      
      cfg.put("gbt.reset-all",true,MuDaq.muDaq.io);
      cfg.put("gbt.reset-all",false,MuDaq.muDaq.io);
    }
    catch ( Exception ex ){ System.err.println("MuDaq.reset() Exception "+ex.getMessage()); return "FAIL|MuDaq.reset";}
    return "OK|MuDaq.reset";
  }

/**
<p>Configure the master GBTX</p>
<p>Called by reflection as part of a sequence.</p>
<p>To allow configuration of un-fused master GBTX this function
selects either the I2C interface (&lt;gbtxI2c&gt; in XML) or the
generic interface (&lt;gbtx&gt; in XML) which can be set to either
I2C or IC</p>
@param args Command arguments.
@return Command response.
**/
public static String configureMasterLink(String [] args)
{
  try
  {
    GbtxParent gbtx = args[0].equals("i2c") ? MuDaq.muDaq.gbtxI2c : MuDaq.muDaq.gbtx;
    
    gbtx.readConfigurationFile();
    gbtx.configure();
    if (!args[0].equals("i2c")) Thread.sleep(500);
    gbtx.verify();

    PdmdbModuleTester.tcmGbtxId = gbtx.get("fuse.serial");
    String state = gbtx.get("powerup-fsm-state");
    String a5 = gbtx.get("a5");

    TesterSequence.testSummaryXml.write(String.format("<tcm testerModule=\"tcm\" gbtxId=\"%s\" gbtxState=\"%s\" />\n",PdmdbModuleTester.tcmGbtxId,state));
  
    if (!a5.equals("a5")) return "FAIL|MuDaq.configureMasterLink|a5!="+a5;
    if (!state.equals("18")) return "FAIL|MuDaq.configureMasterLink|NOT_IDLE|FSM="+state; // 18(hex) is Idle state

    Thread.sleep(500);
    
    MuDaq.muDaq.status.cache(MuDaq.muDaq.io);
    if (status.get("gbt.0.mgt.rx.header").equals("05"))
    {
      return "OK|MuDaq.configureMasterLink|TCM.GBTX|0x"+PdmdbModuleTester.tcmGbtxId;
    }
    else
    {
      return "FAIL|MuDaq.configureMasterLink|HEADER_INVALID";
    }
  }
  catch (Exception ex){System.err.println("MuDaq.configureMasterLink: Exception: "+ex.getMessage()); return "FAIL|MuDaq.configureMasterLink|Exception";}
  
}
    
/**
<p>Switch muDaq K7 transceiver on or off.</p>
<p>Called by reflection as part of a sequence.</p>
@param args [0]=MGT ID, [1]=TX on, [2]=RX on.
@return Command response.
**/
public static String setMgtOnOff(String [] args)
{
  try
  {
    int mgtId = Integer.decode(args[0]).intValue();
    boolean txOn = Integer.decode(args[1]).intValue()==1;
    boolean rxOn = Integer.decode(args[2]).intValue()==1;
    cfg.put("gt."+mgtId+".rxon", rxOn, MuDaq.muDaq.io);
    cfg.put("gt."+mgtId+".txon", txOn, MuDaq.muDaq.io);
    return "OK|MuDaq.setMgtOnOff";
  }
  catch (Exception ex){throw new RuntimeException("MuDaq.setMgtOnOff: Exception: "+ex.getMessage());}
}
    
/**
<p>Reset muDaq MGT transceiver.</p>
<p>Called by reflection as part of a sequence.</p>
@param args [0]=MGT ID, [1]=TX reset, [2]=RX reset.
@return Command response.
**/
public static String setMgtReset(String [] args)
{
  try
  {
    int mgtId = Integer.decode(args[0]).intValue();
    boolean txReset = Integer.decode(args[1]).intValue()==1;
    boolean rxReset = Integer.decode(args[2]).intValue()==1;
    cfg.put("gt."+mgtId+".rxreset", rxReset, MuDaq.muDaq.io);
    cfg.put("gt."+mgtId+".txreset", txReset, MuDaq.muDaq.io);
    return "OK|MuDaq.setMgtReset";
  }
  catch (Exception ex){throw new RuntimeException("MuDaq.setMgtReset: Exception: "+ex.getMessage());}
}

public JFrame getStatusJF(){return statusFrame;}
public JFrame getMasterI2cConfigurationJF(){return masterI2cConfigurationJF;}
public JFrame getMasterConfigurationJF(){return masterConfigurationJF;}
public JFrame getMasterLinkJF(){return masterLinkJF;}
public JFrame getTesterJF(){return testerJF;}
public PdmdbModuleTester getTester(){return tester;}
public PdmdbMaster getPdmdbMaster(){return pdmdbMaster;}

/**
<p>Convenience fuction that creates a menu.</p>
@param title The menu title.
@return The menu.
**/
public JMenu makeMasterLinkMenu(String title)
{
  JMenu menu = new JMenu(title);

  JMenuItem menuItem = new JMenuItem("Master configuration (I2C)...");
  menuItem.addActionListener( ae->masterI2cConfigurationJF.setVisible(true) );
  menu.add(menuItem);

  menuItem = new JMenuItem("Master configuration...");
  menuItem.addActionListener( ae->masterConfigurationJF.setVisible(true) );
  menu.add(menuItem);

  return menu;
}
	
/**
<p>Update the cached configuration register with setting from preferences.</p>
**/
void setStateConfigure()
{
  setStateFromPreferences();
  cfg.set("tfc.enable",true);
  cfg.set("dcms.enable",true);

  return;
};

void setStateShutdown()
{
  cfg.set("tfc.enable",false);
  cfg.set("dcms.enable",false);
  for ( MGTControl mgt : mgts ) mgt.setStateOff();

  return;
};

void setMgtStateFromPreferences()
{
  for ( MGTControl mgt : mgts ) mgt.setStateFromPreferences();
  return;
}

  public void setStateFromPreferences()
  {

    System.out.println("MuDaq.setStateFromPreferences(): Setting state from prefs path "+prefs.absolutePath());

    try
    {
      prefs.sync();
    } catch (BackingStoreException ex) { ex.printStackTrace(); }

// DAQ parameters
    cfg.set("daq.latency-compensation", prefs, "1b");

// Enable TFC commands
    cfg.set("tfctog-enable", prefs, "f");

// GT settings

    for ( MGTControl mgt : mgts ) mgt.setStateFromPreferences();

// Pulser settings
    cfg.set("pulser.count", prefs);
    cfg.set("pulser.delay", prefs);

// TX test pattern
    cfg.set("gt.txpattern",prefs,"AA");

    return;
  };

/**
<p>Trigger GBTX e-fusing.</p>
**/
public void fuseGbtx()
{
  java.lang.Runnable pulse = () -> toggle("gbtx.efuse-program");
  gbtxI2c.fuse(pulse);
  return;
}

/**
<p>Toggles a configuration register (on, then off).</p>
@param regName The register name as described in XML.
**/
public void toggle(String regName)
{
  try
  {
    cfg.put(regName,true,io);
    cfg.set(regName,false);
  }
  catch ( Exception ex ){ throw new RuntimeException("GbtxDepp.write(): "+ex.getMessage()); }
}
    
/**
<p>Fuse GBTX registers.</p>
<p>Called by reflection as part of a sequence.</p>
<p>This function is called as part of the module testing sequence.</p>
<p>Caution - it irreversibly burns the TCM GBTX e-fuses.</p>
<p>The GBTX must have CONFIGSEL=1 (I2C) for this procedure and GBTFUSE3V3 must be enabled.</p>
@param args [0]="enable" to allow fusing, "ignore" to disallow fusing, "force" to force re-fusing an already fused GBTX.
@return Command response.
**/
public static String fuseMasterGbtx(String [] args)
{
  try
  {
// Check that I2C is OK by reading the "a5" register
// Return "FAIL" immediately if the value is not read correctly.
    
    MuDaq.muDaq.gbtxI2c.verify();
    BitSet csrBitSet = MuDaq.muDaq.gbtxI2c.getCsrBitSet(); 
    String a5 = GbtxRegisterList.get("a5").get(csrBitSet);
    if (!a5.equals("a5")) return "FAIL|MuDaq.fuseMasterGbtx|a5!="+a5;

// Check register 366 to see the TCM appears to be already e-fused.
// If so, register 366 will contain 0x07, return "OK" immediately.
// If it is 0x00 continue with e-fusing.
// Anything else, bail out immediately.

    if (MuDaq.muDaq.gbtxI2c.gbtxReadBuf.length<367) return "FAIL|MuDaq.fuseMasterGbtx|Wrong register buffer length";
    byte fused = MuDaq.muDaq.gbtxI2c.gbtxReadBuf[366];
    if (fused==7 && !args[0].equals("force")) return "OK|MuDaq.fuseMasterGbtx|efuse.update.enable="+fused;
    if (fused!=0 && fused!=7) return "FAIL|MuDaq.fuseMasterGbtx|efuse.update.enable="+fused;

// Do the e-fusing. For extra protection, a preference must be set to enable it.
// Also the first argument to this method must be set to "enabled".

    java.lang.Runnable pulse = () -> MuDaq.muDaq.toggle("gbtx.efuse-program");
    Preferences prefs = Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp");
    java.time.LocalDateTime fusingExpiry = java.time.LocalDateTime.parse(prefs.get("fusing-expiry","2000-01-01T12:00:00"));
    java.time.LocalDateTime now = java.time.LocalDateTime.now();
    long hoursUntilExpiry = now.until(fusingExpiry,java.time.temporal.ChronoUnit.HOURS);
    if (args[0].equals("enable") || args[0].equals("force"))
    {
      if (hoursUntilExpiry>0)
      {
        MuDaq.muDaq.cfg.put("gbtx.efuse-power-enable","1",MuDaq.muDaq.io);
        MuDaq.muDaq.gbtxI2c.fuse(pulse);
        MuDaq.muDaq.cfg.put("gbtx.efuse-power-enable","0",MuDaq.muDaq.io);
        return "OK|MuDaq.fuseMasterGbtx|Fusing done.";
      }
      else
      {
        return "FAIL|MuDaq.fuseMasterGbtx|Fusing skipped. Fusing token has expired.";
      }
    }
    else
    {
      return "OK|MuDaq.fuseMasterGbtx|Fusing skipped. Not enabled.";
    }
  }
  catch (Exception ex)
  {
    System.err.println("MuDaq.fuseMasterGbtx(): "+ex.getMessage());
    return "FAIL|MuDaq.fuseMasterGbtx|Exception during fusing";
  }

// To check that it was successful, the GBTX needs to be reset so that the fused values are loaded into the registers. This requires an interaction with the tester board so it is done as a later step of the sequence.

}

/**
<p>Prepare muDaq to measure bit error rate of e-links.</p>
<p>Called by reflection as part of a sequence.</p>
@param args Command parameters
@return Command response.
**/
public static String prepareBER(String [] args){ MuDaq.muDaq.prepareBER(); return "OK|MuDaq.prepareBER";}

public void prepareBER()
{
  try
  {
      
// Set a fixed pattern on the down links
    cfg.put("tfctog-enable","0",io);
    cfg.put("tfc-d","A",io);

// Reset the bitflip counters
    cfg.put("counter-reset",true,io);
    cfg.put("counter-reset",false,io);
  }
  catch (Exception ioe){ throw new RuntimeException("MuDaq.prepareBER|Exception "+ioe.getMessage()); }
}

/**
<p>Gets the current bit error rate counters.</p>
<p>Called by reflection as part of a sequence.</p>
@param args Command parameters
@return Command response.
**/
public static String getBER(String [] args)
{
  java.nio.ByteBuffer bb = MuDaq.muDaq.getBER();
  MuDaq.writeBERCounterXml(bb);
  
  return "OK|MuDaq.getBER";
}

public java.nio.ByteBuffer getBER()
{
  try
  {
    byte [] rdBuf = new byte [192];
    io.get(status.getBaseAddress(),256,rdBuf);
    java.nio.ByteBuffer bb = java.nio.ByteBuffer.wrap(rdBuf);
    bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);
    return bb;
  }
  catch (Exception ioe){ throw new RuntimeException("MuDaq.getBER|Exception "+ioe.getMessage()); }
}

/**
<p>Counts current bit error rate counters.</p>
<p>Called by reflection as part of a sequence.</p>
@param args Command parameters
@return Command response.
**/
  public static String countBitErrors(String [] args)
  {
    try
    {
      MuDaq.muDaq.prepareBER();

      Thread.sleep(500);

      java.nio.ByteBuffer bb = MuDaq.muDaq.getBER();
      MuDaq.writeBERCounterXml(bb);
    }
    catch (Exception ioe){ return "FAIL|MuDaq.countBitErrors|Exception "+ioe.getMessage(); }
    return "OK|MuDaq.countBitErrors";
  }

public static void writeBERCounterXml(java.nio.ByteBuffer bb)
{
    try
    {
      for (int mgt=0; mgt<6; mgt++ )
      {
	TesterSequence.testSummaryXml.write(String.format("<bitErrors testerModule=\"dtm%d\" gbtx=\"%d\" ",mgt/2,mgt%2));
	StringBuffer sb = new StringBuffer();
	for (int eport=0;eport<14;eport++)
	{
	  short count = bb.getShort();
	  sb.append(String.format("eport%d=\"%d\" ",eport,count));
	}
	bb.getInt(); // Swallow the padding
	TesterSequence.testSummaryXml.write(sb.toString());
	String timestamp = java.time.LocalDateTime.now().format(java.time.format.DateTimeFormatter.ofPattern("yyyyMMdd-A"));
	TesterSequence.testSummaryXml.write(String.format("timestamp=\"%s\" />\n",timestamp));
      }
    }
    catch (Exception ioe){ throw new RuntimeException( "MuDaq.writeBERCounterXml: Exception "+ioe.getMessage()); }
}

public class ControlPanel implements ActionListener
{
  public ControlPanel(){}
    
  public JPanel makePanel()
  {
    JPanel cP = new JPanel();
      
    cP.setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(2,2,2,2);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JButton button = null;

    JPanel jp = new JPanel();
    jp.setBorder( BorderFactory.createTitledBorder("Configuration") );

    regNameCB = new JComboBox<String>();
    regNameCB.setEditable(true);
    regNameCB.addItem("daq.latency-compensation");
    regNameCB.addItem("tfctog-enable");
    gbC.gridwidth = 1;
    jp.add(regNameCB,gbC);
    
    regValueTF = new JTextField(8);
    gbC.gridwidth = 1;
    jp.add(regValueTF,gbC);

    button = new JButton("Set");
    button.addActionListener(l->set());
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(button,gbC);

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    cP.add( jp, gbC );

    button = new JButton( "Shutdown" );
    button.setName("shutdown");
    button.addActionListener( this );
    gbC.gridwidth = 1;
    cP.add( button, gbC );

    button = new JButton( "Init" );
    button.setName("init");
    button.addActionListener( this );
    gbC.gridwidth = 1;
    cP.add( button, gbC );

    button = new JButton( "Enable" );
    button.setName("enable");
    button.addActionListener( this );
    gbC.gridwidth = 1;
    cP.add( button, gbC );

    button = new JButton( "Disable" );
    button.setName("disable");
    button.addActionListener( this );
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    cP.add( button, gbC );

    button = new JButton( "Reset" );
    button.addActionListener(ae->reset(null));
    gbC.gridwidth = 1;
    cP.add( button, gbC );

    button = new JButton( "Efuse" );
    button.addActionListener(ae->fuseGbtx());
    gbC.gridwidth = 1;
    cP.add( button, gbC );

    button = new JButton( "Trigger" );
    button.addActionListener(ae->toggle("trigger-tog"));
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    cP.add( button, gbC );

    jp = new JPanel();
    jp.setBorder( BorderFactory.createTitledBorder("Pulser") );

    jp.add(new JLabel("Count"));
    JTextField tf = jP.makeTF("pulser.count", "100", 10, "Pulser count", "Pulser count",true);
    tf.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 11));
    jp.add( tf );

    jp.add(new JLabel("Delay"));
    tf = jP.makeTF("pulser.delay", "10000", 10, "Pulser delay", "Pulser delay",true);
    tf.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 11));
    jp.add( tf );

    button = new JButton( "Pulse" );
    button.setName("pulse");
    button.addActionListener( this );
    jp.add( button );

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    cP.add(jp,gbC);

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    cP.add(scanControl.makePanel(),gbC);

//     tf = makeTF("gt.txpattern", "112233445566778899aabbccddee", 28, "Loopback pattern", "Loopback pattern", false);
//     tf.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 11));
//     gbC.gridwidth = GridBagConstraints.REMAINDER;
//     cP.add( tf.getParent(),gbC );

    jp = new JPanel();
    jp.setBorder( BorderFactory.createTitledBorder("TFC") );

    button = new JButton( "Trigger" );
    button.addActionListener( ae->toggle("tfc-trigger") );
    jp.add( button );

    button = new JButton( "SYNC" );
    button.addActionListener( ae->toggle("tfc-sync") );
    jp.add( button );

    button = new JButton( "Test pulse" );
    button.addActionListener( ae->toggle("tfc-tp") );
    jp.add( button );

    button = new JButton( "BX reset" );
    button.addActionListener( ae->toggle("tfc-bxreset") );
    jp.add( button );

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    cP.add( jp, gbC );

    jp = new JPanel();
    jp.setBorder( BorderFactory.createTitledBorder("DRP") );
    jp.add(drp.makePanel());
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    cP.add( jp, gbC );

    return cP;
  };

public void set()
{
  try
  {
      String regName = (String)regNameCB.getSelectedItem();
      cfg.put(regName,regValueTF.getText(),io);
  }
  catch ( Exception ex ){ throw new RuntimeException("MuDaq.set() Exception "+ex.getMessage()); }
}

public void actionPerformed(ActionEvent e)
{
  JButton jb = (JButton)e.getSource();
  String buttonName = jb.getName();
  
  try
  {
    if ( buttonName.equals("enable") )
    {
      cfg.put("ttc.enable",true,io);

      // Reset the GBT buffer write pointer
      cfg.put("daq.gbtbuf-reset",true,io);
      cfg.set("daq.gbtbuf-reset",false);
    }
    else if ( buttonName.equals("disable") )
    {
      cfg.put("ttc.enable",false,io);
    }
    else if ( buttonName.equals("init") )
    {
      setStateConfigure();
      io.put(cfg.getBaseAddress(),0,cfg.toByteArray(4));
    }
    else if ( buttonName.equals("shutdown") )
    {
      setStateShutdown();
      io.put(cfg.getBaseAddress(),0,cfg.toByteArray(4));
    }
    else if ( buttonName.equals("pulse") )
    {
      cfg.put("pulser.count",prefs,"80",io);
      cfg.put("pulser.delay",prefs,"1000",io);

      // Issue a pulser reset in case the delay pipeline has been changed
      cfg.put("pulser.reset",true,io);
      cfg.put("pulser.reset",false,io);
      
      cfg.put("pulser-go",true,io);
      cfg.set("pulser-go",false);
    }
  }
  catch ( Exception ex ){ throw new RuntimeException("MuDaq.actionPerformed() Exception "+ex.getMessage()); }

  return;
}

}

public class StatusPanel
{
  JPanel statusPanel;
    
  public StatusPanel(){}
  public JPanel makePanel()
  {
    statusPanel = new JPanel();
      
    statusPanel.setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(2,2,2,2);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;
    
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    statusPanel.add(RegisterIoTable.makePanel(tableList),gbC);
    tableList.get(0).addDecodeAction(e->{try{status.cache(io);status.decode();}catch (RegisterIoException re){System.err.println("MuDaq Exception "+re.getMessage());}});
    tableList.get(1).addDecodeAction(e->cfg.decode());
  
    return statusPanel;
  }

}
}
