package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.gui.*;

/**
<p>This class implements PDMDB controls.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
class PdmdbControl implements ActionListener
{
  ScaDepp sca;
  List<JTextField> pwrTFList;
  @XmlElement(name="powerSensor")
  List<PdmdbPowerSensor> pwrList;
  @XmlElement(name="powerEnable")
  int powerEnable;

  public PdmdbControl(){}

  public void init(ScaDepp sca)
  {
    this.sca = sca;
    for (PdmdbPowerSensor pwr : pwrList ) pwr.init(sca);
    sca.gpioDirection.clear(powerEnable);
    sca.gpioOut.clear(powerEnable);
  }

  public JPanel makePanel()
  {
    JPanel jp = new JPanel();

    jp.setLayout(new GridBagLayout());
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JButton button = new JButton("On");
    button.addActionListener(ae -> sca.setGpioDirection(powerEnable,0));
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("Off");
    button.addActionListener(ae -> sca.setGpioDirection(powerEnable,1));
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("Read Power Sensors");
    button.addActionListener(this);
    button.setActionCommand("adc");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    pwrTFList = new ArrayList<JTextField>();
    for (PdmdbPowerSensor pwr : pwrList )
    {
      JTextField tf = new JTextField();
      tf.setEditable(false);
      pwrTFList.add(tf);
      gbC.gridwidth = 1;
      jp.add( tf, gbC );
    }

    return jp;
  }

  public void actionPerformed( ActionEvent e )
  {
    Object ob = e.getSource();
    String command = e.getActionCommand();

    if ( command.matches("adc") )
    {
      int ordinal = 0;
      for (PdmdbPowerSensor pwr : pwrList )
      {
	int pwrVal = pwr.getAdc();
	pwrTFList.get(ordinal).setText(String.format("%1$s: %2$d",pwr.getName(),pwrVal));
	ordinal++;
      }
    }
  }

}
