package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.GridBagConstraints;
import java.awt.event.*;
import java.awt.Color;
import javax.swing.*;
import java.io.*;
import java.util.*;
import javax.swing.filechooser.*;
import java.util.prefs.Preferences;
import cbsw.*;
import cbsw.lhcb.pdmdb.*;
import cbsw.gui.*;

/**
<p>This class encapsulates the interactions with the PDMDB.</p>
**/
@XmlRootElement(name="pdmdb")
@XmlAccessorType(XmlAccessType.NONE)
public class PdmdbMaster
{
  RegisterIo io;
  @XmlElement(name="sca")
  ScaDepp sca;
  @XmlElement(name="db")
  PdmdbControl pdmdb;
  @XmlElement(name="ec")
  List<EcControl> ecList;
  @XmlElement(name="dtm")
  List<DtmControl> dtmList;
  ScaControl scaControl;

  static PdmdbMaster pdmdbMaster;
   
  JTextField claroTF;
  JButton spiTestButton;
  JCheckBox spiTestLoopCB;
  JTextField spiTestTF;
  public static boolean loop = false;
    
/**
<p>No-argument constructor used during unmarshalling.</p>
**/
  public PdmdbMaster(){scaControl = new ScaControl();}

/**
<p>Called after unmarshalling to complete initialisation of this and children.</p>
@param io The I/O interface to the hardware.
**/
  public void init( RegisterIo io )
  {
    this.io = io;
    sca.init(io);
    scaControl.init(sca); ScaControl.setStaticSca(sca);
    pdmdb.init(sca);
    for (DtmControl dtm : dtmList.toArray(new DtmControl[0])){ dtm.init(sca); }
    for (EcControl ec : ecList.toArray(new EcControl[0])){ ec.init(sca); }
  }

  public static void setStaticPdmdbMaster(PdmdbMaster pdmdbMaster){PdmdbMaster.pdmdbMaster=pdmdbMaster;}

  public ScaControl getScaControl(){return scaControl;}

/**
<p>Convenience function to make a GUI panel.</p>
@return The panel.
**/
  public GuiJPanel makePanel()
  {
    GuiJPanel jp = new GuiJPanel(Preferences.userNodeForPackage(PdmdbMaster.class).node("SCA"),"");

    jp.setLayout( new java.awt.GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new java.awt.Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(scaControl.makePanel(),gbC);

    JButton button = new JButton("Save CLARO file");
    button.addActionListener(l->saveClaroConfigurationFile());
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("Choose CLARO file");
    button.addActionListener(l->chooseConfigurationFile());
    gbC.gridwidth = GridBagConstraints.RELATIVE;
    jp.add(button,gbC);

    claroTF = new JTextField(Preferences.userNodeForPackage(ClaroControl.class).node("SCA").get("claro.filename","claro.spi"));
    claroTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add( claroTF, gbC );
    
    spiTestButton = new JButton("CLARO SPI test");
    spiTestButton.addActionListener(e->{((JButton)e.getSource()).setBackground(Color.yellow); (new SpiTestTask()).execute();});
    gbC.gridwidth = 1;
    jp.add(spiTestButton,gbC);

    spiTestLoopCB = new JCheckBox("Loop",false);
    spiTestLoopCB.addActionListener(l->PdmdbMaster.loop=spiTestLoopCB.isSelected());
    gbC.gridwidth = 1;
    jp.add(spiTestLoopCB,gbC);

    gbC.gridwidth = GridBagConstraints.RELATIVE;
    jp.add(new JLabel("Status",JLabel.RIGHT),gbC);
    
    spiTestTF = new JTextField(10);
    spiTestTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(spiTestTF,gbC);

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(pdmdb.makePanel(),gbC);

    JTabbedPane tp = new JTabbedPane();
    for ( EcControl ec : ecList.toArray(new EcControl[0]) ){ tp.add(ec.makePanel()); }
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(tp,gbC);

    tp = new JTabbedPane();
    for (DtmControl dtm : dtmList.toArray(new DtmControl[0])){ tp.add(dtm.makePanel()); }
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add(tp,gbC);
    
    return jp;
  }

/**
<p>Select and load the CLARO configuration registers.</p>
**/
  public void chooseConfigurationFile()
  {
    Preferences prefs = Preferences.userNodeForPackage(ClaroControl.class).node("SCA");
    JFileChooser chooser = new JFileChooser(prefs.get("claro.dir","."));
    FileNameExtensionFilter filter = new FileNameExtensionFilter( "CLARO config files (SPI)", "spi" );
    chooser.setFileFilter(filter);
    int returnVal = chooser.showOpenDialog(null);
    if(returnVal != JFileChooser.APPROVE_OPTION) return;
    if (!chooser.getSelectedFile().isFile()) return;
    prefs.put("claro.dir",chooser.getSelectedFile().getParent());
    prefs.put("claro.filename",chooser.getSelectedFile().getAbsolutePath());
    claroTF.setText(prefs.get("claro.filename","claro.spi"));

    loadClaroConfiguration();

    return;
  }
    
/**
<p>Load the CLARO configuration registers.</p>
**/
  public void loadClaroConfiguration()
  {
    Preferences prefs = Preferences.userNodeForPackage(ClaroControl.class).node("SCA");
    File cf = new File(prefs.get("claro.filename","claro.spi"));
    for (EcControl ec: ecList)
    {
      ArrayList<Claro> claroList = ec.claro.claros;
      for (Claro claro: claroList) claro.set(cf);
    }
  }

/**
<p>Create a CLARO configuration file from the current settings.</p>
**/
  public void saveClaroConfigurationFile()
  {
    JFileChooser chooser = new JFileChooser();
    FileNameExtensionFilter filter = new FileNameExtensionFilter( "CLARO configuration (SPI)", "spi" );
    chooser.setFileFilter(filter);
    int returnVal = chooser.showOpenDialog(null);
  
    if(returnVal == JFileChooser.APPROVE_OPTION)
    {
      try
      {
        FileOutputStream fo = new FileOutputStream(chooser.getSelectedFile());
        for (EcControl ec: ecList)
        {
          ArrayList<Claro> claroList = ec.claro.claros;
          for (Claro claro: claroList)
	  {
	    fo.write(claro.toByteArray());
	  }
        }
        fo.close();
      }
      catch (IOException ex) { System.err.println(ex.getMessage()); }
    }
  }

/**
<p>Configure the DTM module.</p>
<p>Called by reflection during command sequence.</p>
<p>Holds both DTM GBTXs in reset. Deasserts reset for GBTX0 and configures it.
Deasserts reset for GBTX1 and configures it.</p>
@param args [0]=DTM ID
@return Command response
**/
  public static String configureDtm(String [] args)
  {
    try
    {
      StringBuffer xml = new StringBuffer();
      int dtmId = Integer.decode(args[0]).intValue();
      DtmControl dtm = PdmdbMaster.pdmdbMaster.dtmList.get(dtmId);
      dtm.holdAllReset();
      
      String [] dtmString = new String [2];
      xml.append(String.format("<dtm testerModule=\"dtm%d\"",dtmId));

      for (GbtxControl gbtx : dtm.gbtxList)
      {
        gbtx.reset();
        gbtx.configure();

// Give some time for startup to complete
        try {Thread.sleep(1000);} catch(InterruptedException e){}

// Read the startup state
//	String startupState = gbtx.get("powerup-fsm-state");
//	System.err.println("PdmdbMaster.configureDtm() DTM"+dtmId+".GBTX"+gb+" Startup state="+startupState);

// Read the muDaq status to get the GBT header bits
      
        MuDaq.muDaq.status.cache(MuDaq.muDaq.io);

        int mgtId = 2 + 2*dtmId + gbtx.gbtx;
        String dtmGbtHeader = MuDaq.muDaq.status.get("gbt."+mgtId+".mgt.rx.header");
        String dtmVerify = gbtx.verify();
        dtmString[gbtx.gbtx] = "DTM"+dtmId+".GBTX"+gbtx.gbtx+"|"+dtmVerify+"|gbtHeader="+dtmGbtHeader;
        String [] dtmTokens = dtmVerify.split("\\|");
        if (dtmTokens.length > 1 && dtmTokens[0].equals("OK")) PdmdbModuleTester.dtmGbtxId[dtmId][0] = dtmTokens[1];
        xml.append(String.format(" gbtx%dId=\"%s\" gbtx%dState=\"%s\"",gbtx.gbtx,dtmTokens[1],gbtx.gbtx,dtmTokens[2]));
      }
      xml.append(" />\n");
      TesterSequence.testSummaryXml.write(xml.toString());
    
      return "OK|PdmdbMaster.configureDtm|"+dtmString[0]+"|"+dtmString[1];
    }
    catch (Exception ex){ return "FAIL|PdmdbMaster.configureDtm|Exception "+ex.getMessage(); }

  }
    
/**
<p>Program the PDMDB FPGA.</p>
<p>Called by reflection during command sequence.</p>
@param args [0]=DTM ID
@return Command response
**/
  public static String programFpga(String [] args)
  {
    int dtmId = Integer.decode(args[0]).intValue();
    XilS7Control xil = PdmdbMaster.pdmdbMaster.dtmList.get(dtmId).xilS7;

    xil.program();

    return "OK|PdmdbMaster.programFpga";
  }
    
/**
<p>Perform the automatic GBTX ePort phase training.</p>
<p>Called by reflection during command sequence.</p>
@param args [0]=DTM ID
@return Command response
**/
  public static String trainEports(String [] args)
  {
    try
    {
      int dtmId = Integer.decode(args[0]).intValue();
      DtmControl dtm = PdmdbMaster.pdmdbMaster.dtmList.get(dtmId);
      
// Set a fixed pattern on the down links
      MuDaq.muDaq.cfg.put("tfctog-enable","0",MuDaq.muDaq.io);
      MuDaq.muDaq.cfg.put("tfc-d","A",MuDaq.muDaq.io);

      for (GbtxParent gbtx : dtm.gbtxList )
      {

	gbtx.put("eport.pa.mode","1"); // Switch to training mode
        for (String eport : gbtx.activeEPorts) gbtx.put(eport+".train","1");

        try {Thread.sleep(200);} catch (InterruptedException ie){}
	
        for (String eport : gbtx.activeEPorts) gbtx.put(eport+".train","0");

        try {Thread.sleep(100);} catch (InterruptedException ie){}
	
        byte [] trainBuf = new byte [28];
        gbtx.get(trainBuf,399);

	StringBuffer sb = new StringBuffer();
	for (int group=0;group<7;group++) sb.append(String.format("egroup"+group+"Eport0=\"%d\" egroup"+group+"Eport4=\"%d\" ",trainBuf[4*group]&0xf,trainBuf[4*group+2]&0xf));
	TesterSequence.testSummaryXml.write("<eportPhase testerModule=\"dtm"+dtmId+"\" gbtx=\""+gbtx.gbtx+"\" ");
	String timestamp = java.time.LocalDateTime.now().format(java.time.format.DateTimeFormatter.ofPattern("yyyyMMdd-A"));
	sb.append("timestamp=\""+timestamp+"\" ");
	TesterSequence.testSummaryXml.write(sb.toString());
	TesterSequence.testSummaryXml.write("/>\n");

	gbtx.put("eport.pa.mode","0"); // Back to static mode

// Apply the training
	for (int group=0;group<7;group++)
	{
	  gbtx.put("eport.pa."+group+".0.phase",String.format("%X",trainBuf[4*group]&0xf));
	  gbtx.put("eport.pa."+group+".4.phase",String.format("%X",trainBuf[4*group+2]&0xf));
	}
      }
    }
    catch (Exception ioe){ return "FAIL|PdmdbMaster.trainEports|Exception "+ioe.getMessage(); }
    return "OK|PdmdbMaster.trainEports";
  }

// Loop test of CLARO configuration registers

  class SpiTestTask extends SwingWorker<Boolean, String>
  {
    @Override
    protected Boolean doInBackground()
    {
      boolean pass = true;
      int trial=0;
      int errors=0;
      while ( loop  )
      {
	int irand = (int)(0xff&Math.round(256 * Math.random()));
	for (EcControl ec: ecList)
	{
	  if (!ec.claro.testEnable) continue;
	  ArrayList<Claro> claroList = ec.claro.claros;
	  for (Claro claro: claroList) claro.setRandom(irand);

          sca.allocateTxRxBuffers(10*claroList.size()); // 10 SCA commands per CLARO
          for ( Claro claro : claroList ) claro.prepareSpi(true);
          sca.putBytes(0,true);

          if (!ClaroControl.verify(sca,claroList))
	  {
	    errors++;
	    System.err.println("PdmdbMaster.doInBackground(): verification failed for trial "+trial+" retrying...");
	    if (!ClaroControl.verify(sca,claroList))
	    {
	      pass = false;
 	      System.err.println("PdmdbMaster.doInBackground(): retry failed");
	    }
	    else
	    {
 	      System.err.println("PdmdbMaster.doInBackground(): retry OK");
	    }
	  }

          if (trial%10==0) publish(String.format("Trials: %d Errors: %d",trial,errors));
          trial++;
	}
      }
      return Boolean.valueOf(pass);
    }
      
    @Override
    protected void done(){ try { if (get()) spiTestButton.setBackground(Color.green); else spiTestButton.setBackground( Color.orange ); } catch (Exception e){} }

    @Override
    protected void process( List<String> chunks ){for ( String pass : chunks ) spiTestTF.setText(pass);}
  }
}
