/*
PDMDB module tester comms
2018-
Steve Wotton
Cavendish Lab (HEP)
*/

package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.file.*;
import java.nio.file.attribute.*;
import java.net.Socket;
import java.net.URL;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Font;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.prefs.*;
import java.util.*;
import java.util.ArrayList;
import java.time.LocalTime;
import java.lang.Object;
import cbsw.gui.*;
import javax.swing.filechooser.*;
import java.text.DecimalFormat;
import cbsw.lhcb.pdmdb.*;
import cbsw.*;

/**
<p>This class encapsulates the interactions with the Module Tester Board (MTB) hardware.</p>
**/
public class PdmdbModuleTester implements ActionListener {

Preferences prefs;

static final int proxyMsgSize = 4096;
static ByteBuffer proxyMsg;

static SocketChannel socketChannel;
static ReadableByteChannel ichannel;
boolean threadStopFlag = false;
    
JFrame programJF;
JTextField cmdTF;
static JTextArea textArea;
JScrollPane textSP;
    
static int cmdSeq=0;
static int rxSeq=0;

static String tcmQR;
static String [] dtmQR;
static String tcmGbtxId;
static String [][] dtmGbtxId;

ActionListener socketUrlAL = new ActionListener () { public void actionPerformed(ActionEvent e)
{
  String url = JOptionPane.showInputDialog("Please enter socket URL",prefs.get("mtb.socket","http://:7999"));
  if (url != null) prefs.put("mtb.socket",url);
  return;
}
};

static
{
  dtmQR = new String [3];
  dtmGbtxId = new String [3][2];
  
  // Proxy message buffer
  proxyMsg = ByteBuffer.allocate(proxyMsgSize);
  proxyMsg.order(ByteOrder.LITTLE_ENDIAN);

  Preferences mtbPrefs = Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp");

  try { makeSocket(new URL(mtbPrefs.get("mtb.socket","http://:7999"))); }
  catch (MalformedURLException ue){System.err.println("PdmdbModuleTester.PdmdbModuleTester() Warning: " + ue.getMessage());}
}
    
public PdmdbModuleTester(){}

public void init(Preferences prefs)
{
  this.prefs = prefs;
  System.err.println("Prefs path: "+prefs.absolutePath());
}

    /*
public JPanel makePanel()
{
  JPanel jP = new JPanel();
// Set layout style

  jP.setLayout( new GridBagLayout() );
  GridBagConstraints gbC = new GridBagConstraints();
  gbC.fill = GridBagConstraints.VERTICAL;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new Insets(2,2,2,2);
  gbC.weightx = 1.0f;
  //  gbC.weighty = 1.0f;
  gbC.fill = GridBagConstraints.BOTH;


  cmdTF = jP.makeTF("command","",80,"Command","Enter command",false);
  cmdTF.addActionListener(this);
  cmdTF.setFont(new Font("Monospaced",Font.PLAIN,12));
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  jP.add(cmdTF.getParent(),gbC);
  
  textArea = new JTextArea(10,10); // Minimum width. Will expand to fill available width.
  textArea.setEditable(false);
  textArea.setFont(new Font("Monospaced",Font.PLAIN,12));
  textSP = new JScrollPane(textArea);
  gbC.gridwidth = GridBagConstraints.REMAINDER;
  gbC.fill = GridBagConstraints.HORIZONTAL;
  jP.add(textSP,gbC);

  return jP;
}
    */
	
public void actionPerformed( ActionEvent e )
{
  String cmd = e.getActionCommand();
  java.awt.Component src = (java.awt.Component)e.getSource();
  if ( src.getName().equals( "command" ) )
  {
    textArea.setText(null); // Clear the text area ready for the next command
    exec(((JTextField)src).getText());
  }
 return;
}

public static String [] readResponse(int timeout)
{

  try
  {
    socketChannel.socket().setSoTimeout(timeout);
    proxyMsg.clear();
    ichannel.read(proxyMsg);
    proxyMsg.flip();
    byte [] rdMsg = new byte[proxyMsg.limit()];
    proxyMsg.get(rdMsg);
    String msg = new String(rdMsg);
    TesterSequence.logMessage("<["+LocalTime.now().toString()+"] "+msg);
    return msg.split("\\|");
  }
  catch (Exception e) {System.out.println("PdmdbModuleTester.readResponse(): "+e.toString());}

  return null;
}

// Clean out any pending responses
public static void readFlush()
{
  while (true)
  {
     String [] msg = readResponse(500);
     if (msg == null) break;
     StringBuffer str = new StringBuffer();
     for (String token : msg) str.append(token+" ");
     str.append("\n");
     System.err.println("PdmdbModuleTester.readFlush(): "+str);
  }
// A good time to reset the RX sequence number to equal the command sequence number
  rxSeq = cmdSeq;
}

// Write XML data
static String xml(String [] args)
{
  try
  {
    BufferedWriter tcmXml = Files.newBufferedWriter(TesterSequence.testDirectory.resolve(Path.of("tcm.xml")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);
    tcmXml.write("<tcm qr=\""+PdmdbModuleTester.tcmQR+"\" gbtx=\"0x"+PdmdbModuleTester.tcmGbtxId+"\" />");
    tcmXml.flush();
    tcmXml.close();
  
    BufferedWriter dtmXml = Files.newBufferedWriter(TesterSequence.testDirectory.resolve(Path.of("dtm.xml")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);
    for (int dtm=0;dtm<3;dtm++)
    {
      dtmXml.write("<dtm qr=\""+PdmdbModuleTester.dtmQR[dtm]+"\" gbtx0=\""+PdmdbModuleTester.dtmGbtxId[dtm][0]+"\" gbtx1=\""+PdmdbModuleTester.dtmGbtxId[dtm][1]+"\" position=\""+dtm+"\" />");
    }
    dtmXml.flush();
    dtmXml.close();
  }
  catch (IOException ioe){return "FAIL|PdmdbModuleTester.xml|IO Exception";}

  return "OK|PdmdbModuleTester.xml";
}

// Called by Command.Method.invoke()
public static String exec(String [] args)
{
  cmdSeq++;

  if (args==null) return "FAIL|PdmdbModuleTester.exec";

  String cmd = Integer.toString(cmdSeq);
  for (String s : args) cmd = cmd + "|" + s;
  cmd = cmd + "\n";

  TesterSequence.logMessage(">["+LocalTime.now().toString()+"] "+cmd);
  try
  {
    ByteBuffer msgBB = ByteBuffer.wrap( cmd.getBytes() );
    socketChannel.write(msgBB);
  }
  catch (Exception e) {System.out.println("PdmdbModuleTester.exec(String[]): "+e.toString()+" for "+cmd); return "FAIL|PdmdbModuleTester.exec";}

  return "OK|PdmdbModuleTester.exec";
}

public static void exec(String msg)
{
  cmdSeq++;
  
  String cmd = cmdSeq + "|" + msg + "\n";
  TesterSequence.logMessage(">["+LocalTime.now().toString()+"] "+cmd);
  try
  {
    ByteBuffer msgBB = ByteBuffer.wrap( cmd.getBytes() );
    socketChannel.write(msgBB);
  }
  catch (Exception e) {System.out.println("PdmdbModuleTester.exec(String): "+e.toString()+" for "+msg);}
  
  return;
}

static String scanModule(String name, String indicatorPattern)
{
  PdmdbModuleTester.exec("INDICATE|"+indicatorPattern);
  PdmdbModuleTester.rxSeq++;
  PdmdbModuleTester.readResponse(3000);
  String [] dummy = PdmdbModuleTester.readResponse(3000);
  JOptionPane pane = new JOptionPane(new String("Scan "+name),JOptionPane.PLAIN_MESSAGE,JOptionPane.DEFAULT_OPTION,null,null);
  //  pane.setBackground(java.awt.Color.BLUE);
  pane.setWantsInput(true);
  JDialog dialog = pane.createDialog(null, name);
  dialog.setVisible(true);
  String qr = ((String)pane.getInputValue()).toUpperCase();

  if (qr.matches("^4RXCAMDB[DT][TC][0-1][0-9][0-9][0-9]")) return qr;
  else return "";
}

public static String scan(String [] args)
{
  try
  {
    String dtm0 = scanModule("DTM0","1000");
    if (dtm0.equals("")) throw new RuntimeException("Invalid scanned code (DTM0)");
    TesterSequence.testSummaryXml.write(String.format("<index testerModule=\"dtm0\" code=\"%s\" directory=\"%s\" />\n",dtm0,TesterSequence.testSubdirectory));
  
    String tcm = scanModule("TCM","0001");
    if (tcm.equals("")) throw new RuntimeException("Invalid scanned code (TCM)");
    TesterSequence.testSummaryXml.write(String.format("<index testerModule=\"tcm\" code=\"%s\" directory=\"%s\" />\n",tcm,TesterSequence.testSubdirectory));

    String dtm1 = scanModule("DTM1","0100");
    if (dtm1.equals("")) throw new RuntimeException("Invalid scanned code (DTM1)");
    TesterSequence.testSummaryXml.write(String.format("<index testerModule=\"dtm1\" code=\"%s\" directory=\"%s\" />\n",dtm1,TesterSequence.testSubdirectory));
  
    String dtm2 = scanModule("DTM2","0010");
    if (dtm2.equals("")) throw new RuntimeException("Invalid scanned code (DTM2)");
    TesterSequence.testSummaryXml.write(String.format("<index testerModule=\"dtm2\" code=\"%s\" directory=\"%s\" />\n",dtm2,TesterSequence.testSubdirectory));

    PdmdbModuleTester.exec("INDICATE|0000");
    PdmdbModuleTester.rxSeq++;
    PdmdbModuleTester.readResponse(3000);
    String [] dummy = PdmdbModuleTester.readResponse(3000);
  
    return "OK|PdmdbModuleTester.scan|tcm="+tcm+" dtm0="+dtm0+" dtm1="+dtm1+" dtm2="+dtm2;
  }
  catch (IOException ioe){ return "FAIL|PdmdbModuleTester.scan|IOException: "+ioe.getMessage();}
  catch (RuntimeException re) { return "FAIL|PdmdbModuleTester.scan|"+re.getMessage(); }

}

public static String readDacs(String [] args)
{
  int samples;
  int bins;
  double binMinimum;
  double binWidth;
  double binMaximum;
  double maximumRMS;
  try
  {
    samples = Integer.decode(args[0]);
    bins = Integer.valueOf(args[1]);
    binMinimum = Double.valueOf(args[2]);
    binWidth = Double.valueOf(args[3]);
    binMaximum = binMinimum + bins*binWidth;
    maximumRMS = Double.valueOf(args[4]);
  }
  catch (NullPointerException npe){ return "FAIL|PdmdbModuleTester.readDacs|Invalid argument"; }

  if (samples < 1) return "FAIL|PdmdbModuleTester.readDacs|Invalid argument";

  double dacVal [] = new double[4];

  try
  {
    BufferedWriter writer = Files.newBufferedWriter(TesterSequence.testDirectory.resolve(Path.of("dac.dat")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);

// Bin description
    writer.write(""+bins+","+binMinimum+","+binWidth); // #bins, low limit, bin width
    writer.newLine();
// Column headers
    for (int dac=0;dac<3;dac++) writer.write("dac"+Integer.toString(dac)+",");
    writer.write("dac"+Integer.toString(3));
    writer.newLine();

// Data
    for (int i=0;i<samples;i++)
    {
      PdmdbModuleTester.exec("READ_DACS");
      PdmdbModuleTester.rxSeq++;
      PdmdbModuleTester.readResponse(3000); // ACK
      String [] response = PdmdbModuleTester.readResponse(3000);
      for (String token: response)
      {
	String [] dacResponse = token.split("=");
	if (dacResponse==null || dacResponse.length!=2) continue;
	if (!dacResponse[0].startsWith("DAC")) continue;
	int dacId = Integer.parseInt(dacResponse[0].substring(3,4));
	if (dacId<0 || dacId>3) break;
	dacVal[dacId] = Double.valueOf(dacResponse[1]);
      }
      for (int dac=0;dac<3;dac++) writer.write(Double.toString(dacVal[dac])+",");
      writer.write(Double.toString(dacVal[3]));
      writer.newLine();
    }
    writer.flush();
    writer.close();

// Create the histograms (get data from the file we've just created)
    BufferedReader reader = Files.newBufferedReader(TesterSequence.testDirectory.resolve(Path.of("dac.dat")));
    Histogram hist = new Histogram("");
    try { hist.readStream(reader); } catch(Histogram.EOFException eof){}
    reader.close();

// Set the Paint for all series to be the same block colour
    hist.setPaint(new java.awt.Color(0x3333cc));

// Output the plots and dataset statistics
    for ( Comparable key : hist.keys() )
    {
      hist.hide(); // Hide any series that are currently visible
      hist.show(key); // Draw this series
      hist.saveAsSVG(TesterSequence.testDirectory.resolve(Path.of((String)key+".svg")).toFile()); // Write as an SVG file
      double mean = hist.mean(key); // Unbinned mean
      double rms = hist.rms(key); // Unbinned RMS
      TesterSequence.testSummaryXml.write(String.format("<dac id=\"%s\" mean=\"%.2f\" rms=\"%.2f\" />\n",(String)key,mean,rms));
    }
    TesterSequence.testSummaryXml.flush();
  }
  catch (IOException ioe){return "FAIL|PdmdbModuleTester.readDacs|IO exception";}
    
  return "OK|PdmdbModuleTester.readDacs";
}

public static int getCmdSeq(){return cmdSeq;}

public static void makeSocket(URL url)
{
  try
  {

// Check the port number

    int port = url.getPort();
    if ( port != 7999 ) return;

    socketChannel = SocketChannel.open(new InetSocketAddress(url.getHost(), port));

    ichannel = Channels.newChannel(socketChannel.socket().getInputStream());
  }
  catch ( java.nio.channels.UnresolvedAddressException uae ){System.err.println("PdmdbModuleTester.makeSocket(): Socket address unresolved for "+url.getHost());}
  catch ( IOException ioe ) { System.err.println( "PdmdbModuleTester.makeSocket(): [" + url.toString() + "] " + ioe.getMessage() ); }
  readFlush();
  return;
}

}
