package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.*;
import cbsw.lhcb.pdmdb.*;
import cbsw.gui.*;

/**
<p>This class implements post-processing of Commands.</p>
**/
@XmlRootElement(name="test")
@XmlAccessorType(XmlAccessType.NONE)
public class PdmdbPostProcessor
{
Preferences prefs;

@XmlElement(name="index")
java.util.ArrayList<Index> indices;
@XmlElement
Tcm tcm;
@XmlElement(name="eye")
java.util.ArrayList<Eye> eyes;
@XmlElement(name="adc")
java.util.ArrayList<Adc> adcs;
@XmlElement(name="dac")
java.util.ArrayList<Dac> dacs;
@XmlElementWrapper(name="clocks")
@XmlElement(name="clock")
java.util.ArrayList<Clock> clocks;
@XmlElementWrapper(name="spi")
@XmlElement(name="ss")
java.util.ArrayList<SpiClock> spiClocks;
@XmlElement(name="dtm")
java.util.ArrayList<Dtm> dtms;
@XmlElement(name="power")
java.util.ArrayList<Power> powers;
    
  public PdmdbPostProcessor()
  {
    prefs = Preferences.userNodeForPackage(PdmdbPostProcessor.class).node("JPDMDBDepp");
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class Index
  {
    @XmlAttribute
    String testerModule;
    @XmlAttribute
    String code;
    @XmlAttribute
    String directory;
    public Index(){}
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class Tcm
  {
    @XmlAttribute
    String testerModule;
    @XmlAttribute
    String gbtxId;
    @XmlAttribute
    String gbtxState;
    public Tcm(){}
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class Eye
  {
    @XmlAttribute
    String id;
    @XmlAttribute
    String binsWithoutErrors;
    public Eye(){}
  }
    
  @XmlAccessorType(XmlAccessType.NONE)
  public static class Adc
  {
    @XmlAttribute
    String id;
    @XmlAttribute
    String mean;
    @XmlAttribute
    String rms;
    public Adc(){}
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class Dac
  {
    @XmlAttribute
    String id;
    @XmlAttribute
    String mean;
    @XmlAttribute
    String rms;
    public Dac(){}
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class Clock
  {
    @XmlAttribute
    String id;
    @XmlAttribute
    String frequency;
    public Clock(){}
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class SpiClock
  {
    @XmlAttribute
    String index;
    @XmlAttribute
    String toggles;
    @XmlAttribute
    String clocks;
    public SpiClock(){}
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class Dtm
  {
    @XmlAttribute
    String testerModule;
    @XmlAttribute
    String gbtx0Id;
    @XmlAttribute
    String gbtx0State;
    @XmlAttribute
    String gbtx1Id;
    @XmlAttribute
    String gbtx1State;
    public Dtm(){}
  }

  @XmlAccessorType(XmlAccessType.NONE)
  public static class Power
  {
    @XmlAttribute
    String testerModule;
    @XmlAttribute
    String i1v5;
    @XmlAttribute
    String i2v5;
    public Power(){}
  }

  public JPanel makePanel()
  {

    JPanel jP = new JPanel();
    jP.setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(2,2,2,2);
    gbC.weightx = 1.0f;
    //    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    javax.swing.JPanel buttonJP = new javax.swing.JPanel();
    buttonJP.setLayout(new java.awt.GridLayout(0,10));
  
    javax.swing.JButton button = new javax.swing.JButton("ID");
    buttonJP.add(button);
    button = new javax.swing.JButton("TCM");
    buttonJP.add(button);
    button = new javax.swing.JButton("DTM");
    buttonJP.add(button);
    button = new javax.swing.JButton("Power");
    buttonJP.add(button);
    button = new javax.swing.JButton("Clocks");
    buttonJP.add(button);
    button = new javax.swing.JButton("ADC");
    buttonJP.add(button);
    button = new javax.swing.JButton("DAC");
    buttonJP.add(button);
    button = new javax.swing.JButton("SPI");
    buttonJP.add(button);
    button = new javax.swing.JButton("Links");
    buttonJP.add(button);

    gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    jP.add(buttonJP,gbC);

    return jP;
  }
    
  public JMenu makeMainMenu(String title)
  {
    JMenu menuBarMenu = new JMenu(title);

    JMenuItem menuItem = new JMenuItem("Exit");
    menuItem.addActionListener(a->System.exit(0));
    menuBarMenu.add(menuItem);
    
    return menuBarMenu;
  }

public static void main( String[] args )
{

// The pause here is for the splash screen which closes as soon as the first swing window is made visible.

  try
  {
    Thread.sleep(1000);
  }
  catch ( InterruptedException inte )
  {
    System.err.println("Warning: " + inte.getMessage() );
  }

//  String laf = UIManager.getSystemLookAndFeelClassName();
  String laf = UIManager.getCrossPlatformLookAndFeelClassName();
  try 
  {
    UIManager.setLookAndFeel(laf);
  }
  catch (UnsupportedLookAndFeelException exc)
  {
    System.err.println("Warning: UnsupportedLookAndFeel: " + laf);
  }
  catch (Exception exc)
  {
    System.err.println("Error loading " + laf + ": " + exc);
  }

// Minor tweaks to the LAF
  
  UIManager.put("Button.background",new Color(220,220,220));
  UIManager.put("TextField.background",new Color(240,240,240));
  UIManager.put("TextArea.background",new Color(240,240,240));
  UIManager.put("Table.background",new Color(240,240,240));
  
  PdmdbPostProcessor pp = (PdmdbPostProcessor)Cbsw.unmarshal("https://www.hep.phy.cam.ac.uk/lhcb/PDMDB/xml/TestSummary.xml",PdmdbPostProcessor.class);
  System.err.println(pp.indices.get(0).code);

  JFrame frame = GuiJPanel.makeFrame(pp.makePanel(),"PdmdbPostProcessor");
  frame.getJMenuBar().add(pp.makeMainMenu("Main"));
  frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  frame.pack();
  frame.setVisible(true);

  return;
}
    
}
