package cbsw;

import java.util.BitSet;
import javax.xml.bind.annotation.*;

@XmlAccessorType(XmlAccessType.NONE)
public class RegisterDescriptor
{
  @XmlValue
  String description;
  @XmlAttribute
  String name;
  @XmlAttribute
  int offset;
  @XmlAttribute
  int length;

  public RegisterDescriptor(){}

  public String getDescription(){return description;}
  public String getName(){return name;}
  public int getOffset(){return offset;}
  public int getLength(){return length;}

// Return an array of bytes spanning the register bits offset from start of new array
  public byte [] getBytes(BitSet regBits, int arrayOffset)
  {
    int bitLow = offset&0xfffffff8;
    int bitHigh = ((offset+length-1)&0xfffffff8)+8;
    BitSet bits = regBits.get(bitLow,bitHigh);
    byte [] result = new byte [arrayOffset+(bitHigh-bitLow)/8];
    System.arraycopy(bits.toByteArray(),0,result,arrayOffset,bits.toByteArray().length);
    return result;
  }

  public byte [] getBytes(BitSet regBits) { return getBytes(regBits,0); }

  public String get(BitSet regBits)
  {
    BitSet bits = regBits.get(offset,offset+length);
    byte [] result = new byte [(length+7)/8];
    System.arraycopy(bits.toByteArray(),0,result,0,bits.toByteArray().length);
    String resultString = new String();
    for ( byte b: result ) resultString = String.format("%1$02x",b) + resultString;
    return resultString;
  }

  public void set(BitSet regBits, boolean flag)
  {
    if (length != 1) return;
    if (flag)
    {
      regBits.set(offset);
    }
    else
    {
      regBits.clear(offset);
    }
  }

  public void set(BitSet regBits, String regString)
  {
    int off = offset;
    int end = offset + length;

    regBits.clear(off,end);

    for (int c=regString.length()-1;c>=0;c--)
    {
      if (off >= end) break;
      int reg = Integer.parseInt(regString.substring(c,c+1),16);
      for (int i=0;i<4;i++)
      {
	if (off >= end) break;
        if ((reg>>i & 1) == 1) regBits.set(off);
	off++;
      }
    }
  }
}
