/*
RegisterIoTable class

Steve Wotton, 2018
Cavendish Lab (HEP)
*/

package cbsw;

import javax.xml.bind.annotation.*;
import java.awt.GridBagConstraints;
import java.awt.BorderLayout;
import javax.swing.*;
import java.awt.event.*;
import cbsw.*;

@XmlAccessorType(XmlAccessType.NONE)
public class RegisterIoTable
{
    RegisterIo io;
    @XmlAttribute
    String title;
    @XmlAttribute
    int baseAddress;
    @XmlAttribute
    int csrAddress;
    @XmlAttribute
    int csrSize;
    @XmlAttribute
    int tableDataDisplayWidth;
    String [] tableColumnNames = {"Addr","Data"};
    Object [][] tableData;
    JScrollPane tableSP;
    JTable table;
    byte [] csrCache;
    JButton decodeButton;

  public RegisterIoTable(){}

  public String getTitle(){ return title; }

  public void init(RegisterIo io)
  {
    this.io = io;
    csrCache = new byte [csrSize];
  }

@Deprecated
  public JTable getStatusTable(){ return table; }
    
  public JTable getTable(){ return table; }

    public JPanel makePanel()
    {
        tableData = new Object[(csrSize+tableDataDisplayWidth-1)/tableDataDisplayWidth][];

// Initialise the register table

        for ( int reg=0;reg<csrSize;reg+=tableDataDisplayWidth )
        {
	  int row = reg/tableDataDisplayWidth;
	  tableData[row] = new Object [2];
	  tableData[row][0] = Integer.valueOf(reg);
	  tableData[row][1] = "";
	}

// Table of register values

        table = new JTable(tableData,tableColumnNames);
        table.setFont(new java.awt.Font("Monospaced",java.awt.Font.PLAIN,11));
        table.setPreferredScrollableViewportSize(new java.awt.Dimension(40+tableDataDisplayWidth*16, 200));
        table.getColumnModel().getColumn(0).setPreferredWidth(40);
        table.getColumnModel().getColumn(1).setPreferredWidth(tableDataDisplayWidth*16);

        JScrollPane jSP = new JScrollPane(table);
        JPanel jP = new JPanel();
	jP.setLayout(new BorderLayout());
        jP.add(jSP,BorderLayout.NORTH);
        jP.setBorder( BorderFactory.createTitledBorder(title) );

	decodeButton = new JButton("Decode");
        jP.add(decodeButton,BorderLayout.SOUTH);
	
        javax.swing.Timer timer = new javax.swing.Timer( 1000, l->{cache(); updateTable();} );
        timer.setInitialDelay(1000);
        timer.setCoalesce( false );
        timer.start();

	return jP;
     }

public void addDecodeAction(ActionListener al){ decodeButton.addActionListener(al);}
    
public void writeArray( byte [] regs )
{
  try
  { 
    io.put( baseAddress, csrAddress, regs );
  } catch (RegisterIoException de) {}
  return;
}

// Read the register array from hardware into local cache.
public void cache()
{
  try
  {
    io.get( baseAddress, csrAddress, csrCache );
  } catch (Exception de) {}
  return;
}

public byte [] getCache(){return csrCache;}

@Deprecated
public void updateStatusTable(){ updateTable(); }

// Update table from previously cached data
public void updateTable()
{
  for ( int reg=0;reg<csrSize;reg+=tableDataDisplayWidth )
  {
    int row = reg/tableDataDisplayWidth;
    String sreg = "";
    for (int n=0;n<tableDataDisplayWidth;n++)
    {
      sreg = String.format("%1$02x",csrCache[reg+n]) + sreg;
    }
    table.setValueAt(sreg,row,1);
  }
}
    
static public JPanel makePanel(java.util.ArrayList<RegisterIoTable> tableList)
{
  JPanel dtjP = new JPanel();
    
  dtjP.setLayout(new java.awt.GridBagLayout());
  GridBagConstraints gbC = new GridBagConstraints();
  gbC.fill = GridBagConstraints.VERTICAL;
  gbC.anchor = GridBagConstraints.CENTER;
  gbC.insets = new java.awt.Insets(1,1,1,1);
  gbC.weightx = 1.0f;
  gbC.weighty = 1.0f;
  gbC.fill = GridBagConstraints.BOTH;

  for (RegisterIoTable dT : tableList )
  {
    JPanel jP = dT.makePanel();
    gbC.gridwidth = 1;
    dtjP.add( jP, gbC );
  }
  return dtjP;
}

}
