package cbsw.lhcb.pdmdb;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.*;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.gui.*;
import java.lang.reflect.*;

/**
<p>This class encapsulates interactions with the SCA ASIC.</p>
**/
public class ScaControl
{
  ScaDepp sca;
  JFrame scaExpertFrame;
    
// Static member containing the instance referenced by exec
static ScaDepp staticSca;

  public ScaControl(){}

  public GuiJPanel makePanel()
  {
    GuiJPanel jP = new GuiJPanel(Preferences.userNodeForPackage(ScaControl.class).node("SCA"),"");

    ScaExpert scaExpert = new ScaExpert();
    scaExpertFrame = GuiJPanel.makeFrame(scaExpert.makePanel(),"SCA expert panel");
    scaExpertFrame.pack();

    jP.setBorder( BorderFactory.createTitledBorder("GBT-SCA commands") );
    jP.setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JButton button = new JButton("Connect");
    button.addActionListener(a->sca.connect());
    gbC.gridwidth = 1;
    jP.add(button,gbC);

    button = new JButton("Reset");
    button.addActionListener(a->sca.reset());
    gbC.gridwidth = 1;
    jP.add(button,gbC);

    button = new JButton("Setup");
    button.addActionListener(a->{sca.enableInterfaces();sca.setup();});
    gbC.gridwidth = 1;
    jP.add(button,gbC);

    button = new JButton("Expert");
    button.addActionListener(a->scaExpertFrame.setVisible(!scaExpertFrame.isVisible()));
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jP.add(button,gbC);

    return jP;
  }

  public void init(ScaDepp sca){ this.sca = sca; }
    
  public static void setStaticSca(ScaDepp sca){ScaControl.staticSca = sca;}

  public static String setup(String [] args)
  {
    staticSca.reset();
    if (!staticSca.enableInterfaces()) return "FAIL|ScaControl.setup";
    staticSca.setup();
    String scaId = staticSca.readId();
    try { TesterSequence.testSummaryXml.write(String.format("<sca id=\"%s\"/>\n",scaId)); }
    catch (IOException ioe){return "FAIL|ScaControl.setup|scaId="+scaId;}
    return "OK|ScaControl.setup|scaId="+scaId;
  }
  public static String setDac(String [] args)
  {
    staticSca.setDac(Integer.decode(args[0]).intValue(),(byte)Integer.decode(args[1]).intValue());
    return "OK|ScaControl.setDac";
  }
    
  public static String setGpio(String [] args)
  {
    staticSca.setGpio(Integer.decode(args[0]).intValue(),Integer.decode(args[1]).intValue());
    return "OK|ScaControl.setGpio";
  }
    
  public static String setGpioDirection(String [] args)
  {
    staticSca.setGpioDirection(Integer.decode(args[0]).intValue(),Integer.decode(args[1]).intValue());
    return "OK|ScaControl.setGpioDirection";
  }
    
  public static String setAdcCurrentSource(String [] args)
  {
    staticSca.setAdcCurrentSource(Integer.decode(args[0]).intValue(),Integer.decode(args[1]).intValue());
    return "OK|ScaControl.setAdcCurrentSource";
  }
    
  class ScaExpert
  {
    JComboBox<String> groupCB, commandCB, channelCB;
    JTextField [] dataTF;
    JTextField rawTF;
    JTextArea textArea;
    JScrollPane textSP;
      
    public ScaExpert(){}

    public GuiJPanel makePanel()
    {
      GuiJPanel jP = new GuiJPanel(Preferences.userNodeForPackage(ScaControl.class).node("SCA"),"");

      jP.setBorder( BorderFactory.createTitledBorder("GBT-SCA expert commands") );
      jP.setLayout( new GridBagLayout() );
      GridBagConstraints gbC = new GridBagConstraints();
      gbC.fill = GridBagConstraints.VERTICAL;
      gbC.anchor = GridBagConstraints.CENTER;
      gbC.insets = new Insets(1,1,1,1);
      gbC.weightx = 1.0f;
      gbC.weighty = 1.0f;
      gbC.fill = GridBagConstraints.BOTH;

      JButton button = new JButton("Send");
      button.addActionListener(a->sendCommand());
      gbC.gridwidth = 1;
      jP.add(button,gbC);

      groupCB = new JComboBox<String>();
      groupCB.addItem("NC");
      groupCB.addItem("I2C");
      groupCB.addItem("SPI");
      groupCB.addItem("GPIO");
      groupCB.addItem("JTAG");
      groupCB.addItem("DAC");
      groupCB.addItem("ADC");
      groupCB.addActionListener(a->updateCommandCB());
      gbC.gridwidth = 1;
      jP.add(jP.addBorder(groupCB, "Group"),gbC);

      commandCB = new JComboBox<String>();
      gbC.gridwidth = 1;
      jP.add(jP.addBorder(commandCB, "Command"),gbC);

// For I2C it is necessary to also select the channel
      channelCB = new JComboBox<String>();
      gbC.gridwidth = 1;
      jP.add(jP.addBorder(channelCB, "Channel"),gbC);

      updateCommandCB();

      JPanel dJp = new JPanel();
      dJp.setBorder( BorderFactory.createTitledBorder("D[0..3]") );
      dataTF = new JTextField [4];

      for ( int i=0;i<4;i++ )
      {
        dataTF[i] = new JTextField("0x00");
        dataTF[i].setToolTipText("Prefix 0x, 0X or # for hexadecimal, 0 for octal");
        dJp.add(dataTF[i]);
      }

      gbC.gridwidth = GridBagConstraints.REMAINDER;
      jP.add(dJp,gbC);

      rawTF = new JTextField(20);
      gbC.gridwidth = 1;
      jP.add(rawTF,gbC);

      button = new JButton("Send raw");
      button.addActionListener(a->sca.sendRaw(rawTF.getText()));
      gbC.gridwidth = 1;
      jP.add(button,gbC);
      
      JCheckBox enableTextCB = new JCheckBox("Enable text");
      enableTextCB.addActionListener(a->sca.setDebug(enableTextCB.isSelected()));
      gbC.gridwidth = GridBagConstraints.REMAINDER;
      jP.add(enableTextCB,gbC);
      
      textArea = new JTextArea(4,10); // Minimum width. Will expand to fill available width.
      textArea.setEditable(false);
      textArea.setFont(new Font("Monospaced",Font.PLAIN,12));
      textSP = new JScrollPane(textArea);
      gbC.gridwidth = GridBagConstraints.REMAINDER;
      gbC.fill = GridBagConstraints.HORIZONTAL;
      jP.add(textSP,gbC);

      sca.setTextArea(textArea);

      return jP;
    }

    public void sendCommand()
    {
        byte [] buf = new byte [4];
        int i = 0;
        try
        {
          for ( JTextField tf : dataTF )
          {
            buf[i++] = (byte)Integer.decode(tf.getText()).intValue();
          }
        }
        catch ( java.lang.NumberFormatException ne ){}

        sca.allocateTxRxBuffers(1);
        String [] tokens = ((String)commandCB.getSelectedItem()).split("_");
        if (tokens[0].equals("I2C"))
        {
          int channel = Integer.decode(((String)channelCB.getSelectedItem())).intValue()-3;
          sca.infoTx(commandCB.getSelectedItem()+"/"+channel,buf);
        }
        else
        {
          sca.infoTx((String)commandCB.getSelectedItem(),buf);
        }

        sca.putBytes( 0, true );

    }
      
    public void updateCommandCB()
    {
      commandCB.removeAllItems();    
      HashMap<String, ScaCommandDescriptor > gMap = Sca.map.get((String)groupCB.getSelectedItem());
      Set<String> keys = (Set<String>)gMap.keySet();
      Iterator<String> it = keys.iterator();
      while (it.hasNext())
      {
        String key = it.next();
        commandCB.addItem(key);
      }
      ScaCommandDescriptor commandDescriptor = gMap.get((String)commandCB.getSelectedItem());
      channelCB.removeAllItems();
      String [] channels = commandDescriptor.getChannel().split(",");
      for ( String channel : channels ) channelCB.addItem(channel);
    }

  }
}
