package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.file.*;
import java.nio.file.attribute.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.gui.*;

/**
<p>This class encapsulates the I/O interface with the SCA ASIC.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
class ScaDepp
{
  static ScaDepp staticThis;
  Preferences prefs;
  cbsw.RegisterIo io;
  @XmlAttribute(name="deppBaseRx")
  int deppBaseRx;
  @XmlAttribute(name="deppBaseTx")
  int deppBaseTx;
  byte hdlcSeqTx = 0;
  @XmlAttribute(name="hdlcAddress")
  byte scaHdlcAddress;
  @XmlAttribute(name="id")
  byte scaId;
  @XmlAttribute(name="spiFrequency")
  int spiFrequency;

  ByteBuffer scaTx;
  ByteBuffer scaRx;

  boolean debug = false;
  JTextArea textArea;

  public BitSet gpioOut, gpioDirection, adcCurrentSource;

  public ScaDepp()
  {
    staticThis = this;
    prefs = Preferences.userNodeForPackage(ScaDepp.class).node("SCA");
    gpioDirection = new BitSet(32);
    gpioOut = new BitSet(32);
    adcCurrentSource = new BitSet(32);
    scaHdlcAddress = (byte)prefs.getInt("sca-address",0);
    textArea = null;
  }

  public void init(cbsw.RegisterIo io){ this.io = io; }

  public void setDebug(boolean debug){ this.debug = debug; }

  public void setTextArea(JTextArea tA){ textArea = tA; }

  public void allocateTxRxBuffers(int nFrames)
  {
    scaTx = ByteBuffer.allocate(nFrames*16 + 5);
    scaTx.order(ByteOrder.LITTLE_ENDIAN);
    scaTx.position(4);
    scaRx = ByteBuffer.allocate(nFrames*4);
  }

  public void resetBuffers()
  {
    scaTx.clear();
    scaTx.position(4);
    scaRx.clear();
  }

  public int getRxIndex(int frame, int i){ return scaRx.get(frame*4 + i); }

  public void showText(String txt)
  {
      if (textArea == null)
      {
        System.out.println(txt);
      }
      else
      {
	textArea.append(txt+"\n");
      }
  }


  public void getBytes(int addr, byte [] buf )
  {
      int tries = 0;
      boolean retry = true;
      while (retry)
      {
        try
        {
	  io.get(deppBaseRx,addr,buf);
	  retry = false;
        }
	catch (cbsw.RegisterIoException de)
	{
	  tries++;
	  if (tries>50)
	  {
	    retry = false;
       	    System.err.println("ScaDepp::getBytes(): DEPP exception. "+de.getDescription());
	  }
	}
      }
  }

  public void showBuffer()
  {
    byte [] buf = scaTx.array();
    StringBuffer hexSB = new StringBuffer();
    for (int b=4;b<buf.length;b++)
    {
      hexSB.append(String.format("%1$02x ",buf[b]));
      if (b%16==3)
      {
        System.err.println(hexSB.toString());
        hexSB.setLength(0);
      }
    }
  }
    
  public void putBytes(int addr, boolean waitflag )
  {
      
// Pad to end with zeros
      int cap = scaTx.capacity();
      for (int pad=scaTx.position();pad<cap;pad++) {scaTx.put( (byte)0 );}

// Write the header
      scaTx.position(0);
      int len = scaTx.capacity() - 4;
      scaTx.putShort( (short)len );
      if (waitflag) addr |= 0x8000;
      scaTx.putShort( (short)addr );

      byte [] buf = scaTx.array();
      if (buf.length%16 != 5){System.err.println("ScaDepp::putBytes(): Wrong buffer length."); return;}
      if (buf[buf.length-1] != 0){System.err.println("ScaDepp::putBytes(): Buffer incorrectly terminated."); return;}

      int tries = 0;
      boolean retry = true;
      while(retry)
      {
        try
        {
	  io.put(deppBaseTx, scaTx);
	  retry = false;
        }
	catch (cbsw.RegisterIoException de)
	{
	  tries++;
	  if (tries>10)
	  {
	    retry = false;
  	    System.err.println("ScaDepp::putBytes(): DEPP exception. "+de.getDescription()+" Base="+deppBaseTx+" Addr="+addr+" Length="+buf.length);
	    showBuffer();
	  }
	}
      }
  }

  public int crc(byte [] buf)
  {
    int poly = 0x1021;
    boolean fb;

    int crc = 0xffff;

    for ( int nbyte=0; nbyte<buf.length; nbyte++ )
    {
      for ( int i = 0; i<8; i++ )
      {

        boolean crc15 = (crc & 0x8000) != 0 ? true : false;
        int nbit = i;
        boolean db = (buf[nbyte] & (1<<nbit)) != 0 ? true : false;

        fb = crc15 != db;

        crc = fb ? poly ^ (crc<<1) : crc<<1;
        crc = fb ? (crc | 0x1) : (crc & 0xfffe);
      }
    }

    // Reflect the crc bits

    int crcout = 0;
    for ( int nbit=0; nbit<16; nbit++ )
    {
      int isSet = (crc & (1<<nbit)) != 0 ? 1 : 0;
      crcout <<= 1;
      crcout |= isSet;
    }

    return (crcout&0xffff);

  }

  public void sendRaw( String raw )
  {
    // HDLC transaction looks like this:
    // SOF,Addr,Control,ID,Ch,Len,Cmd,D0,D1,D2,D3,FCS,EOF
    // For SCA, Addr=0, ID=0x00 and 0xFF are reserved.

    System.out.println("raw len= "+raw.length());
    allocateTxRxBuffers(1);
    
    scaTx.put( (byte)((1+raw.length()/2)<<3) ); // Length in bits including this byte

    for (int b=0;b<raw.length()/2;b++)
    {
      scaTx.put( (byte)Integer.parseInt(raw.substring(2*b,2*b+2),16) );
    }

    scaId++;
    if (scaId == -1) scaId = 0x1;
    hdlcSeqTx++;
    if (hdlcSeqTx == 0x8) hdlcSeqTx = 0x0;

    // Write the bytes to the transaction buffer
    putBytes( 0, false );

    return;
  }

  public void getResponseBytes(String command, int frame, byte [] rxbuf, byte [] buf)
  {
    ScaDescriptor sca = ScaRx.getDescriptor( command );

    for (int j=0;j<sca.data.length();j++)
    {
      if ( sca.data.charAt(j) == '-' ) continue;
      int idx = Integer.parseInt(sca.data.substring(j,j+1),16);
      buf[idx%4] = rxbuf[16*frame+6+j];
    }
  }

  public void infoTx( String command, int info)
  {
      byte [] buf = new byte [4];
      buf[0] = (byte)(info&0xff);
      buf[1] = (byte)((info>>8)&0xff);
      buf[2] = (byte)((info>>16)&0xff);
      buf[3] = (byte)((info>>24)&0xff);
      infoTx(command,buf);
      return;
  }

  public void infoTx( String command, BitSet info)
  {
      byte [] buf = Arrays.copyOfRange(info.toByteArray(),0,4);
      infoTx(command,buf);
      return;
  }

  public void infoTx( String command, byte [] info )
  {
    ScaDescriptor sca = Sca.getDescriptor( command );
    ScaDescriptor scaDescRx = ScaRx.getDescriptor( command );

    // HDLC transaction looks like this:
    // SOF,Addr,Control,ID,Ch,Len,Cmd,D0,D1,D2,D3,FCS,EOF
    // For SCA, Addr=0, ID=0x00 and 0xFF are reserved.

    // Align the start position to next 16 byte boundary

    int startBufAddr = ((scaTx.position()+11)/16)*16+4;
    scaTx.position(startBufAddr);
    int hdlclen = 7+sca.data.length();

    scaTx.put( (byte)(hdlclen<<3) ); // Length in bits including this byte
    scaTx.put( scaHdlcAddress );
    byte hdlcSeq = (byte)((hdlcSeqTx&0x7) << 5);
    hdlcSeq |= ((hdlcSeqTx&0x7) << 1);
    scaTx.put(hdlcSeq);
    scaTx.put( scaId ); // ID
    scaTx.put( sca.channel );
    scaTx.put( sca.length );
    scaTx.put( sca.code );

    if (sca.data.length()>4) {System.err.println("ScaDepp.infoTx(): Invalid tx info descriptor length.");}

    for (int j=0;j<sca.data.length();j++)
    {
      if ( sca.data.charAt(j) == '-' )
      {
	scaTx.put( (byte)0 );
      }
      else
      {
        int idx = Integer.parseInt(sca.data.substring(j,j+1),16);
        scaTx.put( info[idx%4] );
      }
    }

// The RX index array locates the returned data within a frame
    if (scaDescRx.data.length()>4) {System.err.println("ScaDepp.infoTx(): Invalid rx info descriptor length.");}
    
    byte [] xref = {(byte)-1,(byte)-1,(byte)-1,(byte)-1};
    for (int j=0;j<4;j++)
    {
      if ( j>=scaDescRx.data.length() || scaDescRx.data.charAt(j) == '-' ) continue;
      int idx = Integer.parseInt(scaDescRx.data.substring(j,j+1),16);
      xref[idx%4] = (byte)(6+j);
    }
    scaRx.put( xref );

    byte [] scaBuffer = new byte [hdlclen];
    scaTx.position(startBufAddr);
    scaTx.get(scaBuffer,0,hdlclen);

    if (debug)
    {
      StringBuffer hexSB = new StringBuffer();
      for ( int j=0;j<hdlclen;j++ ) hexSB.append(String.format("%1$02x",scaBuffer[j]));
      hexSB.append( " " + command + String.format(" [seq,ch,code,crc]=[%1$02x,%2$02x,%3$02x,%4$04x]",scaId,sca.channel,sca.code,crc(Arrays.copyOfRange(scaBuffer,1,hdlclen))));
      showText( hexSB.toString() );
    }

    scaId++;
    if (scaId == -1) scaId = 0x1;
    hdlcSeqTx++;
    if (hdlcSeqTx == 0x8) hdlcSeqTx = 0x0;

    return;

  }

  // Send supervisory frame
  public void sfTx( HdlcControl ctl )
  {
    allocateTxRxBuffers(1);
    scaTx.put( (byte)(3<<3) );
    scaTx.put( scaHdlcAddress );
    scaTx.put( ctl.code );

    byte [] scaBuffer = new byte [3];
    scaTx.position(4);
    scaTx.get(scaBuffer,0,scaBuffer.length);

    if (debug)
    {
      StringBuffer hexSB = new StringBuffer();
      for ( int j=0;j<scaBuffer.length;j++ ) hexSB.append(String.format("%1$02x",scaBuffer[j]));
      hexSB.append( " " + ctl.name() + String.format(" [crc]=[%1$04x]",crc(Arrays.copyOfRange(scaBuffer,1,3))));
      showText( hexSB.toString() );
    }

    putBytes( 0x0000, true );

    return;
  }

  public void connect()
  {
    hdlcSeqTx = 0;
    sfTx( HdlcControl.UF_SABM );
  }

  public void reset()
  {
    scaId = 0x1;
    hdlcSeqTx = 0;
    sfTx( HdlcControl.UF_RSET );
  }

  public HashMap<String, ScaCommandDescriptor > getMap(String group)
  {
    return Sca.getGroup(group);
  }

// Initial setup
  public void setup()
  {

    allocateTxRxBuffers(8);

// GPIO initial setup

    infoTx("GPIO_W_DATAOUT",gpioOut);
    infoTx("GPIO_W_DIRECTION",gpioDirection);


// SPI initial setup. Frequency in kHz in XML file.
    infoTx("SPI_W_CTRL",0x00000000); // SS manual mode
    int spiFrequencyDivisor = 20000/spiFrequency - 1;
    infoTx("SPI_W_FREQ",spiFrequencyDivisor&0xffff);
    infoTx("SPI_W_SS",0x00000000); // Start with nothing selected


// I2C initial setup
    infoTx("I2C_W_CTRL/0",0x00000000);
    infoTx("I2C_W_CTRL/1",0x00000000);
    infoTx("I2C_W_CTRL/2",0x00000000);

// Write the bytes to the transaction buffer
    putBytes( 0, true );

  }

// Activates required interfaces
  public boolean enableInterfaces()
  {
    allocateTxRxBuffers(6);
// Activate interfaces
    infoTx("NC_W_CRB",0x3e); // GPIO, SPI, I2C0, I2C1 & I2C2
    infoTx("NC_W_CRC",0x00); // Other I2C disabled
    infoTx("NC_W_CRD",0x38); // ADC, DAC & JTAG

    infoTx("NC_R_CRB",0x00);
    infoTx("NC_R_CRC",0x00);
    infoTx("NC_R_CRD",0x00);
    //    infoTx("NC_W_CRA",0x55); // Was SLVS but has disappeared from manual

    putBytes( 0x0000, true );
    try { Thread.sleep(100); } catch(InterruptedException ex){}

    byte [] rxbuf = new byte [0x30];
    try { io.get( deppBaseRx, 0x0030, rxbuf ); } catch (cbsw.RegisterIoException de){System.err.println("ScaDepp::enableInterfaces(): RegisterIoException "+de.getDescription());}

    return (rxbuf[0x07]==0x3e)&&(rxbuf[0x04]==0x00)&&(rxbuf[0x17]==0x00)&&(rxbuf[0x14]==0x00)&&(rxbuf[0x27]==0x38)&&(rxbuf[0x24]==0x00);
  }

// Read SCA ID
  public String readId()
  {
    allocateTxRxBuffers(1);
    infoTx("NC_R_ID",0x00);

    putBytes( 0x0000, true );
    try { Thread.sleep(100); } catch(InterruptedException ex){}

    byte [] rxbuf = new byte [16];
    try { io.get( deppBaseRx, 0x0000, rxbuf ); } catch (cbsw.RegisterIoException de){System.err.println("ScaDepp.readId(): RegisterIoException "+de.getDescription());}

    return String.format("0x%02x%02x%02x",(rxbuf[6]<<16),(rxbuf[9]<<8),rxbuf[8]);
  }

  public void setDac(int dacId,byte dacValue)
  {
    allocateTxRxBuffers(1);
    infoTx("DAC_W_"+dacId,dacValue);
    putBytes(0,true);
    return;
  }

  public void setGpio(int gpioId, int gpioValue)
  {
    if (gpioValue==1) gpioOut.set(gpioId);
    else gpioOut.clear(gpioId);
      
    allocateTxRxBuffers(1);
    infoTx("GPIO_W_DATAOUT",gpioOut);
    putBytes(0,true);
    return;
  }
    
  public void setGpioDirection(int gpioId, int gpioValue)
  {
    if (gpioValue==1) gpioDirection.set(gpioId);
    else gpioDirection.clear(gpioId);
      
    allocateTxRxBuffers(1);
    infoTx("GPIO_W_DIRECTION",gpioDirection);
    putBytes(0,true);
    return;
  }
    
  public void setAdcCurrentSource(int adcId, int adcValue)
  {
    if (adcValue==1) adcCurrentSource.set(adcId);
    else adcCurrentSource.clear(adcId);
      
    allocateTxRxBuffers(1);
    infoTx("ADC_W_CURR",adcCurrentSource);
    putBytes(0,true);
    return;
  }

// Read I2C register
    
  public byte readI2c(int i2cBus, int i2cSlaveAddress, int off)
  {
    allocateTxRxBuffers(6);

    infoTx("I2C_W_CTRL/"+i2cBus,(2<<2)|0x2);

// First two bytes are register start offset
    infoTx("I2C_W_DAT0/"+i2cBus,off);
    infoTx("I2C_M_7B_W/"+i2cBus,i2cSlaveAddress);

// Now read back register byte
    infoTx("I2C_W_CTRL/"+i2cBus,(1<<2)|0x2);
    infoTx("I2C_M_7B_R/"+i2cBus,i2cSlaveAddress);
    infoTx("I2C_R_DAT3/"+i2cBus,0x00000000);

    putBytes(0,true);

// Get the byte from the buffered responses

    byte [] rxbuf = new byte [16];
    byte [] buf = new byte [4];
    getBytes( 80, rxbuf );
    getResponseBytes("I2C_R_DAT0/0",0,rxbuf,buf); // Can choose any I2C bus

    //    System.out.println(String.format("buf=%1$02x %2$02x %3$02x %4$02x",buf[0],buf[1],buf[2],buf[3]));
    
    return buf[0];
  }

// Write I2C register

  public void writeI2c(int i2cBus, int i2cSlaveAddress, int off, int reg)
  {
    allocateTxRxBuffers(3);

    infoTx("I2C_W_CTRL/"+i2cBus,(3<<2)|0x2);

// First two bytes are register start offset followed by data byte
    infoTx("I2C_W_DAT0/"+i2cBus,((reg&0xff)<<16)|off);
    infoTx("I2C_M_7B_W/"+i2cBus,i2cSlaveAddress);

    putBytes(0,true);
    
    return;
  }

  static public String gpioControl(String [] args)
  {
// Args are testid:expected|gpio:direction:value|gpio:direction:value...

    String [] gpios = Arrays.copyOfRange(args,1,args.length);
    
    for (String gpio : gpios)
    {
      String [] tokens = gpio.split(":"); 
      int gpioId = Integer.decode(tokens[0]);
      if (tokens[1].equalsIgnoreCase("out"))
        staticThis.gpioDirection.set(gpioId);
      else
        staticThis.gpioDirection.clear(gpioId);
      
      if (tokens[2].equals("1"))
        staticThis.gpioOut.set(gpioId);
      else
        staticThis.gpioOut.clear(gpioId);
    }
    
    staticThis.allocateTxRxBuffers(2);
    staticThis.infoTx("GPIO_W_DIRECTION",staticThis.gpioDirection);
    staticThis.infoTx("GPIO_W_DATAOUT",staticThis.gpioOut);
    staticThis.putBytes(0,true);

// Some of the GPIOs have an RC network so give some recovery time
    try {Thread.sleep(30);} catch (InterruptedException ie){}
    
    staticThis.allocateTxRxBuffers(1);
    staticThis.infoTx("GPIO_R_DATAIN",0x00);
    staticThis.putBytes(0,true);
    
// Get the byte from the buffered responses

    byte [] rxbuf = new byte [16];
    byte [] buf = new byte [4];
    staticThis.getBytes( 0, rxbuf );
    staticThis.getResponseBytes("GPIO_R_DATAIN",0,rxbuf,buf);
    BitSet gpioRead = BitSet.valueOf(buf);

    StringBuffer rv = new StringBuffer("OK|ScaDepp.gpioControl");

    boolean pass = true;
    String [] testTokens = args[0].split(":");
    for (String gpio : gpios)
    {
      String [] tokens = gpio.split(":"); 
      int gpioId = Integer.decode(tokens[0]);
      boolean gpioState = gpioRead.get(gpioId);
      if (gpioState)
      {
	if (testTokens[1].equals("0")) pass = false;
	rv.append("|1");
      }
      else
      {
	if (testTokens[1].equals("1")) pass = false;
	rv.append("|0");
      }
    }
    try { TesterSequence.testSummaryXml.write(String.format("<gpio test=\"%s\" pass=\"%b\" />\n",testTokens[0],pass)); }
    catch (IOException ioe){ return "FAIL|ScaDepp.gpioControl|IOException";}

    return rv.toString();
  }

  public int getAdc(int adc)
  {
    int framesPerADC = 3; // i.e. number of SCA commands per ADC
    allocateTxRxBuffers(framesPerADC);
    //    infoTx("ADC_W_CURR",0x00000000);
    infoTx("ADC_W_CURR",adcCurrentSource);
    infoTx("ADC_W_MUX",adc);
    byte [] go = {(byte)0x01};
    infoTx("ADC_GO",go);

    putBytes(0,true);

    //    try {Thread.sleep(10);}catch(InterruptedException ex){};

// Now decode the SCA responses to extract the ADC values

    byte [] rxbuf = new byte [16];
    byte [] buf = new byte [4];
    getBytes( 32, rxbuf );
    getResponseBytes("ADC_GO",0,rxbuf,buf);
    
    ByteBuffer rxBB = ByteBuffer.wrap(buf); rxBB.order(ByteOrder.LITTLE_ENDIAN);
    return rxBB.getInt(0);
  }
    
  public static String getAdc(String [] args)
  {
    int samples;
    int bins;
    double binMinimum;
    double binWidth;
    double binMaximum;
    double maximumRMS;
    try
    {
      samples = Integer.decode(args[0]);
      bins = Integer.valueOf(args[1]);
      binMinimum = Double.valueOf(args[2]);
      binWidth = Double.valueOf(args[3]);
      binMaximum = binMinimum + bins*binWidth;
      maximumRMS = Double.valueOf(args[4]);
    }
    catch (NullPointerException npe){ return "FAIL|ScaDepp.getAdc|Invalid argument"; }
  
    if (samples < 1) return "FAIL|ScaDepp.getAdc|Invalid argument";
    
    try
    {
      BufferedWriter writer = Files.newBufferedWriter(TesterSequence.testDirectory.resolve(Path.of("adc.dat")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);

// Bin description
      writer.write(""+bins+","+binMinimum+","+binWidth); // #bins, low limit, bin width
      writer.newLine();
// Column headers
      for (int adc=0;adc<31;adc++) writer.write("adc"+Integer.toString(adc)+",");
      writer.write("adc"+Integer.toString(31));
      writer.newLine();

// Data
      int adcVal;
      for (int i=0;i<samples;i++)
      {
        for (int adc=0;adc<31;adc++)
        {
	  adcVal = staticThis.getAdc(adc);
	  writer.write(Integer.toString(adcVal)+",");
        }
	adcVal = staticThis.getAdc(31);
	writer.write(Integer.toString(adcVal));
	writer.newLine();
      }
      writer.flush();
      writer.close();
    
// Create the histograms (get data from the file we've just created)
      BufferedReader reader = Files.newBufferedReader(TesterSequence.testDirectory.resolve(Path.of("adc.dat")));
      Histogram hist = new Histogram("");
      try { hist.readStream(reader); } catch(Histogram.EOFException eof){}
      reader.close();
      
// Set the Paint for all series to be the same block colour
      hist.setPaint(new Color(0x3333cc));

// Output the plots and dataset statistics
      for ( Comparable key : hist.keys() )
      {
        hist.hide(); // Hide any series that are currently visible
        hist.show(key); // Draw this series
        hist.saveAsSVG(TesterSequence.testDirectory.resolve(Path.of((String)key+".svg")).toFile()); // Write as an SVG file
        double mean = hist.mean(key); // Unbinned mean
        double rms = hist.rms(key); // Unbinned RMS
        TesterSequence.testSummaryXml.write(String.format("<adc id=\"%s\" mean=\"%.2f\" rms=\"%.2f\" />\n",(String)key,mean,rms));
      }
      TesterSequence.testSummaryXml.flush();
    }
    catch (IOException ioe){return "FAIL|ScaDepp.getAdc|IO exception";}

    return "OK|ScaDepp.getAdc";
  }
}

