package cbsw.lhcb.pdmdb;

import java.util.prefs.Preferences;
import javax.xml.bind.annotation.*;
import java.nio.file.Paths;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.net.URL;
import java.net.MalformedURLException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.*;
import java.util.*;
import cbsw.*;
import cbsw.lhcb.pdmdb.*;

/**
<p>This class encapsulates a collection of the SCA command responses.</p>
<p>Unmarshall SCA RX register descriptors into statically accessible members of this ScaRx class.</p>
<p>Note that the class members are static but unfortunately not immutable because of the need to use JAXB setters.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="scacommands")
class ScaRx
{
public static ArrayList<ScaCommandDescriptor> descriptors;
public static HashMap< String, HashMap< String, ScaCommandDescriptor > > map;

  @XmlElement(name="SCACommand")
  public void setDescriptors(ArrayList<ScaCommandDescriptor> descriptors) { ScaRx.descriptors = descriptors; }
  public ArrayList<ScaCommandDescriptor> getDescriptors(){return descriptors;}
    
  static
  {
    Cbsw.unmarshal("https://www.hep.phy.cam.ac.uk/lhcb/PDMDB/sca/SCARXCommands.xml",ScaRx.class);

    ScaRx.map = new HashMap< String, HashMap< String, ScaCommandDescriptor > >();
    map.put("NC", new HashMap< String, ScaCommandDescriptor >());
    map.put("I2C", new HashMap< String, ScaCommandDescriptor >());
    map.put("SPI", new HashMap< String, ScaCommandDescriptor >());
    map.put("GPIO", new HashMap< String, ScaCommandDescriptor >());
    map.put("JTAG", new HashMap< String, ScaCommandDescriptor >());
    map.put("DAC", new HashMap< String, ScaCommandDescriptor >());
    map.put("ADC", new HashMap< String, ScaCommandDescriptor >());

    for (ScaCommandDescriptor reg : ScaRx.descriptors)
    {
      HashMap<String,ScaCommandDescriptor> gMap = ScaRx.map.get(reg.getGroup());
      gMap.put( reg.getName(), reg );
    }  
  }
    
  public ScaRx(){}

  public static HashMap<String, ScaCommandDescriptor> getGroup(String gname){ return ScaRx.map.get(gname); }

  public static ScaDescriptor getDescriptor(String command)
  {

    String [] tokens = command.split("[_/]");
    String [] cmdch = command.split("/");
 
    HashMap<String, ScaCommandDescriptor> gMap = map.get(tokens[0]);
    ScaCommandDescriptor commandDescriptor;

    byte scaChannel;
    if (tokens[0].equals("I2C"))
    {
      // I2C commands have bus number appended after "/"
      commandDescriptor = gMap.get(cmdch[0]);
      scaChannel = (byte)(Integer.decode(cmdch[cmdch.length-1]).intValue()+3);
    }
    else
    {
    // For all other commands get channel from map
      commandDescriptor = gMap.get(command);
      scaChannel = (byte)Integer.decode(commandDescriptor.getChannel()).intValue();
    }

    byte scaCode = commandDescriptor.getCode();

    byte scaLength = (byte)commandDescriptor.getLength();

    return new ScaDescriptor(scaChannel,scaCode,scaLength,commandDescriptor.getData());
  }
    
  public static void main( String[] args )
  {
    for (ScaCommandDescriptor reg : ScaRx.descriptors){ System.out.println(reg.getName()); }  
  }

}

