package cbsw.lhcb.pdmdb;

import java.util.List;
import java.util.ArrayList;
import java.util.prefs.Preferences;
import java.awt.GridBagConstraints;
import java.text.DecimalFormat;
import cbsw.gui.GuiJPanel;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.nio.file.*;
import java.nio.file.attribute.*;

/**
<p>This class implements phase scans and DAC scans.</p>
<p>For phase scans dont forget to set a suitable pattern in the GUI (10101010101...)</p>
**/
public class ScanControl
{
  static ScanControl staticThis;
  cbsw.RegisterIo io;
  Preferences prefs;
  javax.swing.JTextField progressTF;

/**
<p>Enumerate scan modes.</p>
**/
  enum ScanMode { PHASE_SCAN, DAC_SCAN; }

  public ScanControl(){ prefs = Preferences.userNodeForPackage(JPDMDBDepp.class).node("JPDMDBDepp"); staticThis = this; }

  public void init(cbsw.RegisterIo io) { this.io = io; }

  public GuiJPanel makePanel()
  {
    GuiJPanel jP = new GuiJPanel(prefs,"scan.");

    jP.setLayout( new java.awt.GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();

    gbC.insets = new java.awt.Insets(2,2,2,2);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;

    gbC.fill = GridBagConstraints.NONE;
    gbC.anchor = GridBagConstraints.CENTER;

    gbC.gridwidth = 1;
    jP.add( jP.makeTF(new DecimalFormat("#"),"step", "2", 10, "Increment", "Increment size [1,63] ([1,255])",true).getParent(),gbC );
    jP.add( jP.makeTF(new DecimalFormat("#"),"begin", "0", 10, "Begin value", "Start value [0,63] ([0,255])",true).getParent(),gbC );
    jP.add( jP.makeTF(new DecimalFormat("#"),"end", "63", 10, "End value", "End value [0,63] ([0,255])",true).getParent(),gbC );

    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jP.add( jP.makeTF("fileName", "scan.dat", 20, "Output filename", "Data file name",true).getParent(),gbC );
    /*
    gbC.gridwidth = 1;
    jP.add( jP.makeTF(new DecimalFormat("#"),"dtm", "0", 1, "DTM", "DTM to scan [0,2]",true).getParent(),gbC );
    jP.add( jP.makeTF(new DecimalFormat("#"),"gbtx", "0", 1, "GBTX", "GBTX to scan [0,1]",true).getParent(),gbC );
    */
    javax.swing.JButton button = new javax.swing.JButton( "Scan phase" );
    button.addActionListener( l->{(new ScanTask(ScanMode.PHASE_SCAN)).execute();} );
    gbC.gridwidth = 1;
    jP.add( button, gbC );

    button = new javax.swing.JButton( "Scan DAC" );
    button.addActionListener( l->{(new ScanTask(ScanMode.DAC_SCAN)).execute();} );
    gbC.gridwidth = 1;
    jP.add( button, gbC );

    progressTF = new javax.swing.JTextField(16);
    progressTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jP.add( progressTF,gbC );
    
    return jP;
  }
    
  static public String doDacScan(String [] args)
  {
    Path dataDirectory;
// Create directory for phase scan data files
    try
    {
      dataDirectory = Files.createDirectories(TesterSequence.testDirectory.resolve(Path.of("DacScanData")));
    }
    catch (java.io.IOException ioe){return "FAIL|ScanControl.doDacScan|IOException "+ioe.getMessage();}
    
// Make a single list of all the GBTXes
    List<GbtxControl> gbtxes = new ArrayList<GbtxControl>();
    for (DtmControl dtm : PdmdbMaster.pdmdbMaster.dtmList) gbtxes.addAll(dtm.gbtxList);
    
    try
    {

      MuDaq.muDaq.cfg.put("tfc-d","0",staticThis.io); // Set a fixed pattern on the down links
      MuDaq.muDaq.cfg.put("tfctog-enable","8",staticThis.io); // Enable the TFC test pulse
// Do scan
      String fileNameFormat = dataDirectory.toString()+"/dacScan.%d.%d.dat";
      staticThis.scan(ScanMode.DAC_SCAN,0,63,1,gbtxes,fileNameFormat);
    }
    catch ( cbsw.RegisterIoException de ){ return "FAIL|ScanControl.doDacScan|RegisterIoException "+de.getDescription(); }
    
// Make histograms

    makeDacScanPlots(dataDirectory);
    
    return "OK|ScanControl.doDacScan";
  }

// Create 2d plots for each dacScan.dtmId.gbtxId.dat file
  public static void makeDacScanPlots(Path dataPath)
  {
    for (int gbtxIndex=0;gbtxIndex<6;gbtxIndex++)
    {
      int dtmId = gbtxIndex/2;
      int gbtxId = gbtxIndex%2;
      java.io.FileInputStream reader;
      
      try
      {
        reader = new java.io.FileInputStream(dataPath.resolve(Path.of("dacScan."+dtmId+"."+gbtxId+".dat")).toFile());
      }
      catch (java.io.FileNotFoundException fnf){continue;}

      double [][] xyzArray = new double [3][64*112];

      byte [] dataBuf = new byte [4+7168];
      java.nio.ByteBuffer bb = java.nio.ByteBuffer.wrap(dataBuf);
      bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);

      try
      {
	while ( reader.read(dataBuf) != -1 )
	{
          int [] hitCounter = new int [112];
	  bb.position(0);
	  int stepValue =  bb.getInt();
	  //	  System.err.println("dtm="+dtmId+" gbtx="+gbtxId+" step="+stepValue);
          for (int ievt=0;ievt<128;ievt++)
          {
	    bb.position(4+56*ievt+14); // 4 14-byte timeslots per event, data in second slot
    	    for (int igrp=0;igrp<14;igrp++)
	    {
	      byte thisByte = bb.get();
	      for (int ihit=0;ihit<8;ihit++)
	      {
		if ( (thisByte & (byte)(1<<ihit)) != 0 ) hitCounter[8*igrp+ihit]++;
	      }
	    }
          }
    	  for (int igrp=0;igrp<14;igrp++)
	  {
	    for (int ihit=0;ihit<8;ihit++)
	    {
 	      xyzArray[0][stepValue*112+8*igrp+ihit] = stepValue;
	      xyzArray[1][stepValue*112+8*igrp+ihit] = 8*igrp+ihit;
	      xyzArray[2][stepValue*112+8*igrp+ihit] = hitCounter[8*igrp+ihit];
	    }
	  }
        }

        cbsw.gui.Histogram2D hist = new cbsw.gui.Histogram2D("");
	hist.setPaintScale(0.0,128.0,128.0);
        org.jfree.data.xy.DefaultXYZDataset dataset = hist.getDataset();
        String key = "gbtx"+gbtxIndex;
        dataset.addSeries(key,xyzArray);
        hist.saveAsSVG(dataPath.resolve(Path.of(key+".svg")).toFile()); // Write as an SVG file
      }
      catch (java.io.IOException ioe){}
    }
  }
    
  static public String doPhaseScan(String [] args)
  {
    Path dataDirectory;
// Create directory for phase scan data files
    try
    {
      dataDirectory = Files.createDirectories(TesterSequence.testDirectory.resolve(Path.of("PhaseScanData")));
    }
    catch (java.io.IOException ioe){return "FAIL|ScanControl.doPhaseScan|IOException "+ioe.getMessage();}
    
// Make a single list of all the GBTXes
    List<GbtxControl> gbtxes = new ArrayList<GbtxControl>();
    for (DtmControl dtm : PdmdbMaster.pdmdbMaster.dtmList) gbtxes.addAll(dtm.gbtxList);
    
    try
    {
// Set a fixed pattern on the down links

      MuDaq.muDaq.cfg.put("tfctog-enable","0",staticThis.io);
      MuDaq.muDaq.cfg.put("tfc-d","A",staticThis.io);
// Do scan
      String fileNameFormat = dataDirectory.toString()+"/phaseScan.%d.%d.dat";
      staticThis.scan(ScanMode.PHASE_SCAN,0,14,1,gbtxes,fileNameFormat);
    }
    catch ( cbsw.RegisterIoException de ){ return "FAIL|ScanControl.doPhaseScan|RegisterIoException "+de.getDescription(); }
    
// Make histograms

    makePhaseScanPlots(dataDirectory);
    
    return "OK|ScanControl.doPhaseScan";
  }

// Create 2D plot for each phaseScan.dtmId.gbtxId.dat file
  public static void makePhaseScanPlots(Path dataPath)
  {
    for (int gbtxIndex=0;gbtxIndex<6;gbtxIndex++)
    {
      int dtmId = gbtxIndex/2;
      int gbtxId = gbtxIndex%2;
      java.io.FileInputStream reader;
      
      try
      {
        reader = new java.io.FileInputStream(dataPath.resolve(Path.of("phaseScan."+dtmId+"."+gbtxId+".dat")).toFile());
      }
      catch (java.io.FileNotFoundException fnf){continue;}

      double [][] xyzArray = new double [3][15*14];
      byte [] lastByte = new byte [14];

      byte [] dataBuf = new byte [4+7168];
      java.nio.ByteBuffer bb = java.nio.ByteBuffer.wrap(dataBuf);
      bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);

      try
      {
	while ( reader.read(dataBuf) != -1 )
	{
          int [] groupCounter = new int [14];
	  bb.position(0);
	  int stepValue =  bb.getInt();
          for (int ievt=0;ievt<512;ievt++)
          {
    	    for (int igrp=0;igrp<14;igrp++)
	    {
	      byte thisByte = bb.get();
	      if (ievt>0 && (lastByte[igrp]^thisByte) != 0 ) groupCounter[igrp]++;
	      lastByte[igrp] = thisByte;
	    }
          }
    	  for (int igrp=0;igrp<14;igrp++)
	  {
	    xyzArray[0][stepValue*14+igrp] = stepValue;
	    xyzArray[1][stepValue*14+igrp] = igrp;
	    xyzArray[2][stepValue*14+igrp] = groupCounter[igrp];
	  }
        }

        cbsw.gui.Histogram2D hist = new cbsw.gui.Histogram2D("");
	hist.setPaintScale(0.0,256.0,16.0);
        org.jfree.data.xy.DefaultXYZDataset dataset = hist.getDataset();
        String key = "gbtx"+gbtxIndex;
        dataset.addSeries(key,xyzArray);
        hist.saveAsSVG(dataPath.resolve(Path.of(key+".svg")).toFile()); // Write as an SVG file
      }
      catch (java.io.IOException ioe){}
    }
  }

  public void scan(ScanMode scanMode, int lowerLimit, int upperLimit, int step, List<GbtxControl> gbtxes, String fileNameFormat)
  {
      cbsw.Csr cfg = MuDaq.muDaq.cfg;

      try
      {
	HashMap<String,FileOutputStream> map = new HashMap<String,FileOutputStream>();
	for (GbtxControl gbtx : gbtxes)
	{
	  map.put( "dtm"+gbtx.dtm+".gbtx"+gbtx.gbtx,new FileOutputStream(String.format(fileNameFormat,gbtx.dtm,gbtx.gbtx)));
	}

	byte [] buf = new byte [4+7168];
	java.nio.ByteBuffer bb = java.nio.ByteBuffer.wrap(buf);
	bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);

        cfg.put("pulser.count",prefs,"80",staticThis.io);
        cfg.put("pulser.delay",prefs,"1000",staticThis.io);
      
        cfg.put("pulser.reset","1",staticThis.io); // Assert this reset is to ensure that the delay pipeline is reset
        cfg.put("pulser.reset","0",staticThis.io); // Deassert
      
	for( int stepValue=lowerLimit; Math.abs(stepValue-lowerLimit) <= Math.abs(upperLimit-lowerLimit); stepValue+=step )
        {
	    //	  publish(String.format("Step value %d",stepValue)); // Tell the world where we are

	  switch (scanMode)
	  {
	    case DAC_SCAN:
            for (int ec=0;ec<4;ec++) ScaControl.staticSca.setDac(ec,(byte)stepValue); // Set the DACs
	    break;

	    case PHASE_SCAN:
	    for (GbtxControl gbtx : gbtxes)
	    {
	      for (String eport : gbtx.activeEPorts) gbtx.put(eport+".phase",String.format("%X",stepValue));
	    }
	    break;
	    default:
	  }
	  	  

// Reset the GBT buffer write pointer
          cfg.put("daq.gbtbuf-reset",true,staticThis.io);
          cfg.set("daq.gbtbuf-reset",false);
// Reset the bitflip counters
          cfg.put("counter-reset",true,staticThis.io);
          cfg.put("counter-reset",false,staticThis.io);
      
// Scan is driven by the pulser
	
          cfg.put("pulser-go",true,staticThis.io);
          cfg.set("pulser-go",false);

	  Thread.sleep(10);

 // Read the buffered data and write to file
 // The data in the two FPGA buffers overlap. Read 47..0 from the first and 111..48 from the second.
	  byte [] getBuf0 = new byte [4096];
	  byte [] getBuf1 = new byte [4096];

	  for ( GbtxControl gbtx : gbtxes )
	  {
	    int gbtxIndex = 2*gbtx.dtm + gbtx.gbtx;
	    io.get(0x08+gbtxIndex+2,0,getBuf0);
	    io.get(0x88+gbtxIndex+2,0,getBuf1);
	    bb.clear();
	    bb.putInt(stepValue);
	    for (int ievt=0;ievt<512;ievt++)
	    {
	      for (int igrp=0;igrp<6;igrp++)
	      {
		byte thisByte = getBuf0[8*ievt+igrp];
	        bb.put(thisByte);
	      }
	      for (int igrp=0;igrp<8;igrp++)
	      {
		byte thisByte = getBuf1[8*ievt+igrp];
		bb.put(thisByte);
	      }
	    }
	    bb.flip();
	    map.get("dtm"+gbtx.dtm+".gbtx"+gbtx.gbtx).write(buf);	    
	  }
	  
        }
	
	for (GbtxControl gbtx : gbtxes) map.get("dtm"+gbtx.dtm+".gbtx"+gbtx.gbtx).close();
	
	// Restore a reasonable phase. 0xd seems best for production tester. 
	if (scanMode.equals(ScanMode.PHASE_SCAN))
	{
	  for (GbtxControl gbtx : gbtxes)
	  {
	    for (String eport : gbtx.activeEPorts) gbtx.put(eport+".phase","5");
	  }
	}
	
      }
      catch ( Exception ex ){ System.err.println("ScanControl.scan() Exception "+ex.getMessage()); }
  }

  class ScanTask extends javax.swing.SwingWorker<Boolean, String>
  {
    ScanMode scanMode;

    public ScanTask(ScanMode scanMode){this.scanMode = scanMode;}
      
    @Override
    protected Boolean doInBackground()
    {

      int step;
      int lowerLimit;
      int upperLimit;

      String fileNameFormat;

// Make a single list of all the GBTXes
      List<GbtxControl> gbtxes = new ArrayList<GbtxControl>();
      for (DtmControl dtm : PdmdbMaster.pdmdbMaster.dtmList) gbtxes.addAll(dtm.gbtxList);

    try
    {
      switch (scanMode)
      {
        case DAC_SCAN:
        step = prefs.getInt( "scan.step" , 2 );
        lowerLimit = prefs.getInt( "scan.begin" , 0 );
        upperLimit = prefs.getInt( "scan.end" , 63 );
	fileNameFormat = "dac-"+prefs.get("scan.fileName","scan.%d.%d.dat");
// Clear any pattern on downlink and enable TFC control
        MuDaq.muDaq.cfg.put("tfc-d","0",staticThis.io);
        MuDaq.muDaq.cfg.put("tfctog-enable","1",staticThis.io);
	break;

        case PHASE_SCAN: // Force the step value range to be 0-14 for phase scans
	step = 1;
	lowerLimit = 0;
	upperLimit = 14;
	fileNameFormat = "phase-" + prefs.get("scan.fileName","scan.%d.%d.dat");
// Set fixed pattern on downlink and disable TFC control
        MuDaq.muDaq.cfg.put("tfc-d","A",staticThis.io);
        MuDaq.muDaq.cfg.put("tfctog-enable","0",staticThis.io);
	break;

        default:
        step = prefs.getInt( "scan.step" , 2 );
        lowerLimit = prefs.getInt( "scan.begin" , 0 );
        upperLimit = prefs.getInt( "scan.end" , 63 );
	fileNameFormat = prefs.get("scan.fileName","scan.%d.%d.dat");
      }
      scan(scanMode, lowerLimit, upperLimit, step, gbtxes, fileNameFormat);
    }
    catch (cbsw.RegisterIoException ioe){}
	
      return Boolean.valueOf(true);
    }
      
    @Override
    protected void done(){ try { if (get()) progressTF.setText("Scan complete"); } catch (Exception e){} }

    @Override
    protected void process( java.util.List<String> chunks ){for ( String pass : chunks ) progressTF.setText(pass); }
  }
    
}
