package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.util.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.file.*;
//import java.nio.file.attribute.*;

/**
<p>This class encapsulates a sequence of Commands.</p>
<p>The command sequence is unmarshalled from XML and executed by reflection.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="testerSequence")
public class TesterSequence
{
java.util.prefs.Preferences prefs;
    
@XmlElement(name="command")
ArrayList<Command> commandList;
    
SocketChannel channel;

int iterations;
ListIterator<Command> it;
boolean suspend=false;

javax.swing.JTextField cmdTF;
static javax.swing.JTextArea textArea;
javax.swing.JScrollPane textSP;
javax.swing.JButton recordingButton;
    
boolean recording = false;

static javax.swing.JTextField rootDirectoryTF;
static String testSubdirectory;
static Path testDirectory;
static BufferedWriter commandResponseFile;
static BufferedWriter testSummaryXml;
static Path eyeScanDataDirectory;

static TesterSequence staticThis;

public TesterSequence()
{
  prefs = java.util.prefs.Preferences.userNodeForPackage(TesterSequence.class).node("JPDMDBDepp");
  staticThis = this;
}

public javax.swing.JPanel makePanel()
{
    javax.swing.JPanel jP = new javax.swing.JPanel();
    jP.setLayout(new java.awt.GridBagLayout());
    java.awt.GridBagConstraints gbC = new java.awt.GridBagConstraints();
    gbC.fill = java.awt.GridBagConstraints.BOTH;
    gbC.anchor = java.awt.GridBagConstraints.CENTER;
    gbC.insets = new java.awt.Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    
    javax.swing.JPanel dataJP = new javax.swing.JPanel();
    dataJP.setBorder(javax.swing.BorderFactory.createTitledBorder("Test data root"));
    rootDirectoryTF = new javax.swing.JTextField(prefs.get("chooser.tester.dir","ModuleTestData"));
    rootDirectoryTF.setEditable(false);
    dataJP.add(rootDirectoryTF);
    javax.swing.JButton dataJB = new javax.swing.JButton("Change");
    dataJB.addActionListener(ae->chooseDataDirectory());
    dataJP.add(dataJB);
    
    gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    jP.add(dataJP,gbC);

    javax.swing.JPanel buttonJP = new javax.swing.JPanel();
    buttonJP.setLayout(new java.awt.GridLayout(0,3));
  
    for (Command cmd : commandList )
    {
      javax.swing.JButton button = new javax.swing.JButton(cmd.label);
      button.addActionListener(l->{try {cmd.go();}catch (Exception e) {PdmdbModuleTester.readFlush();textArea.append("TesterSequence: "+e.getMessage()+"\n");}});
      //      gbC.gridwidth = 1;
      //      jP.add(button,gbC);
      buttonJP.add(button);

      javax.swing.JTextField statusTF = new javax.swing.JTextField(60);
      statusTF.setEditable(false);
      statusTF.setFont(new java.awt.Font("Monospaced",java.awt.Font.PLAIN,12));
      //      gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
      //      jP.add(statusTF,gbC);

      cmd.setStatusTextField(statusTF);
      cmd.setButton(button);
    }

    gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    jP.add(buttonJP,gbC);
    
    cmdTF = new javax.swing.JTextField(80);
    cmdTF.addActionListener(l->{PdmdbModuleTester.exec(cmdTF.getText());});
    cmdTF.setFont(new java.awt.Font("Monospaced",java.awt.Font.PLAIN,12));
    gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    jP.add(cbsw.gui.GuiJPanel.addBorder(cmdTF,"Command"),gbC);
  
    textArea = new javax.swing.JTextArea(10,10); // Minimum width. Will expand to fill available width.
    textArea.setEditable(false);
    textArea.setFont(new java.awt.Font("Monospaced",java.awt.Font.PLAIN,12));
    textSP = new javax.swing.JScrollPane(textArea);
    gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    gbC.fill = java.awt.GridBagConstraints.HORIZONTAL;
    jP.add(textSP,gbC);
  
    javax.swing.JPanel playJP = new javax.swing.JPanel();
    
    javax.swing.JButton button = new javax.swing.JButton("Play");
    button.addActionListener(l->{prepare();resume();});
    playJP.add(button);

    button = new javax.swing.JButton("Pause");
    button.addActionListener(l->pause());
    playJP.add(button);

    button = new javax.swing.JButton("Resume");
    button.addActionListener(l->resume());
    playJP.add(button);

    recordingButton = new javax.swing.JButton("Record");
    recordingButton.addActionListener(l->recording());
    playJP.add(recordingButton);

    button = new javax.swing.JButton("First step");
    button.addActionListener(l->{prepare();step();});
    playJP.add(button);
    
    button = new javax.swing.JButton("Step");
    button.addActionListener(l->step());
    playJP.add(button);

    button = new javax.swing.JButton("Flush RX pipe");
    button.addActionListener(l->PdmdbModuleTester.readFlush());
    playJP.add(button);

    gbC.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    jP.add(playJP,gbC);

    return jP;
}

public void chooseDataDirectory()
{
  javax.swing.JFileChooser chooser = new javax.swing.JFileChooser(prefs.get("chooser.tester.dir","ModuleTestData"));
  chooser.setFileSelectionMode(javax.swing.JFileChooser.DIRECTORIES_ONLY);
  int returnVal = chooser.showOpenDialog(null);
  if(returnVal != javax.swing.JFileChooser.APPROVE_OPTION) return;
  if (!chooser.getSelectedFile().isDirectory()) return;
  prefs.put("chooser.tester.dir",chooser.getSelectedFile().getAbsolutePath());
  rootDirectoryTF.setText(prefs.get("chooser.tester.dir","ModuleTestData"));
}

public static void logMessage(String msg)
{

  textArea.append(msg);
  if (commandResponseFile == null) return;
  
  try
  {
    commandResponseFile.write(msg);
    commandResponseFile.flush();
  }
  catch (IOException ioe){ throw new RuntimeException("TesterSequence.logMessage|IOException: "+ioe.getMessage()); }

}

public static String prepareTests(String [] args)
{
  String dir = java.util.prefs.Preferences.userNodeForPackage(TesterSequence.class).node("JPDMDBDepp").get("chooser.tester.dir","ModuleTestData");
    
// Directories are created writable only by the owner but readable and traversabl by all
//  FileAttribute<Set<PosixFilePermission>> attrs = PosixFilePermissions.asFileAttribute(PosixFilePermissions.fromString("rwxr-xr-x"));
  try
  {
   testSubdirectory = "TestData-"+java.time.LocalDateTime.now().format(java.time.format.DateTimeFormatter.ofPattern("yyyyMMdd-A"));
// Create (timestamped) directory for this test cycle
   testDirectory = Files.createDirectories(Path.of(dir+File.separator+testSubdirectory));
   System.err.println("Test output directory is "+testDirectory.toString());
// Create/open the command/response file
    commandResponseFile = Files.newBufferedWriter(testDirectory.resolve(Path.of("CommandResponse.txt")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);
// Create/open the test summary (XML) file
    testSummaryXml = Files.newBufferedWriter(testDirectory.resolve(Path.of("TestSummary.xml")),StandardOpenOption.CREATE_NEW,StandardOpenOption.WRITE);
    testSummaryXml.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n" );
    testSummaryXml.write("<test xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n" );
// Create directory for eye scan data files
    eyeScanDataDirectory = Files.createDirectories(testDirectory.resolve(Path.of("EyeScanData")));
    return "OK|TesterSequence.prepareTests";
  }
  catch (IOException ioe){ throw new RuntimeException("TesterSequence.prepareTests|IOException: "+ioe.getMessage()); }
}

public static String endOfTests(String [] args)
{
  try
  {
    commandResponseFile.close(); commandResponseFile = null;
    testSummaryXml.write("</test>\n" );
    testSummaryXml.close(); testSummaryXml = null;
    return "OK|TesterSequence.endOfTests";
  }
  catch (IOException ioe){ throw new RuntimeException("TesterSequence.endOfTests|IOException: "+ioe.getMessage()); }
}
    
public static String bitflipSummary(String [] args)
{
// Capture and save the bitflip counters

  try
  {
    MuDaq.muDaq.status.cache(MuDaq.muDaq.io);

// Make a single list of all the GBTXes
    List<GbtxControl> gbtxes = new ArrayList<GbtxControl>();
    for (DtmControl dtm : PdmdbMaster.pdmdbMaster.dtmList) gbtxes.addAll(dtm.gbtxList);
      
    testSummaryXml.write("<bitflips>\n");
    for ( GbtxControl gbtx : gbtxes )
    {
      testSummaryXml.write(String.format("<gbtx dtm=\"%d\" gbtx=\"%d\" flips=\"",gbtx.dtm,gbtx.gbtx));
      for (int elink=0;elink<14;elink++)
      {
        int mgt = 2 + 2*gbtx.dtm + gbtx.gbtx;
        String hexCount = MuDaq.muDaq.status.get("gbt."+mgt+".egroup."+elink/2+".elink."+elink%2+".bitflips");
// Convert the hex string to a decimal string
        try
        {
          Integer decoded = Integer.decode(hexCount);
          testSummaryXml.write(decoded.toString());
        }
        catch (NumberFormatException nfe){testSummaryXml.write("ovfl");}
        if (elink<13) testSummaryXml.write(",");
      }
      testSummaryXml.write("\" />\n");
    }
    testSummaryXml.write("</bitflips>\n");
    return "OK|TesterSequence.bitflipSummary";
  }
  catch (Exception ioe){ throw new RuntimeException("TesterSequence.bitflipSummary|IOException: "+ioe.getMessage());}
}

public static String clockSummary(String [] args)
{
  try
  {
    if (args.length < 19) throw new RuntimeException("TesterSequence.clockSummary|Insufficient data in response");
    testSummaryXml.write(String.format("<clocks>\n"));
    for (String token : args)
    {
      String [] clock = token.split("[:,=]");
      if (clock.length < 2) continue;
      if (clock[0].matches("TCM_CLK[012345]") || clock[0].matches("DTM[012]_FPGA_CLK") || clock[0].matches("DTM[012]_GBTX1_CLK_OUT" ))
      {
        testSummaryXml.write(String.format("<clock id=\"%s\" frequency=\"%s\" />\n",clock[0].toLowerCase(),clock[1]));
      }
    }
    testSummaryXml.write(String.format("</clocks>\n"));
    return "OK|TesterSequence.clockSummary";
  }
  catch (IOException ioe){ throw new RuntimeException("TesterSequence.clockSummary|IOException: "+ioe.getMessage()); }
}
    
public static String powerSummary(String [] args)
{
  try
  {
    if (args.length < 7) throw new RuntimeException("TesterSequence.powerSummary|Insufficient data in response");
    for (String token : args)
    {
      String [] module = token.split("[:,=]");
      if (module.length < 5) continue;
      if (module[0].matches("DTM[012]") || module[0].equals("TCM"))
      {
	String timestamp = java.time.LocalDateTime.now().format(java.time.format.DateTimeFormatter.ofPattern("yyyyMMdd-A"));
        testSummaryXml.write(String.format("<power testerModule=\"%s\" i1v5=\"%s\" i2v5=\"%s\" timestamp=\"%s\" />\n",module[0].toLowerCase(),module[2],module[4],timestamp));
      }
    }
    return "OK|TesterSequence.powerSummary";
  }
  catch (IOException ioe){ throw new RuntimeException("TesterSequence.powerSummary|IOException: "+ioe.getMessage()); }
}
    
public static String temperatureSummary(String [] args)
{
  try
  {
    if (args.length < 7) throw new RuntimeException("TesterSequence.temperatureSummary|Insufficient data in response");
    for (String token : args)
    {
      String [] module = token.split("[:=]");
      if (module.length < 2) continue;
      if (module[0].matches("DTM[012]") || module[0].equals("TCM"))
      {
	String timestamp = java.time.LocalDateTime.now().format(java.time.format.DateTimeFormatter.ofPattern("yyyyMMdd-A"));
        testSummaryXml.write(String.format("<temperature testerModule=\"%s\" reading=\"%s\" timestamp=\"%s\" />\n",module[0].toLowerCase(),module[1],timestamp));
      }
    }
    return "OK|TesterSequence.temperatureSummary";
  }
  catch (IOException ioe){ throw new RuntimeException("TesterSequence.temperatureSummary|IOException: "+ioe.getMessage()); }
}
    
public static String goTo(String [] args)
{
  try
  {
    if (args.length < 3) throw new RuntimeException("TesterSequence.goTo|Insufficient data in response");
    if (staticThis.iterations == 0)
    {
      staticThis.it = staticThis.commandList.listIterator(Integer.valueOf(args[0]));
      staticThis.iterations = Integer.valueOf(args[1]);
    }
    else if (staticThis.iterations == 1)
    {
      Thread.sleep(Integer.valueOf(args[2])*1000);
      staticThis.iterations--;
    }
    else
    {
      staticThis.it = staticThis.commandList.listIterator(Integer.valueOf(args[0]));
      Thread.sleep(Integer.valueOf(args[2])*1000);
      staticThis.iterations--;
    }
    return "OK|TesterSequence.goTo";

  }
  catch (Exception ex){ throw new RuntimeException("TesterSequence.goTo|Exception: "+ex.getMessage()); }
}
    
private void prepare()
{
  for (Command cmd : commandList ) cmd.clearStatus();

  it = commandList.listIterator();
}

private void pause(){suspend=true;}

private void resume()
{
  suspend=false;
  TestTask tt = new TestTask();
  tt.execute();
}

private void recording()
{
  recording=!recording;
  if (recording) recordingButton.setBackground(java.awt.Color.RED);
  else recordingButton.setBackground(new java.awt.Color(220,220,220));
}
    
private void step()
{
  if (it==null) it = commandList.listIterator();
    
  try { if (it.hasNext()) it.next().go(); }
  catch (Exception e) {PdmdbModuleTester.readFlush(); textArea.append("TesterSequence.step(): "+e.getMessage()+"\n");}
}
    
public void go()
{
  for (Command cmd : commandList ) cmd.clearStatus();
  
  try { for (Command cmd : commandList ) cmd.go(); }
  catch (Exception e) {PdmdbModuleTester.readFlush(); textArea.append("TesterSequence.go(): "+e.getMessage()+"\n");}
}
	
public void launch()
{
  TestTask tt = new TestTask();
  tt.execute();
}
    
class TestTask extends javax.swing.SwingWorker<Boolean, Integer>
{

  @Override
  protected Boolean doInBackground()
  {
    boolean pass = true;
    try
    {
      while(!suspend)
      {
        for (Command cmd : commandList ) cmd.clearStatus();
	textArea.setText("");
        while(it.hasNext())
        {
          it.next().go();
          if (suspend) break;
        }
        it = commandList.listIterator();
      }
    }
    catch (Exception e) {textArea.append("TesterSequence.TestTask.doInBackground(): "+e.getMessage()+"\n");}
    return Boolean.valueOf(pass);
  }
      
  @Override
  protected void done(){ try {  } catch (Exception e){} }

  @Override
  protected void process( List<Integer> chunks ){}
}
    
}
