package cbsw.lhcb.pdmdb;

import javax.xml.bind.annotation.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.file.Files;
import java.util.*;
import java.util.stream.*;
import java.util.function.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import cbsw.lhcb.pdmdb.*;
import cbsw.adept.*;
import cbsw.adept.JniAdeptDepp.*;
import cbsw.gui.*;
import cbsw.TapBuilder;

/**
<p>This class encapsulates JTAG interactions with a Xilinx Series 7 FPGA including programming.</p>
**/
@XmlAccessorType(XmlAccessType.NONE)
//@XmlType(name="tap",factoryMethod="createXilS7Control")
class XilS7Control implements ActionListener
{
  Preferences prefs;
  ScaDepp sca;
  @XmlAttribute(name="tapLength")
  int tapLength;
  @XmlAttribute(name="tapPosition")
  int tapPosition;
  ProgramThread pth;
  final int txPageSize=0x5; // Units of 16byte frames
  final int txPages=16; // The FPGA buffer is 4096 bytes
  TapBuilder tap;
  JTextField statusTF;
  JCheckBox statusLoopCB;
  JTextField binTF;
    
  public XilS7Control()
  {
    prefs = Preferences.userNodeForPackage(XilS7Control.class).node("JPDMDBDepp");
  }

    //  static public XilS7Control createXilS7Control(){return new XilS7Control();}

  public void setTapLength(int tapLength){this.tapLength=tapLength;}
  public void setTapPosition(int tapPosition){this.tapPosition=tapPosition;}
  public void init(ScaDepp sca){this.sca = sca; tap = new TapBuilder(tapLength,tapPosition);}
    
  public void actionPerformed( ActionEvent e )
  {
    Object ob = e.getSource();
    String command = e.getActionCommand();

    if ( command.equals("status") )
    {
      StatusThread sth = new StatusThread();
      sth.start();
    }
    else if ( command.equals("choose") )
    {
      JFileChooser chooser = new JFileChooser(prefs.get("bin.dir","."));
      FileNameExtensionFilter filter = new FileNameExtensionFilter( "FPGA bin files (BIN)", "bin" );
      chooser.setFileFilter(filter);
      int returnVal = chooser.showOpenDialog(null);
      if(returnVal != JFileChooser.APPROVE_OPTION) return;
      if (!chooser.getSelectedFile().isFile()) return;
      prefs.put("bin.dir",chooser.getSelectedFile().getParent());
      prefs.put("bin.file-name",chooser.getSelectedFile().getAbsolutePath());
      binTF.setText(prefs.get("bin.file-name","pdmdb.bin"));
    }
    else if ( command.equals("load") )
    {
      pth = new ProgramThread();
      pth.start();
    }
  }

  public GuiJPanel makePanel()
  {
    GuiJPanel jp = new GuiJPanel(Preferences.userNodeForPackage(XilS7Control.class).node("JPDMDBDepp"),"");

    jp.setName( "FPGA."+tapPosition );
    jp.setBorder( BorderFactory.createTitledBorder("FPGA TAP["+tapPosition+"]") );

    jp.setLayout(new GridBagLayout());
    GridBagConstraints gbC = new GridBagConstraints();
    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.insets = new Insets(1,1,1,1);
    gbC.weightx = 1.0f;
    gbC.weighty = 1.0f;
    gbC.fill = GridBagConstraints.BOTH;

    JButton button;

    button = new JButton("Load FPGA");
    button.addActionListener(this);
    button.setActionCommand("load");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    button = new JButton("Choose file");
    button.addActionListener(this);
    button.setActionCommand("choose");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    binTF = jp.makeTF("bin.file-name","pdmdb.bin",64,null,"FPGA configuration file [.bin]",false);
    //    binTF = new JTextField(prefs.get("bin.file-name","pdmdb.bin"));
    binTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add( binTF, gbC );
    
    button = new JButton("Status");
    button.addActionListener(this);
    button.setActionCommand("status");
    gbC.gridwidth = 1;
    jp.add(button,gbC);

    statusLoopCB = new JCheckBox("Loop",false);
    gbC.gridwidth = 1;
    jp.add(statusLoopCB,gbC);

    statusTF = new JTextField(10);
    statusTF.setEditable(false);
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    jp.add( statusTF, gbC );
    
    button = new JButton("JSHUTDOWN");
    button.addActionListener(al->jshutdown());
    gbC.gridwidth = 1;
    //    jp.add(button,gbC);

    button = new JButton("JSTART");
    button.addActionListener(al->jstart());
    gbC.gridwidth = 1;
    //    jp.add(button,gbC);

    button = new JButton("TLR");
    button.addActionListener(al->tlr());
    gbC.gridwidth = 1;
    //    jp.add(button,gbC);

    button = new JButton("RTI");
    button.addActionListener(al->rti());
    gbC.gridwidth = 1;
    //    jp.add(button,gbC);

    return jp;
  }
    
// Launch transaction. Max 128 bits for now.

  public void packAndGo()
  {
    tap.show();

    ByteBuffer tdoBB = tap.tdoToByteBuffer(); 
    ByteBuffer tmsBB = tap.tmsToByteBuffer();

    int bits = tap.length();
    if (bits < 1) return;

    sca.allocateTxRxBuffers(3*((bits+31)/32)+2);
    
// Control register. LSB mode. Falling edge.
    sca.infoTx("JTAG_W_CTRL",0x00009000 | (bits&0x7f));

    for (int bit=0;bit<bits;bit+=32)
    {
      sca.infoTx("JTAG_W_TDO"+bit/32,tdoBB.getInt(bit/8));
      sca.infoTx("JTAG_W_TMS"+bit/32,tmsBB.getInt(bit/8));
    }
    
    sca.infoTx("JTAG_GO",0x00000000);

    for (int bit = 0;bit<bits;bit+=32)
    {
      sca.infoTx("JTAG_R_TDI"+bit/32,0x00000000);
    }
    
// Write the bytes to the transaction buffer
    sca.putBytes( 0, true );
    
  }
    
  public byte [] unpackData()
  {
// Now decode the SCA responses to extract the data

    int bits = tap.length();
    if (bits < 1) return null;

    int frames = 3*((bits+31)/32)+2;
    
    byte [] rxbuf = new byte [frames*4];
    byte [] hdlcrx = new byte [frames*16];
    sca.getBytes( 0, hdlcrx );
    
    int irx = 0;
    for ( int frame=0;frame<frames;frame++ )
    {
      for (int j=0;j<4;j++)
      {
        int idx = sca.getRxIndex(frame,j);
        if (idx<0) continue;
	  
        rxbuf[irx++] = hdlcrx[frame*16+idx];

      }
    }

    BitSet maskBS = BitSet.valueOf(tap.tdoMaskToByteBuffer());
    BitSet tdoBS =  BitSet.valueOf(rxbuf);
    tdoBS.and(maskBS);
    IntStream tdoIS = tdoBS.stream();
    BitSet outBS = new BitSet();
    int offset = maskBS.nextSetBit(0);

    tdoIS.forEach(i -> outBS.set(i-offset));

    return outBS.toByteArray();

  }
    
// 00: Temperature
// 01: VCCINT
// 02: VCCAUX
// See ug480 for more
    
  public short xadc(int ch)
  {
    tap.clear();
    tap.rti(0);
    tap.ir("110111",false); // XADC_DRP
    String drpAddr = Integer.toString((ch & 0x3ff) | 0x400,2).substring(1,11);
    tap.dr("000001"+drpAddr+"0000000000000000", true);

    packAndGo();
    packAndGo();

    byte [] xdata = Arrays.copyOfRange(unpackData(),0,2);

    ByteBuffer bb = ByteBuffer.wrap(xdata);
    bb.order(ByteOrder.LITTLE_ENDIAN);
    short adc = bb.getShort(0);
    return (short)((adc>>>4)&0xfff);
  }

// Get raw ADC and convert to degrees C following ug480
    
  public double xadcTemperature()
  {
    short temp = xadc(0);

    return temp*503.975/4096.0 - 273.15;
  }

// Get raw ADC and convert to volts following ug480
    
  public double xadcVoltage(int ch)
  {
    short volt = xadc(ch);

    return volt*3.0/4096.0;
  }

  public int idcode()
  {
    tap.clear();
    tap.rti(0);
    tap.ir("001001",false); // IDCODE
    tap.dr("00000000000000000000000000000000", true);

    packAndGo();

    byte [] data = Arrays.copyOfRange(unpackData(),0,4);

    ByteBuffer bb = ByteBuffer.wrap(data);
    bb.order(ByteOrder.LITTLE_ENDIAN);
    return bb.getInt();
  }

// Read the device DNA
// XSC_DNA (0x17) requires that ISC_ENABLE (0x10) is loaded first.
// Follow with ISC_DISABLE (0x16)
// A few clocks in RTI at the beginning are used to byte-align the DNA
    
  public String dna()
  {
    boolean isc_mode=false;
    tap.clear();
    tap.rti(0);
    if (isc_mode)
    {
      tap.ir("010000",true); // ISC_ENABLE
      tap.ir("010111",false); // XSC_DNA
      tap.dr("000000000000000000000000000000000000000000000000000000000", true);
      tap.ir("010110",true); // ISC_DISABLE
    }
    else
    {
      tap.ir("110010",false); // FUSE_DNA
      tap.dr("0000000000000000000000000000000000000000000000000000000000000000", true);
    }

    packAndGo();

    byte [] idbuf = unpackData();

    if (idbuf == null) return "?";

    StringBuffer idSB = new StringBuffer();
    for (int i=idbuf.length-1;i>=0;i--) idSB.append(String.format("%1$02x",idbuf[i]));
    
    return idSB.toString();

  }
    
  public void jstart()
  {

    tap.clear();
    tap.rti(0);
    tap.ir("001100",true); // JSTART

    packAndGo();

// 2000 startup clocks
    
    for (int i=0;i<16;i++)
    {
      sca.allocateTxRxBuffers(6);
      sca.infoTx("JTAG_W_CTRL",0x00009000);
// TMS: RTI for at least 2000 clocks
      sca.infoTx("JTAG_W_TMS0",0x00000000);
      sca.infoTx("JTAG_W_TMS1",0x00000000);
      sca.infoTx("JTAG_W_TMS2",0x00000000);
      sca.infoTx("JTAG_W_TMS3",0x00000000);
// Do it
      sca.infoTx("JTAG_GO",0x00000000);
// Write the bytes to the transaction buffer
      sca.putBytes( 0, true );
    }
    
    System.out.println("jstart(): Done.");
  }
    
// Move to the TLR state (from anywhere)
// TMS: 5 or more 1's take us to TLR

  public void tlr()
  {
    sca.allocateTxRxBuffers(3);
    
// Control register. LSB mode. Falling edge.
    sca.infoTx("JTAG_W_CTRL",0x00009020);
// TMS 11111111111111111111111111111111>
    sca.infoTx("JTAG_W_TMS0",0xffffffff);
// Do it
    sca.infoTx("JTAG_GO",0x00000000);

// Write the bytes to the transaction buffer

    sca.putBytes( 0, true );
    
    System.out.println("tlr(): Done.");
    
  }
        

// Move to the RTI state (from anywhere)
// TMS: 5 or more 1's take us to TLR then 0 to go to RTI

  public void rti()
  {


    sca.allocateTxRxBuffers(3);
    
// Control register. LSB mode. Falling edge.
    sca.infoTx("JTAG_W_CTRL",0x00009020);
// TMS 01111111111111111111111111111111>
    sca.infoTx("JTAG_W_TMS0",0x7fffffff);
// Do it
    sca.infoTx("JTAG_GO",0x00000000);

// Write the bytes to the transaction buffer

    sca.putBytes( 0, true );
    
    System.out.println("rti(): Done.");
    
  }
        


// JSHUTDOWN followed by >12 clocks in RTI
    
  public void jshutdown()
  {

    tap.clear();
    tap.rti(0);
    tap.ir("001101",true); // JSHUTDOWN
    tap.clock(16);

    packAndGo();

    System.out.println("jshutdown(): Done.");
    
  }
        
// Program the FPGA.
// The CFG_IN data are shifted MSB first while instructions are LSB first so this uses a mixture of SCA-JTAG MSB and LSB modes.
// The default ByteOrder.BIG_ENDIAN for the ByteBuffer is used so that getInt() retrieves each 32-bit word in the correct byte order for infoTx() with the SCA JTAG MSB mode. 
    
  public void program()
  {

    File f = new File(prefs.get("bin.file-name","pdmdb.bin"));
    InputStream fi = null;
    try { fi = Files.newInputStream(f.toPath()); } catch (IOException ioe){;}

    long startNano = System.nanoTime();

    jshutdown(); // JSHUTDOWN instruction

// Move the TAP to the CFG_IN DR state. Right hand bit shifted first.

    tap.clear();
    tap.ir("000101",false);
    tap.dr(null,false); // Should be in SHIFT-DR after this

    packAndGo();
    
    sca.allocateTxRxBuffers(5);
// Prepare TMS for next step
    sca.infoTx("JTAG_W_TMS0",0x00000000);
    sca.infoTx("JTAG_W_TMS1",0x00000000);
    sca.infoTx("JTAG_W_TMS2",0x00000000);
    sca.infoTx("JTAG_W_TMS3",0x00000000);

// CTRL is the same for all blocks (except perhaps the last) so keep it outside the loop. CFG_IN data is bitswapped within bytes so use MSB mode.
    sca.infoTx("JTAG_W_CTRL",0x00009800);

// Write the bytes to the transaction buffer
    sca.putBytes( 0, true );
    
// Shift pages of 128bits each.

    int page = 0; // A page is a group of commands sent as one transaction

// The buffer is extended by one for the extra TMS in the last page
    sca.allocateTxRxBuffers(txPageSize*txPages+1);

    byte [] buf = new byte [16];
    ByteBuffer rbb = ByteBuffer.wrap(buf);

    int rd = 0;
    try
    {
    do
    {
      rd = fi.read(buf);

      for (int j=0;j<(buf.length/4);j++)
      {
	int tdo = (3-j)%4; // Reverse TDO/TMS index in MSB mode
	if (j<(rd/4))
	{
	  sca.infoTx("JTAG_W_TDO"+tdo,rbb.getInt(j*4));
	}
	// else if (j==rd/4)
	// {
	//   sca.infoTx("JTAG_W_TMS"+(tdo+1),0x00000001);
	//   sca.infoTx("JTAG_W_TMS"+tdo,0x80000000);
	// }
	// else
	// {
	//   sca.infoTx("JTAG_W_TMS"+tdo,0x00000000);
	// }
      }
      
// Do it
      sca.infoTx("JTAG_GO",0x00000000);
      page++;


// Trigger the transaction every txPages pages
      if (page%txPages == 0)
      {
// Write the bytes to the transaction buffer
	sca.putBytes( 0, true );
	sca.resetBuffers();

// Show progress every now and then
        if (page%(500*txPages)==0) System.out.println("page="+page);
      }
    } while (rd == 16);
    } catch (IOException ioe){;}

// After shifting in config data we go back to RTI
// However we first stay in SHIFT-DR for for 126 clocks which allows for any bypassed devices in the chain. Don't expect to be more than 126 of these. 

    sca.allocateTxRxBuffers(2);
    sca.infoTx("JTAG_W_TMS0",0x00000006);
    sca.infoTx("JTAG_GO",0x00000000);
    sca.putBytes( 0, true );

// Now do the startup procedure
    jstart();
    tlr();
    
    long elapsedNano = System.nanoTime() - startNano;

    try {fi.close();} catch (IOException ioe){}
    
    System.out.println("program(): Done. Elapsed time="+elapsedNano+"ns");
  }
    
  public class ProgramThread extends Thread
  {
    public void run()
    {
      program();
    }
  }

  public void showStatus()
  {
    int id = idcode();
    double temp = xadcTemperature();
    double vccint = xadcVoltage(1);
    double vccaux = xadcVoltage(2);
    String dnaString = dna();
    statusTF.setText(String.format("DNA: %s, IDCODE: %08x, Temp[C]=%3.1f, VCCint[V]=%4.2f, VCCaux[V]=%4.2f",dnaString,id,temp,vccint,vccaux));
    return;
  }
    
  public class StatusThread extends Thread
  {
    public void run()
    {
      if (!statusLoopCB.isSelected()) { showStatus(); return; }
      try { do { showStatus(); Thread.sleep(500); } while (statusLoopCB.isSelected());}
      catch (InterruptedException se){};
    }
  }

}
